// Koka generated module: std/text/regex, koka version: 3.1.3, platform: 64-bit
#include "std_text_regex.h"
/*---------------------------------------------------------------------------
  Copyright 2021, Microsoft Research, Daan Leijen.

  This is free software; you can redistribute it and/or modify it under the
  terms of the Apache License, Version 2.0. A copy of the License can be
  found in the LICENSE file at the root of this distribution.
---------------------------------------------------------------------------*/

#pragma clang diagnostic ignored "-Wdisabled-macro-expansion"
#define PCRE2_CODE_UNIT_WIDTH 8
#define PCRE2_STATIC
#include <pcre2.h>

/* -----------------------------------------------------------------------
  Init/Done
------------------------------------------------------------------------*/
#define KK_CUSTOM_INIT  kk_regex_custom_init
#define KK_CUSTOM_DONE  kk_regex_custom_done

static pcre2_general_context* gen_ctx;
static pcre2_compile_context* cmp_ctx;
static pcre2_match_context*   match_ctx;

static void* kk_pcre2_malloc( PCRE2_SIZE size, void* data ) {
  return kk_malloc( kk_to_ssize_t(size), kk_get_context() );
}
static void kk_pcre2_free( void* p, void* data ) {
  kk_free(p,kk_get_context());
}

static void kk_regex_custom_init( kk_context_t* ctx ) {
  gen_ctx = pcre2_general_context_create( &kk_pcre2_malloc, &kk_pcre2_free, NULL );
  if (gen_ctx != NULL) {
    match_ctx = pcre2_match_context_create( gen_ctx );
    cmp_ctx = pcre2_compile_context_create( gen_ctx );
    if (cmp_ctx != NULL) {
      pcre2_set_newline( cmp_ctx, PCRE2_NEWLINE_ANYCRLF );
      pcre2_set_bsr( cmp_ctx, PCRE2_BSR_ANYCRLF );
    }
  }
}

static void kk_regex_custom_done( kk_context_t* ctx ) {
  if (cmp_ctx != NULL) {
    pcre2_compile_context_free(cmp_ctx);
    cmp_ctx = NULL;
  }
  if (match_ctx != NULL) {
    pcre2_match_context_free(match_ctx);
    match_ctx = NULL;
  }
  if (gen_ctx != NULL) {
    pcre2_general_context_free(gen_ctx);
    gen_ctx = NULL;
  }
}


/* -----------------------------------------------------------------------
  Compile
------------------------------------------------------------------------*/

static void kk_regex_free( void* pre, kk_block_t* b, kk_context_t* ctx ) {
  kk_unused(ctx);
  pcre2_code* re = (pcre2_code*)pre;
  //kk_info_message( "free regex at %p\n", re );
  if (re != NULL) { pcre2_code_free(re); }
}

#define KK_REGEX_OPTIONS  (PCRE2_ALT_BSUX | PCRE2_EXTRA_ALT_BSUX | PCRE2_MATCH_UNSET_BACKREF /* javascript compat */ \
                          | PCRE2_NEVER_BACKSLASH_C | PCRE2_NEVER_UCP | PCRE2_UTF /* utf-8 safety */ \
                          )

static kk_box_t kk_regex_create( kk_string_t pat, bool ignore_case, bool multi_line, kk_context_t* ctx ) {
  kk_ssize_t len;
  const uint8_t* cpat = kk_string_buf_borrow( pat, &len, ctx );
  PCRE2_SIZE errofs = 0;
  int        errnum = 0;
  uint32_t   options = KK_REGEX_OPTIONS;
  if (ignore_case) options |= PCRE2_CASELESS;
  if (multi_line)  options |= PCRE2_MULTILINE;
  pcre2_code* re = pcre2_compile( cpat, PCRE2_ZERO_TERMINATED, options, &errnum, &errofs, cmp_ctx);
  //kk_info_message( "create regex: err:%i, at %p\n", (re==NULL ? 0 : errnum), re );
  kk_string_drop(pat,ctx);
  return kk_cptr_raw_box( &kk_regex_free, re, ctx );
}


/* -----------------------------------------------------------------------
  Match
------------------------------------------------------------------------*/

/*
static void kk_match_data_free( void* pmd, kk_block_t* b ) {
  kk_info_message( "free match_data\n" );
  pcre2_match_data* md = (pcre2_match_data*)pmd;
  if (md != NULL) pcre2_match_data_free(md);
}
*/

static kk_std_core_types__list kk_regex_exec_ex( pcre2_code* re, pcre2_match_data* match_data,
                                           kk_string_t str_borrow, const uint8_t* cstr, kk_ssize_t len, bool allow_empty,
                                           kk_ssize_t start, kk_ssize_t* mstart, kk_ssize_t* end, int* res, kk_context_t* ctx )
{
  // match
  kk_std_core_types__list hd  = kk_std_core_types__new_Nil(ctx);
  uint32_t options = 0;
  if (!allow_empty) options |= (PCRE2_NOTEMPTY_ATSTART | PCRE2_ANCHORED);
  int rc = pcre2_match( re, cstr, len, start, options, match_data, match_ctx );
  if (res != NULL) *res = rc;
  if (rc > 0) {
    // extract captures
    uint32_t    gcount = pcre2_get_ovector_count(match_data);
    PCRE2_SIZE* groups = pcre2_get_ovector_pointer(match_data);
    for( uint32_t i = gcount; i > 0; ) {
      i--;
      kk_ssize_t sstart = groups[i*2];       // on no-match, sstart and send == -1.
      kk_ssize_t send   = groups[i*2 + 1];
      kk_assert(send >= sstart);
      kk_std_core_sslice__sslice sslice = kk_std_core_sslice__new_Sslice( kk_string_dup(str_borrow,ctx), kk_integer_from_ssize_t(sstart,ctx), kk_integer_from_ssize_t(send - sstart,ctx), ctx );
      hd = kk_std_core_types__new_Cons(kk_reuse_null,0,kk_std_core_sslice__sslice_box(sslice,ctx), hd, ctx);
      if (i == 0) {
        if (mstart != NULL) { *mstart = sstart; }
        if (end    != NULL) { *end = send; }
      }
    }
  }
  return hd;
}


static kk_std_core_types__list kk_regex_exec( kk_box_t bre, kk_string_t str, kk_ssize_t start, kk_context_t* ctx )
{
  // unpack
  pcre2_match_data* match_data = NULL;
  kk_std_core_types__list res = kk_std_core_types__new_Nil(ctx);
  pcre2_code* re = (pcre2_code*)kk_cptr_raw_unbox_borrowed(bre,ctx);
  kk_ssize_t len = 0;
  const uint8_t* cstr = NULL;
  if (re == NULL) goto done;
  match_data = pcre2_match_data_create_from_pattern(re, gen_ctx);
  if (match_data==NULL) goto done;
  cstr = kk_string_buf_borrow(str, &len, ctx );

  // and match
  res = kk_regex_exec_ex( re, match_data, str, cstr, len, true, start, NULL, NULL, NULL, ctx );

done:
  if (match_data != NULL) {
    //kk_info_message( "free match data: %p\n", match_data );
    pcre2_match_data_free(match_data);
  }
  kk_string_drop(str,ctx);
  kk_box_drop(bre,ctx);
  return res;
}

static kk_std_core_types__list kk_regex_exec_all( kk_box_t bre, kk_string_t str, kk_ssize_t start, kk_ssize_t atmost, kk_context_t* ctx ) {
  // unpack
  if (atmost < 0) atmost = KK_SSIZE_MAX;
  pcre2_match_data* match_data = NULL;
  kk_std_core_types__list res = kk_std_core_types__new_Nil(ctx);
  pcre2_code* re = (pcre2_code*)kk_cptr_raw_unbox_borrowed(bre,ctx);
  if (re == NULL) goto done;
  match_data = pcre2_match_data_create_from_pattern(re, gen_ctx);
  if (match_data==NULL) goto done;
  {
    kk_ssize_t len;
    const uint8_t* cstr = kk_string_buf_borrow(str, &len, ctx );

    // and match
    kk_std_core_types__list* tail = NULL;
    bool allow_empty = true;
    int rc = 1;
    kk_ssize_t next = start;
    while( rc > 0 && start < len && atmost > 0) {
      atmost--;
      rc = 0;
      kk_ssize_t mstart = start;
      kk_std_core_types__list cap = kk_regex_exec_ex( re, match_data, str, cstr, len, allow_empty, start, &mstart, &next, &rc, ctx );
      if (rc > 0) {
        // found a match;
        // push string up to match, and the actual matched regex
        kk_std_core_sslice__sslice pre = kk_std_core_sslice__new_Sslice( kk_string_dup(str,ctx), kk_integer_from_ssize_t(start,ctx), kk_integer_from_ssize_t(mstart - start,ctx), ctx );
        kk_std_core_types__list   prelist = kk_std_core_types__new_Cons( kk_reuse_null, 0, kk_std_core_sslice__sslice_box(pre,ctx), kk_std_core_types__new_Nil(ctx), ctx );
        kk_std_core_types__list   capcons = kk_std_core_types__new_Cons( kk_reuse_null, 0, kk_std_core_types__list_box(cap,ctx), kk_std_core_types__new_Nil(ctx) /*tail*/, ctx );
        kk_std_core_types__list   cons = kk_std_core_types__new_Cons( kk_reuse_null, 0, kk_std_core_types__list_box(prelist,ctx), capcons, ctx );
        if (tail==NULL) res = cons;
                  else *tail = cons;
        tail = &kk_std_core_types__as_Cons(capcons,ctx)->tail;
        allow_empty = (next > start);
        start = next;
      }
      else if (rc <= 0 && !allow_empty) {
        // skip one character and try again
        // todo: handle cr/lf pairs better?
        const uint8_t* p = kk_utf8_next( cstr + start );
        start = (p - cstr);
        allow_empty = true;
        rc = 1;
      }
    }

    // push final string part as well and end the list
    kk_std_core_sslice__sslice post    = kk_std_core_sslice__new_Sslice( kk_string_dup(str,ctx), kk_integer_from_ssize_t(next,ctx), kk_integer_from_ssize_t(len - next,ctx), ctx );
    kk_std_core_types__list   postlist= kk_std_core_types__new_Cons( kk_reuse_null, 0, kk_std_core_sslice__sslice_box(post,ctx), kk_std_core_types__new_Nil(ctx), ctx );
    kk_std_core_types__list   cons    = kk_std_core_types__new_Cons( kk_reuse_null, 0, kk_std_core_types__list_box(postlist,ctx), kk_std_core_types__new_Nil(ctx), ctx );
    if (tail==NULL) res = cons;
              else *tail = cons;
  }

done:
  if (match_data != NULL) {
    //kk_info_message( "free match data: %p\n", match_data );
    pcre2_match_data_free(match_data);
  }
  kk_string_drop(str,ctx);
  kk_box_drop(bre,ctx);
  return res;
}

/*
kk_std_core_sslice__sslice kk_slice_upto( struct kk_std_core_Sslice slice1, struct kk_std_core_Sslice slice2, kk_context_t* ctx ) {
  kk_ssize_t start = slice1.start;
  kk_ssize_t len   = (slice1.start <= slice2.start ? slice2.start - slice1.start : -1);
  return kk_std_core_sslice__new_Sslice(slice1.str, start, len, ctx);
}
*/

kk_std_text_regex__regex kk_std_text_regex_regex_fs__copy(kk_std_text_regex__regex _this, kk_std_core_types__optional obj, kk_std_core_types__optional src, kk_context_t* _ctx) { /* (regex, obj : ? any, src : ? string) -> regex */ 
  kk_box_t _x_x179;
  if (kk_std_core_types__is_Optional(obj, _ctx)) {
    kk_box_t _box_x0 = obj._cons._Optional.value;
    kk_box_t _uniq_obj_107 = kk_box_unbox(_box_x0, _ctx);
    kk_box_dup(_uniq_obj_107, _ctx);
    kk_std_core_types__optional_drop(obj, _ctx);
    _x_x179 = _uniq_obj_107; /*any*/
  }
  else {
    kk_std_core_types__optional_drop(obj, _ctx);
    {
      kk_box_t _x = _this.obj;
      kk_box_dup(_x, _ctx);
      _x_x179 = _x; /*any*/
    }
  }
  kk_string_t _x_x180;
  if (kk_std_core_types__is_Optional(src, _ctx)) {
    kk_box_t _box_x1 = src._cons._Optional.value;
    kk_string_t _uniq_src_114 = kk_string_unbox(_box_x1);
    kk_string_dup(_uniq_src_114, _ctx);
    kk_std_core_types__optional_drop(src, _ctx);
    kk_std_text_regex__regex_drop(_this, _ctx);
    _x_x180 = _uniq_src_114; /*string*/
  }
  else {
    kk_std_core_types__optional_drop(src, _ctx);
    {
      kk_string_t _x_0 = _this.src;
      kk_string_dup(_x_0, _ctx);
      kk_std_text_regex__regex_drop(_this, _ctx);
      _x_x180 = _x_0; /*string*/
    }
  }
  return kk_std_text_regex__new_Regex(_x_x179, _x_x180, _ctx);
}

kk_box_t kk_std_text_regex_regex_create(kk_string_t source_0, bool ignore_case, bool multi_line, kk_context_t* _ctx) { /* (source : string, ignore-case : bool, multi-line : bool) -> any */ 
  return kk_regex_create(source_0,ignore_case,multi_line,kk_context());
}

kk_std_core_types__list kk_std_text_regex_regex_exec(kk_box_t regex_0, kk_string_t str, kk_ssize_t start, kk_context_t* _ctx) { /* (regex : any, str : string, start : ssize_t) -> list<sslice/sslice> */ 
  return kk_regex_exec(regex_0,str,start,kk_context());
}

kk_std_core_types__list kk_std_text_regex_regex_exec_all(kk_box_t regex_0, kk_string_t str, kk_ssize_t start, kk_ssize_t atmost, kk_context_t* _ctx) { /* (regex : any, str : string, start : ssize_t, atmost : ssize_t) -> list<list<sslice/sslice>> */ 
  return kk_regex_exec_all(regex_0,str,start,atmost,kk_context());
}
 
// Create a new regular expression. Takes two optional parameters. Set `ignoreCase` to `True`
// to ignore uppercase/lowercase distinction. If  `multiline` is set to `True`, then `^` and `$`
// match also the beginning and end of every line (instead of the entire input).

kk_std_text_regex__regex kk_std_text_regex_regex(kk_string_t regex_0, kk_std_core_types__optional ignorecase, kk_std_core_types__optional multiline, kk_context_t* _ctx) { /* (regex : string, ignorecase : ? bool, multiline : ? bool) -> regex */ 
  kk_box_t _x_x181;
  kk_string_t _x_x182 = kk_string_dup(regex_0, _ctx); /*string*/
  bool _x_x183;
  if (kk_std_core_types__is_Optional(ignorecase, _ctx)) {
    kk_box_t _box_x2 = ignorecase._cons._Optional.value;
    bool _uniq_ignorecase_161 = kk_bool_unbox(_box_x2);
    kk_std_core_types__optional_drop(ignorecase, _ctx);
    _x_x183 = _uniq_ignorecase_161; /*bool*/
  }
  else {
    kk_std_core_types__optional_drop(ignorecase, _ctx);
    _x_x183 = false; /*bool*/
  }
  bool _x_x184;
  if (kk_std_core_types__is_Optional(multiline, _ctx)) {
    kk_box_t _box_x3 = multiline._cons._Optional.value;
    bool _uniq_multiline_165 = kk_bool_unbox(_box_x3);
    kk_std_core_types__optional_drop(multiline, _ctx);
    _x_x184 = _uniq_multiline_165; /*bool*/
  }
  else {
    kk_std_core_types__optional_drop(multiline, _ctx);
    _x_x184 = false; /*bool*/
  }
  _x_x181 = kk_std_text_regex_regex_create(_x_x182, _x_x183, _x_x184, _ctx); /*any*/
  return kk_std_text_regex__new_Regex(_x_x181, regex_0, _ctx);
}

kk_std_text_regex__regex kk_std_text_regex_rx_nongroup;
 
// Match a regular expression `regex` over a string `s`.
// Matches at most `atmost` times (and matches all by default).
// Returns always an odd number of elements where every even
// element is a match and the odd ones the string parts between the
// matches.
// See also `find-all` and `strings`.

kk_std_core_types__list kk_std_text_regex_exec_all(kk_std_text_regex__regex regex_0, kk_string_t s, kk_std_core_types__optional atmost, kk_context_t* _ctx) { /* (regex : regex, s : string, atmost : ? int) -> list<list<sslice/sslice>> */ 
  kk_box_t _x_x193;
  {
    kk_box_t _x = regex_0.obj;
    kk_box_dup(_x, _ctx);
    kk_std_text_regex__regex_drop(regex_0, _ctx);
    _x_x193 = _x; /*any*/
  }
  kk_ssize_t _x_x194;
  kk_integer_t _x_x195;
  if (kk_std_core_types__is_Optional(atmost, _ctx)) {
    kk_box_t _box_x6 = atmost._cons._Optional.value;
    kk_integer_t _uniq_atmost_207 = kk_integer_unbox(_box_x6, _ctx);
    kk_integer_dup(_uniq_atmost_207, _ctx);
    kk_std_core_types__optional_drop(atmost, _ctx);
    _x_x195 = _uniq_atmost_207; /*int*/
  }
  else {
    kk_std_core_types__optional_drop(atmost, _ctx);
    _x_x195 = kk_integer_from_small(-1); /*int*/
  }
  _x_x194 = kk_std_core_int_ssize__t(_x_x195, _ctx); /*ssize_t*/
  return kk_std_text_regex_regex_exec_all(_x_x193, s, (KK_IZ(0)), _x_x194, _ctx);
}
 
// Return the full matched string of a capture group

kk_string_t kk_std_text_regex_captured(kk_std_core_types__list matched_0, kk_context_t* _ctx) { /* (matched : list<sslice/sslice>) -> string */ 
  if (kk_std_core_types__is_Cons(matched_0, _ctx)) {
    struct kk_std_core_types_Cons* _con_x196 = kk_std_core_types__as_Cons(matched_0, _ctx);
    kk_box_t _box_x7 = _con_x196->head;
    kk_std_core_sslice__sslice s = kk_std_core_sslice__sslice_unbox(_box_x7, KK_BORROWED, _ctx);
    kk_std_core_types__list _pat_3 = _con_x196->tail;
    if kk_likely(kk_datatype_ptr_is_unique(matched_0, _ctx)) {
      kk_std_core_types__list_drop(_pat_3, _ctx);
      kk_std_core_sslice__sslice_dup(s, _ctx);
      kk_box_drop(_box_x7, _ctx);
      kk_datatype_ptr_free(matched_0, _ctx);
    }
    else {
      kk_std_core_sslice__sslice_dup(s, _ctx);
      kk_datatype_ptr_decref(matched_0, _ctx);
    }
    return kk_std_core_sslice_string(s, _ctx);
  }
  {
    return kk_string_empty();
  }
}
extern kk_box_t kk_std_text_regex_captures_fun198(kk_function_t _fself, kk_box_t _b_x10, kk_context_t* _ctx) {
  kk_function_static_drop(_fself,kk_context());
  kk_string_t _x_x199;
  kk_std_core_types__list _x_x200 = kk_std_core_types__list_unbox(_b_x10, KK_OWNED, _ctx); /*list<sslice/sslice>*/
  _x_x199 = kk_std_text_regex_captured(_x_x200, _ctx); /*string*/
  return kk_string_box(_x_x199);
}
 
// Find a match for a regular expression.
// See also `exec`

kk_std_core_types__maybe kk_std_text_regex_find(kk_string_t s, kk_std_text_regex__regex r, kk_context_t* _ctx) { /* (s : string, r : regex) -> maybe<string> */ 
  kk_std_core_types__list _match_x155;
  kk_box_t _x_x201;
  {
    kk_box_t _x = r.obj;
    kk_box_dup(_x, _ctx);
    kk_std_text_regex__regex_drop(r, _ctx);
    _x_x201 = _x; /*any*/
  }
  _match_x155 = kk_std_text_regex_regex_exec(_x_x201, s, (KK_IZ(0)), _ctx); /*list<sslice/sslice>*/
  if (kk_std_core_types__is_Cons(_match_x155, _ctx)) {
    struct kk_std_core_types_Cons* _con_x202 = kk_std_core_types__as_Cons(_match_x155, _ctx);
    kk_box_t _box_x13 = _con_x202->head;
    kk_std_core_sslice__sslice m = kk_std_core_sslice__sslice_unbox(_box_x13, KK_BORROWED, _ctx);
    kk_std_core_types__list _pat_3 = _con_x202->tail;
    if kk_likely(kk_datatype_ptr_is_unique(_match_x155, _ctx)) {
      kk_std_core_types__list_drop(_pat_3, _ctx);
      kk_std_core_sslice__sslice_dup(m, _ctx);
      kk_box_drop(_box_x13, _ctx);
      kk_datatype_ptr_free(_match_x155, _ctx);
    }
    else {
      kk_std_core_sslice__sslice_dup(m, _ctx);
      kk_datatype_ptr_decref(_match_x155, _ctx);
    }
    kk_string_t _b_x14_15 = kk_std_core_sslice_string(m, _ctx); /*string*/;
    return kk_std_core_types__new_Just(kk_string_box(_b_x14_15), _ctx);
  }
  {
    return kk_std_core_types__new_Nothing(_ctx);
  }
}
 
// Does a regular expression pattern occur in a string `s`?
// (note: called `test` in javascript)

bool kk_std_text_regex_contains(kk_string_t s, kk_std_text_regex__regex r, kk_context_t* _ctx) { /* (s : string, r : regex) -> bool */ 
  kk_std_core_types__list list_10011;
  kk_box_t _x_x203;
  {
    kk_box_t _x = r.obj;
    kk_box_dup(_x, _ctx);
    kk_std_text_regex__regex_drop(r, _ctx);
    _x_x203 = _x; /*any*/
  }
  list_10011 = kk_std_text_regex_regex_exec(_x_x203, s, (KK_IZ(0)), _ctx); /*list<sslice/sslice>*/
  if (kk_std_core_types__is_Cons(list_10011, _ctx)) {
    struct kk_std_core_types_Cons* _con_x204 = kk_std_core_types__as_Cons(list_10011, _ctx);
    kk_box_t _box_x16 = _con_x204->head;
    kk_std_core_types__list _pat_1 = _con_x204->tail;
    if kk_likely(kk_datatype_ptr_is_unique(list_10011, _ctx)) {
      kk_std_core_types__list_drop(_pat_1, _ctx);
      kk_box_drop(_box_x16, _ctx);
      kk_datatype_ptr_free(list_10011, _ctx);
    }
    else {
      kk_datatype_ptr_decref(list_10011, _ctx);
    }
    return true;
  }
  {
    return false;
  }
}
 
// Filter only for the matched parts.

kk_std_core_types__list kk_std_text_regex__trmc_filter_matches(kk_std_core_types__list xs, kk_std_core_types__cctx _acc, kk_context_t* _ctx) { /* (xs : list<list<sslice/sslice>>, ctx<list<list<sslice/sslice>>>) -> list<list<sslice/sslice>> */ 
  kk__tailcall: ;
  if (kk_std_core_types__is_Cons(xs, _ctx)) {
    struct kk_std_core_types_Cons* _con_x205 = kk_std_core_types__as_Cons(xs, _ctx);
    kk_box_t _box_x17 = _con_x205->head;
    kk_std_core_types__list _pat_1 = _con_x205->tail;
    if (kk_std_core_types__is_Cons(_pat_1, _ctx)) {
      struct kk_std_core_types_Cons* _con_x206 = kk_std_core_types__as_Cons(_pat_1, _ctx);
      kk_box_t _box_x18 = _con_x206->head;
      kk_std_core_types__list _pat_0 = kk_std_core_types__list_unbox(_box_x17, KK_BORROWED, _ctx);
      kk_std_core_types__list mm = _con_x206->tail;
      kk_std_core_types__list m = kk_std_core_types__list_unbox(_box_x18, KK_BORROWED, _ctx);
      kk_reuse_t _ru_x164 = kk_reuse_null; /*@reuse*/;
      if kk_likely(kk_datatype_ptr_is_unique(xs, _ctx)) {
        if kk_likely(kk_datatype_ptr_is_unique(_pat_1, _ctx)) {
          kk_datatype_ptr_free(_pat_1, _ctx);
        }
        else {
          kk_std_core_types__list_dup(m, _ctx);
          kk_std_core_types__list_dup(mm, _ctx);
          kk_datatype_ptr_decref(_pat_1, _ctx);
        }
        kk_std_core_types__list_drop(_pat_0, _ctx);
        _ru_x164 = (kk_datatype_ptr_reuse(xs, _ctx));
      }
      else {
        kk_std_core_types__list_dup(m, _ctx);
        kk_std_core_types__list_dup(mm, _ctx);
        kk_datatype_ptr_decref(xs, _ctx);
      }
      kk_std_core_types__list _trmc_x10055 = kk_datatype_null(); /*list<list<sslice/sslice>>*/;
      kk_std_core_types__list _trmc_x10056 = kk_std_core_types__new_Cons(_ru_x164, 0, kk_std_core_types__list_box(m, _ctx), _trmc_x10055, _ctx); /*list<list<sslice/sslice>>*/;
      kk_field_addr_t _b_x28_33 = kk_field_addr_create(&kk_std_core_types__as_Cons(_trmc_x10056, _ctx)->tail, _ctx); /*@field-addr<list<list<sslice/sslice>>>*/;
      { // tailcall
        kk_std_core_types__cctx _x_x207 = kk_cctx_extend_linear(_acc,(kk_std_core_types__list_box(_trmc_x10056, _ctx)),_b_x28_33,kk_context()); /*ctx<0>*/
        xs = mm;
        _acc = _x_x207;
        goto kk__tailcall;
      }
    }
  }
  {
    kk_std_core_types__list_drop(xs, _ctx);
    kk_box_t _x_x208 = kk_cctx_apply_linear(_acc,(kk_std_core_types__list_box(kk_std_core_types__new_Nil(_ctx), _ctx)),kk_context()); /*-1*/
    return kk_std_core_types__list_unbox(_x_x208, KK_OWNED, _ctx);
  }
}
 
// Filter only for the matched parts.

kk_std_core_types__list kk_std_text_regex_filter_matches(kk_std_core_types__list xs_0, kk_context_t* _ctx) { /* (xs : list<list<sslice/sslice>>) -> list<list<sslice/sslice>> */ 
  kk_std_core_types__cctx _x_x209 = kk_cctx_empty(kk_context()); /*ctx<0>*/
  return kk_std_text_regex__trmc_filter_matches(xs_0, _x_x209, _ctx);
}
 
// Find all matches for a regular expression in a string.


// lift anonymous function
struct kk_std_text_regex_find_all_fun214__t {
  struct kk_function_s _base;
};
static kk_box_t kk_std_text_regex_find_all_fun214(kk_function_t _fself, kk_box_t _b_x42, kk_context_t* _ctx);
static kk_function_t kk_std_text_regex_new_find_all_fun214(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_text_regex_find_all_fun214, _ctx)
  return kk_function_static_dup(_fself,kk_context());
}

static kk_box_t kk_std_text_regex_find_all_fun214(kk_function_t _fself, kk_box_t _b_x42, kk_context_t* _ctx) {
  kk_function_static_drop(_fself,kk_context());
  kk_string_t _x_x215;
  kk_std_core_types__list _x_x216 = kk_std_core_types__list_unbox(_b_x42, KK_OWNED, _ctx); /*list<sslice/sslice>*/
  _x_x215 = kk_std_text_regex_captured(_x_x216, _ctx); /*string*/
  return kk_string_box(_x_x215);
}

kk_std_core_types__list kk_std_text_regex_find_all(kk_string_t s, kk_std_text_regex__regex r, kk_std_core_types__optional atmost, kk_context_t* _ctx) { /* (s : string, r : regex, atmost : ? int) -> list<string> */ 
  kk_std_core_types__list xs_10070;
  kk_box_t _x_x210;
  {
    kk_box_t _x = r.obj;
    kk_box_dup(_x, _ctx);
    kk_std_text_regex__regex_drop(r, _ctx);
    _x_x210 = _x; /*any*/
  }
  kk_ssize_t _x_x211;
  kk_integer_t _x_x212;
  if (kk_std_core_types__is_Optional(atmost, _ctx)) {
    kk_box_t _box_x39 = atmost._cons._Optional.value;
    kk_integer_t _uniq_atmost_355 = kk_integer_unbox(_box_x39, _ctx);
    kk_integer_dup(_uniq_atmost_355, _ctx);
    kk_std_core_types__optional_drop(atmost, _ctx);
    _x_x212 = _uniq_atmost_355; /*int*/
  }
  else {
    kk_std_core_types__optional_drop(atmost, _ctx);
    _x_x212 = kk_integer_from_small(-1); /*int*/
  }
  _x_x211 = kk_std_core_int_ssize__t(_x_x212, _ctx); /*ssize_t*/
  xs_10070 = kk_std_text_regex_regex_exec_all(_x_x210, s, (KK_IZ(0)), _x_x211, _ctx); /*list<list<sslice/sslice>>*/
  kk_std_core_types__list xs_10015;
  kk_std_core_types__cctx _x_x213 = kk_cctx_empty(kk_context()); /*ctx<0>*/
  xs_10015 = kk_std_text_regex__trmc_filter_matches(xs_10070, _x_x213, _ctx); /*list<list<sslice/sslice>>*/
  kk_function_t _brw_x153 = kk_std_text_regex_new_find_all_fun214(_ctx); /*(2105) -> 2107 2106*/;
  kk_std_core_types__list _brw_x154 = kk_std_core_list_map(xs_10015, _brw_x153, _ctx); /*list<2106>*/;
  kk_function_drop(_brw_x153, _ctx);
  return _brw_x154;
}
 
// monadic lift

kk_string_t kk_std_text_regex__mlift_concat_replace_10069(kk_std_core_types__list acc, kk_std_core_types__list mm, kk_std_core_types__list pre, kk_function_t repl, kk_string_t _y_x10059, kk_context_t* _ctx) { /* forall<e> (acc : list<string>, mm : list<list<sslice/sslice>>, pre : list<sslice/sslice>, repl : (list<sslice/sslice>) -> e string, string) -> e string */ 
  kk_string_t _b_x46_52;
  if (kk_std_core_types__is_Cons(pre, _ctx)) {
    struct kk_std_core_types_Cons* _con_x217 = kk_std_core_types__as_Cons(pre, _ctx);
    kk_box_t _box_x45 = _con_x217->head;
    kk_std_core_sslice__sslice s = kk_std_core_sslice__sslice_unbox(_box_x45, KK_BORROWED, _ctx);
    kk_std_core_types__list _pat_3 = _con_x217->tail;
    if kk_likely(kk_datatype_ptr_is_unique(pre, _ctx)) {
      kk_std_core_types__list_drop(_pat_3, _ctx);
      kk_std_core_sslice__sslice_dup(s, _ctx);
      kk_box_drop(_box_x45, _ctx);
      kk_datatype_ptr_free(pre, _ctx);
    }
    else {
      kk_std_core_sslice__sslice_dup(s, _ctx);
      kk_datatype_ptr_decref(pre, _ctx);
    }
    _b_x46_52 = kk_std_core_sslice_string(s, _ctx); /*string*/
  }
  else {
    _b_x46_52 = kk_string_empty(); /*string*/
  }
  kk_std_core_types__list _x_x219;
  kk_std_core_types__list _x_x220 = kk_std_core_types__new_Cons(kk_reuse_null, 0, kk_string_box(_b_x46_52), acc, _ctx); /*list<82>*/
  _x_x219 = kk_std_core_types__new_Cons(kk_reuse_null, 0, kk_string_box(_y_x10059), _x_x220, _ctx); /*list<82>*/
  return kk_std_text_regex_concat_replace(mm, repl, _x_x219, _ctx);
}


// lift anonymous function
struct kk_std_text_regex_concat_replace_fun225__t {
  struct kk_function_s _base;
  kk_std_core_types__list acc_0;
  kk_std_core_types__list mm_0;
  kk_std_core_types__list pre_0;
  kk_function_t repl_0;
};
static kk_box_t kk_std_text_regex_concat_replace_fun225(kk_function_t _fself, kk_box_t _b_x57, kk_context_t* _ctx);
static kk_function_t kk_std_text_regex_new_concat_replace_fun225(kk_std_core_types__list acc_0, kk_std_core_types__list mm_0, kk_std_core_types__list pre_0, kk_function_t repl_0, kk_context_t* _ctx) {
  struct kk_std_text_regex_concat_replace_fun225__t* _self = kk_function_alloc_as(struct kk_std_text_regex_concat_replace_fun225__t, 5, _ctx);
  _self->_base.fun = kk_kkfun_ptr_box(&kk_std_text_regex_concat_replace_fun225, kk_context());
  _self->acc_0 = acc_0;
  _self->mm_0 = mm_0;
  _self->pre_0 = pre_0;
  _self->repl_0 = repl_0;
  return kk_datatype_from_base(&_self->_base, kk_context());
}

static kk_box_t kk_std_text_regex_concat_replace_fun225(kk_function_t _fself, kk_box_t _b_x57, kk_context_t* _ctx) {
  struct kk_std_text_regex_concat_replace_fun225__t* _self = kk_function_as(struct kk_std_text_regex_concat_replace_fun225__t*, _fself, _ctx);
  kk_std_core_types__list acc_0 = _self->acc_0; /* list<string> */
  kk_std_core_types__list mm_0 = _self->mm_0; /* list<list<sslice/sslice>> */
  kk_std_core_types__list pre_0 = _self->pre_0; /* list<sslice/sslice> */
  kk_function_t repl_0 = _self->repl_0; /* (list<sslice/sslice>) -> 446 string */
  kk_drop_match(_self, {kk_std_core_types__list_dup(acc_0, _ctx);kk_std_core_types__list_dup(mm_0, _ctx);kk_std_core_types__list_dup(pre_0, _ctx);kk_function_dup(repl_0, _ctx);}, {}, _ctx)
  kk_string_t _y_x10059_0_74 = kk_string_unbox(_b_x57); /*string*/;
  kk_string_t _x_x226 = kk_std_text_regex__mlift_concat_replace_10069(acc_0, mm_0, pre_0, repl_0, _y_x10059_0_74, _ctx); /*string*/
  return kk_string_box(_x_x226);
}

kk_string_t kk_std_text_regex_concat_replace(kk_std_core_types__list matches, kk_function_t repl_0, kk_std_core_types__list acc_0, kk_context_t* _ctx) { /* forall<e> (matches : list<list<sslice/sslice>>, repl : (list<sslice/sslice>) -> e string, acc : list<string>) -> e string */ 
  kk__tailcall: ;
  if (kk_std_core_types__is_Cons(matches, _ctx)) {
    struct kk_std_core_types_Cons* _con_x221 = kk_std_core_types__as_Cons(matches, _ctx);
    kk_box_t _box_x54 = _con_x221->head;
    kk_std_core_types__list _pat_0 = _con_x221->tail;
    if (kk_std_core_types__is_Cons(_pat_0, _ctx)) {
      struct kk_std_core_types_Cons* _con_x222 = kk_std_core_types__as_Cons(_pat_0, _ctx);
      kk_box_t _box_x55 = _con_x222->head;
      kk_std_core_types__list pre_0 = kk_std_core_types__list_unbox(_box_x54, KK_BORROWED, _ctx);
      kk_std_core_types__list mm_0 = _con_x222->tail;
      kk_std_core_types__list m = kk_std_core_types__list_unbox(_box_x55, KK_BORROWED, _ctx);
      kk_reuse_t _ru_x167 = kk_reuse_null; /*@reuse*/;
      kk_reuse_t _ru_x166 = kk_reuse_null; /*@reuse*/;
      if kk_likely(kk_datatype_ptr_is_unique(matches, _ctx)) {
        if kk_likely(kk_datatype_ptr_is_unique(_pat_0, _ctx)) {
          _ru_x166 = (kk_datatype_ptr_reuse(_pat_0, _ctx));
        }
        else {
          kk_std_core_types__list_dup(m, _ctx);
          kk_std_core_types__list_dup(mm_0, _ctx);
          kk_datatype_ptr_decref(_pat_0, _ctx);
        }
        _ru_x167 = (kk_datatype_ptr_reuse(matches, _ctx));
      }
      else {
        kk_std_core_types__list_dup(m, _ctx);
        kk_std_core_types__list_dup(mm_0, _ctx);
        kk_std_core_types__list_dup(pre_0, _ctx);
        kk_datatype_ptr_decref(matches, _ctx);
      }
      kk_string_t x_10071;
      kk_function_t _x_x223 = kk_function_dup(repl_0, _ctx); /*(list<sslice/sslice>) -> 446 string*/
      x_10071 = kk_function_call(kk_string_t, (kk_function_t, kk_std_core_types__list, kk_context_t*), _x_x223, (_x_x223, m, _ctx), _ctx); /*string*/
      if (kk_yielding(kk_context())) {
        kk_reuse_drop(_ru_x166,kk_context());
        kk_reuse_drop(_ru_x167,kk_context());
        kk_string_drop(x_10071, _ctx);
        kk_box_t _x_x224 = kk_std_core_hnd_yield_extend(kk_std_text_regex_new_concat_replace_fun225(acc_0, mm_0, pre_0, repl_0, _ctx), _ctx); /*2401*/
        return kk_string_unbox(_x_x224);
      }
      {
        kk_string_t _b_x59_70;
        if (kk_std_core_types__is_Cons(pre_0, _ctx)) {
          struct kk_std_core_types_Cons* _con_x227 = kk_std_core_types__as_Cons(pre_0, _ctx);
          kk_box_t _box_x58 = _con_x227->head;
          kk_std_core_sslice__sslice s_0 = kk_std_core_sslice__sslice_unbox(_box_x58, KK_BORROWED, _ctx);
          kk_std_core_types__list _pat_3_0 = _con_x227->tail;
          if kk_likely(kk_datatype_ptr_is_unique(pre_0, _ctx)) {
            kk_std_core_types__list_drop(_pat_3_0, _ctx);
            kk_std_core_sslice__sslice_dup(s_0, _ctx);
            kk_box_drop(_box_x58, _ctx);
            kk_datatype_ptr_free(pre_0, _ctx);
          }
          else {
            kk_std_core_sslice__sslice_dup(s_0, _ctx);
            kk_datatype_ptr_decref(pre_0, _ctx);
          }
          _b_x59_70 = kk_std_core_sslice_string(s_0, _ctx); /*string*/
        }
        else {
          _b_x59_70 = kk_string_empty(); /*string*/
        }
        { // tailcall
          kk_std_core_types__list _x_x229;
          kk_std_core_types__list _x_x230 = kk_std_core_types__new_Cons(_ru_x167, 0, kk_string_box(_b_x59_70), acc_0, _ctx); /*list<82>*/
          _x_x229 = kk_std_core_types__new_Cons(_ru_x166, 0, kk_string_box(x_10071), _x_x230, _ctx); /*list<82>*/
          matches = mm_0;
          acc_0 = _x_x229;
          goto kk__tailcall;
        }
      }
    }
  }
  if (kk_std_core_types__is_Cons(matches, _ctx)) {
    struct kk_std_core_types_Cons* _con_x231 = kk_std_core_types__as_Cons(matches, _ctx);
    kk_box_t _box_x63 = _con_x231->head;
    kk_std_core_types__list _pat_2_0_0 = _con_x231->tail;
    kk_std_core_types__list post = kk_std_core_types__list_unbox(_box_x63, KK_BORROWED, _ctx);
    kk_function_drop(repl_0, _ctx);
    kk_reuse_t _ru_x169 = kk_reuse_null; /*@reuse*/;
    if kk_likely(kk_datatype_ptr_is_unique(matches, _ctx)) {
      _ru_x169 = (kk_datatype_ptr_reuse(matches, _ctx));
    }
    else {
      kk_std_core_types__list_dup(post, _ctx);
      kk_datatype_ptr_decref(matches, _ctx);
    }
    kk_string_t _b_x65_72;
    if (kk_std_core_types__is_Cons(post, _ctx)) {
      struct kk_std_core_types_Cons* _con_x232 = kk_std_core_types__as_Cons(post, _ctx);
      kk_box_t _box_x64 = _con_x232->head;
      kk_std_core_sslice__sslice s_0_0 = kk_std_core_sslice__sslice_unbox(_box_x64, KK_BORROWED, _ctx);
      kk_std_core_types__list _pat_3_0_0 = _con_x232->tail;
      if kk_likely(kk_datatype_ptr_is_unique(post, _ctx)) {
        kk_std_core_types__list_drop(_pat_3_0_0, _ctx);
        kk_std_core_sslice__sslice_dup(s_0_0, _ctx);
        kk_box_drop(_box_x64, _ctx);
        kk_datatype_ptr_free(post, _ctx);
      }
      else {
        kk_std_core_sslice__sslice_dup(s_0_0, _ctx);
        kk_datatype_ptr_decref(post, _ctx);
      }
      _b_x65_72 = kk_std_core_sslice_string(s_0_0, _ctx); /*string*/
    }
    else {
      _b_x65_72 = kk_string_empty(); /*string*/
    }
    kk_std_core_types__list _x_x234 = kk_std_core_types__new_Cons(_ru_x169, 0, kk_string_box(_b_x65_72), acc_0, _ctx); /*list<82>*/
    return kk_std_core_list_reverse_join(_x_x234, _ctx);
  }
  {
    kk_function_drop(repl_0, _ctx);
    return kk_std_core_list_reverse_join(acc_0, _ctx);
  }
}
 
// Replace the all occurrences of `regex` by the result of the replacement fun `repl` in a string `s`.

kk_string_t kk_std_text_regex_replace_all(kk_string_t s, kk_std_text_regex__regex r, kk_function_t repl, kk_std_core_types__optional atmost, kk_context_t* _ctx) { /* forall<e> (s : string, r : regex, repl : (list<sslice/sslice>) -> e string, atmost : ? int) -> e string */ 
  kk_std_core_types__list _x_x235;
  kk_box_t _x_x236;
  {
    kk_box_t _x = r.obj;
    kk_box_dup(_x, _ctx);
    kk_std_text_regex__regex_drop(r, _ctx);
    _x_x236 = _x; /*any*/
  }
  kk_ssize_t _x_x237;
  kk_integer_t _x_x238;
  if (kk_std_core_types__is_Optional(atmost, _ctx)) {
    kk_box_t _box_x75 = atmost._cons._Optional.value;
    kk_integer_t _uniq_atmost_454 = kk_integer_unbox(_box_x75, _ctx);
    kk_integer_dup(_uniq_atmost_454, _ctx);
    kk_std_core_types__optional_drop(atmost, _ctx);
    _x_x238 = _uniq_atmost_454; /*int*/
  }
  else {
    kk_std_core_types__optional_drop(atmost, _ctx);
    _x_x238 = kk_integer_from_small(-1); /*int*/
  }
  _x_x237 = kk_std_core_int_ssize__t(_x_x238, _ctx); /*ssize_t*/
  _x_x235 = kk_std_text_regex_regex_exec_all(_x_x236, s, (KK_IZ(0)), _x_x237, _ctx); /*list<list<sslice/sslice>>*/
  return kk_std_text_regex_concat_replace(_x_x235, repl, kk_std_core_types__new_Nil(_ctx), _ctx);
}
 
// Replace using a replacement string that can contain `$$` for a `$` sign, `$n` for a capture group,
// `$&` for the entire match `==$0`.


// lift anonymous function
struct kk_std_text_regex_replace_captures_fun250__t {
  struct kk_function_s _base;
  kk_std_core_types__list caps;
};
static kk_string_t kk_std_text_regex_replace_captures_fun250(kk_function_t _fself, kk_std_core_types__list cap, kk_context_t* _ctx);
static kk_function_t kk_std_text_regex_new_replace_captures_fun250(kk_std_core_types__list caps, kk_context_t* _ctx) {
  struct kk_std_text_regex_replace_captures_fun250__t* _self = kk_function_alloc_as(struct kk_std_text_regex_replace_captures_fun250__t, 2, _ctx);
  _self->_base.fun = kk_kkfun_ptr_box(&kk_std_text_regex_replace_captures_fun250, kk_context());
  _self->caps = caps;
  return kk_datatype_from_base(&_self->_base, kk_context());
}

static kk_string_t kk_std_text_regex_replace_captures_fun250(kk_function_t _fself, kk_std_core_types__list cap, kk_context_t* _ctx) {
  struct kk_std_text_regex_replace_captures_fun250__t* _self = kk_function_as(struct kk_std_text_regex_replace_captures_fun250__t*, _fself, _ctx);
  kk_std_core_types__list caps = _self->caps; /* list<sslice/sslice> */
  kk_drop_match(_self, {kk_std_core_types__list_dup(caps, _ctx);}, {}, _ctx)
  if (kk_std_core_types__is_Cons(cap, _ctx)) {
    struct kk_std_core_types_Cons* _con_x251 = kk_std_core_types__as_Cons(cap, _ctx);
    kk_box_t _box_x79 = _con_x251->head;
    kk_std_core_types__list _pat_4 = _con_x251->tail;
    if (kk_std_core_types__is_Cons(_pat_4, _ctx)) {
      struct kk_std_core_types_Cons* _con_x252 = kk_std_core_types__as_Cons(_pat_4, _ctx);
      kk_std_core_sslice__sslice _pat_0_0_0 = kk_std_core_sslice__sslice_unbox(_box_x79, KK_BORROWED, _ctx);
      kk_box_t _box_x80 = _con_x252->head;
      kk_std_core_types__list _pat_8 = _con_x252->tail;
      if (kk_std_core_types__is_Cons(_pat_8, _ctx)) {
        struct kk_std_core_types_Cons* _con_x253 = kk_std_core_types__as_Cons(_pat_8, _ctx);
        kk_std_core_sslice__sslice digit = kk_std_core_sslice__sslice_unbox(_box_x80, KK_BORROWED, _ctx);
        kk_box_t _box_x81 = _con_x253->head;
        kk_std_core_types__list _pat_12 = _con_x253->tail;
        if (kk_std_core_types__is_Cons(_pat_12, _ctx)) {
          struct kk_std_core_types_Cons* _con_x254 = kk_std_core_types__as_Cons(_pat_12, _ctx);
          kk_std_core_sslice__sslice amp = kk_std_core_sslice__sslice_unbox(_box_x81, KK_BORROWED, _ctx);
          kk_box_t _box_x82 = _con_x254->head;
          kk_std_core_types__list _pat_16 = _con_x254->tail;
          if (kk_std_core_types__is_Nil(_pat_16, _ctx)) {
            kk_std_core_sslice__sslice dollar = kk_std_core_sslice__sslice_unbox(_box_x82, KK_BORROWED, _ctx);
            if kk_likely(kk_datatype_ptr_is_unique(cap, _ctx)) {
              if kk_likely(kk_datatype_ptr_is_unique(_pat_4, _ctx)) {
                if kk_likely(kk_datatype_ptr_is_unique(_pat_8, _ctx)) {
                  if kk_likely(kk_datatype_ptr_is_unique(_pat_12, _ctx)) {
                    kk_std_core_sslice__sslice_dup(dollar, _ctx);
                    kk_box_drop(_box_x82, _ctx);
                    kk_datatype_ptr_free(_pat_12, _ctx);
                  }
                  else {
                    kk_std_core_sslice__sslice_dup(dollar, _ctx);
                    kk_datatype_ptr_decref(_pat_12, _ctx);
                  }
                  kk_std_core_sslice__sslice_dup(amp, _ctx);
                  kk_box_drop(_box_x81, _ctx);
                  kk_datatype_ptr_free(_pat_8, _ctx);
                }
                else {
                  kk_std_core_sslice__sslice_dup(amp, _ctx);
                  kk_std_core_sslice__sslice_dup(dollar, _ctx);
                  kk_datatype_ptr_decref(_pat_8, _ctx);
                }
                kk_std_core_sslice__sslice_dup(digit, _ctx);
                kk_box_drop(_box_x80, _ctx);
                kk_datatype_ptr_free(_pat_4, _ctx);
              }
              else {
                kk_std_core_sslice__sslice_dup(amp, _ctx);
                kk_std_core_sslice__sslice_dup(digit, _ctx);
                kk_std_core_sslice__sslice_dup(dollar, _ctx);
                kk_datatype_ptr_decref(_pat_4, _ctx);
              }
              kk_box_drop(_box_x79, _ctx);
              kk_datatype_ptr_free(cap, _ctx);
            }
            else {
              kk_std_core_sslice__sslice_dup(amp, _ctx);
              kk_std_core_sslice__sslice_dup(digit, _ctx);
              kk_std_core_sslice__sslice_dup(dollar, _ctx);
              kk_datatype_ptr_decref(cap, _ctx);
            }
            bool _match_x141;
            kk_integer_t _brw_x147;
            {
              kk_integer_t _x_0 = dollar.start;
              kk_integer_dup(_x_0, _ctx);
              kk_std_core_sslice__sslice_drop(dollar, _ctx);
              _brw_x147 = _x_0; /*int*/
            }
            bool _brw_x148 = kk_integer_gte_borrow(_brw_x147,(kk_integer_from_small(0)),kk_context()); /*bool*/;
            kk_integer_drop(_brw_x147, _ctx);
            _match_x141 = _brw_x148; /*bool*/
            if (_match_x141) {
              kk_std_core_sslice__sslice_drop(digit, _ctx);
              kk_std_core_types__list_drop(caps, _ctx);
              kk_std_core_sslice__sslice_drop(amp, _ctx);
              kk_define_string_literal(, _s_x255, 1, "$", _ctx)
              return kk_string_dup(_s_x255, _ctx);
            }
            {
              kk_integer_t grp;
              bool _match_x143;
              kk_integer_t _brw_x145;
              {
                kk_integer_t _x_0_0 = amp.start;
                kk_integer_dup(_x_0_0, _ctx);
                kk_std_core_sslice__sslice_drop(amp, _ctx);
                _brw_x145 = _x_0_0; /*int*/
              }
              bool _brw_x146 = kk_integer_gte_borrow(_brw_x145,(kk_integer_from_small(0)),kk_context()); /*bool*/;
              kk_integer_drop(_brw_x145, _ctx);
              _match_x143 = _brw_x146; /*bool*/
              if (_match_x143) {
                kk_std_core_sslice__sslice_drop(digit, _ctx);
                grp = kk_integer_from_small(0); /*int*/
              }
              else {
                kk_string_t s_10028 = kk_std_core_sslice_string(digit, _ctx); /*string*/;
                kk_std_core_types__maybe m_10026;
                bool _x_x256;
                kk_std_core_types__optional _match_x144 = kk_std_core_types__new_None(_ctx); /*forall<a> ? a*/;
                if (kk_std_core_types__is_Optional(_match_x144, _ctx)) {
                  kk_box_t _box_x83 = _match_x144._cons._Optional.value;
                  bool _uniq_hex_533 = kk_bool_unbox(_box_x83);
                  kk_std_core_types__optional_drop(_match_x144, _ctx);
                  _x_x256 = _uniq_hex_533; /*bool*/
                }
                else {
                  kk_std_core_types__optional_drop(_match_x144, _ctx);
                  _x_x256 = false; /*bool*/
                }
                m_10026 = kk_std_core_int_xparse(s_10028, _x_x256, _ctx); /*maybe<int>*/
                if (kk_std_core_types__is_Nothing(m_10026, _ctx)) {
                  grp = kk_integer_from_small(0); /*int*/
                }
                else {
                  kk_box_t _box_x84 = m_10026._cons.Just.value;
                  kk_integer_t x = kk_integer_unbox(_box_x84, _ctx);
                  kk_integer_dup(x, _ctx);
                  kk_std_core_types__maybe_drop(m_10026, _ctx);
                  grp = x; /*int*/
                }
              }
              kk_std_core_types__maybe _match_x142 = kk_std_core_list__index(caps, grp, _ctx); /*maybe<5214>*/;
              if (kk_std_core_types__is_Nothing(_match_x142, _ctx)) {
                return kk_string_empty();
              }
              {
                kk_box_t _box_x85 = _match_x142._cons.Just.value;
                kk_std_core_sslice__sslice s_0 = kk_std_core_sslice__sslice_unbox(_box_x85, KK_BORROWED, _ctx);
                kk_std_core_sslice__sslice_dup(s_0, _ctx);
                kk_std_core_types__maybe_drop(_match_x142, _ctx);
                return kk_std_core_sslice_string(s_0, _ctx);
              }
            }
          }
        }
      }
    }
  }
  {
    kk_std_core_types__list_drop(caps, _ctx);
    kk_std_core_types__list_drop(cap, _ctx);
    kk_define_string_literal(, _s_x258, 1, "$", _ctx)
    return kk_string_dup(_s_x258, _ctx);
  }
}

kk_string_t kk_std_text_regex_replace_captures(kk_std_core_types__list caps, kk_string_t repl, kk_context_t* _ctx) { /* (caps : list<sslice/sslice>, repl : string) -> string */ 
  kk_std_text_regex__regex r_10075;
  kk_box_t _x_x239;
  kk_string_t _x_x240;
  kk_define_string_literal(, _s_x241, 20, "\\$(\?:(\\d)|(\\&)|(\\$))", _ctx)
  _x_x240 = kk_string_dup(_s_x241, _ctx); /*string*/
  bool _x_x242;
  kk_std_core_types__optional _match_x151 = kk_std_core_types__new_None(_ctx); /*forall<a> ? a*/;
  if (kk_std_core_types__is_Optional(_match_x151, _ctx)) {
    kk_box_t _box_x76 = _match_x151._cons._Optional.value;
    bool _uniq_ignorecase_161 = kk_bool_unbox(_box_x76);
    kk_std_core_types__optional_drop(_match_x151, _ctx);
    _x_x242 = _uniq_ignorecase_161; /*bool*/
  }
  else {
    kk_std_core_types__optional_drop(_match_x151, _ctx);
    _x_x242 = false; /*bool*/
  }
  bool _x_x243;
  kk_std_core_types__optional _match_x150 = kk_std_core_types__new_None(_ctx); /*forall<a> ? a*/;
  if (kk_std_core_types__is_Optional(_match_x150, _ctx)) {
    kk_box_t _box_x77 = _match_x150._cons._Optional.value;
    bool _uniq_multiline_165 = kk_bool_unbox(_box_x77);
    kk_std_core_types__optional_drop(_match_x150, _ctx);
    _x_x243 = _uniq_multiline_165; /*bool*/
  }
  else {
    kk_std_core_types__optional_drop(_match_x150, _ctx);
    _x_x243 = false; /*bool*/
  }
  _x_x239 = kk_std_text_regex_regex_create(_x_x240, _x_x242, _x_x243, _ctx); /*any*/
  kk_string_t _x_x244;
  kk_define_string_literal(, _s_x245, 20, "\\$(\?:(\\d)|(\\&)|(\\$))", _ctx)
  _x_x244 = kk_string_dup(_s_x245, _ctx); /*string*/
  r_10075 = kk_std_text_regex__new_Regex(_x_x239, _x_x244, _ctx); /*std/text/regex/regex*/
  kk_std_core_types__list _x_x246;
  kk_box_t _x_x247;
  {
    kk_box_t _x = r_10075.obj;
    kk_box_dup(_x, _ctx);
    kk_std_text_regex__regex_drop(r_10075, _ctx);
    _x_x247 = _x; /*any*/
  }
  kk_ssize_t _x_x248;
  kk_integer_t _x_x249;
  kk_std_core_types__optional _match_x149 = kk_std_core_types__new_None(_ctx); /*forall<a> ? a*/;
  if (kk_std_core_types__is_Optional(_match_x149, _ctx)) {
    kk_box_t _box_x78 = _match_x149._cons._Optional.value;
    kk_integer_t _uniq_atmost_454 = kk_integer_unbox(_box_x78, _ctx);
    kk_integer_dup(_uniq_atmost_454, _ctx);
    kk_std_core_types__optional_drop(_match_x149, _ctx);
    _x_x249 = _uniq_atmost_454; /*int*/
  }
  else {
    kk_std_core_types__optional_drop(_match_x149, _ctx);
    _x_x249 = kk_integer_from_small(-1); /*int*/
  }
  _x_x248 = kk_std_core_int_ssize__t(_x_x249, _ctx); /*ssize_t*/
  _x_x246 = kk_std_text_regex_regex_exec_all(_x_x247, repl, (KK_IZ(0)), _x_x248, _ctx); /*list<list<sslice/sslice>>*/
  return kk_std_text_regex_concat_replace(_x_x246, kk_std_text_regex_new_replace_captures_fun250(caps, _ctx), kk_std_core_types__new_Nil(_ctx), _ctx);
}
 
// Replace all occurrences of `regex` with the replacement string `repl` in a string `s`.
// The replacement string can contain `$$` for a `$` sign, `$n` for a capture group,
// `$&` for the entire match `==$0`.


// lift anonymous function
struct kk_std_text_regex_string_fs_replace_all_fun266__t {
  struct kk_function_s _base;
  kk_string_t repl;
};
static kk_string_t kk_std_text_regex_string_fs_replace_all_fun266(kk_function_t _fself, kk_std_core_types__list caps, kk_context_t* _ctx);
static kk_function_t kk_std_text_regex_string_fs_new_replace_all_fun266(kk_string_t repl, kk_context_t* _ctx) {
  struct kk_std_text_regex_string_fs_replace_all_fun266__t* _self = kk_function_alloc_as(struct kk_std_text_regex_string_fs_replace_all_fun266__t, 2, _ctx);
  _self->_base.fun = kk_kkfun_ptr_box(&kk_std_text_regex_string_fs_replace_all_fun266, kk_context());
  _self->repl = repl;
  return kk_datatype_from_base(&_self->_base, kk_context());
}

static kk_string_t kk_std_text_regex_string_fs_replace_all_fun266(kk_function_t _fself, kk_std_core_types__list caps, kk_context_t* _ctx) {
  struct kk_std_text_regex_string_fs_replace_all_fun266__t* _self = kk_function_as(struct kk_std_text_regex_string_fs_replace_all_fun266__t*, _fself, _ctx);
  kk_string_t repl = _self->repl; /* string */
  kk_drop_match(_self, {kk_string_dup(repl, _ctx);}, {}, _ctx)
  return kk_std_text_regex_replace_captures(caps, repl, _ctx);
}


// lift anonymous function
struct kk_std_text_regex_string_fs_replace_all_fun271__t {
  struct kk_function_s _base;
  kk_string_t repl;
};
static kk_string_t kk_std_text_regex_string_fs_replace_all_fun271(kk_function_t _fself, kk_std_core_types__list ___wildcard_x158__35, kk_context_t* _ctx);
static kk_function_t kk_std_text_regex_string_fs_new_replace_all_fun271(kk_string_t repl, kk_context_t* _ctx) {
  struct kk_std_text_regex_string_fs_replace_all_fun271__t* _self = kk_function_alloc_as(struct kk_std_text_regex_string_fs_replace_all_fun271__t, 2, _ctx);
  _self->_base.fun = kk_kkfun_ptr_box(&kk_std_text_regex_string_fs_replace_all_fun271, kk_context());
  _self->repl = repl;
  return kk_datatype_from_base(&_self->_base, kk_context());
}

static kk_string_t kk_std_text_regex_string_fs_replace_all_fun271(kk_function_t _fself, kk_std_core_types__list ___wildcard_x158__35, kk_context_t* _ctx) {
  struct kk_std_text_regex_string_fs_replace_all_fun271__t* _self = kk_function_as(struct kk_std_text_regex_string_fs_replace_all_fun271__t*, _fself, _ctx);
  kk_string_t repl = _self->repl; /* string */
  kk_drop_match(_self, {kk_string_dup(repl, _ctx);}, {}, _ctx)
  kk_std_core_types__list_drop(___wildcard_x158__35, _ctx);
  return repl;
}

kk_string_t kk_std_text_regex_string_fs_replace_all(kk_string_t s, kk_std_text_regex__regex regex_0, kk_string_t repl, kk_std_core_types__optional atmost, kk_context_t* _ctx) { /* (s : string, regex : regex, repl : string, atmost : ? int) -> string */ 
  bool _match_x140;
  kk_string_t _x_x259 = kk_string_dup(repl, _ctx); /*string*/
  kk_string_t _x_x260;
  kk_define_string_literal(, _s_x261, 1, "$", _ctx)
  _x_x260 = kk_string_dup(_s_x261, _ctx); /*string*/
  _match_x140 = kk_string_contains(_x_x259,_x_x260,kk_context()); /*bool*/
  if (_match_x140) {
    kk_std_core_types__list _x_x262;
    kk_box_t _x_x263;
    {
      kk_box_t _x = regex_0.obj;
      kk_box_dup(_x, _ctx);
      kk_std_text_regex__regex_drop(regex_0, _ctx);
      _x_x263 = _x; /*any*/
    }
    kk_ssize_t _x_x264;
    kk_integer_t _x_x265;
    if (kk_std_core_types__is_Optional(atmost, _ctx)) {
      kk_box_t _box_x86 = atmost._cons._Optional.value;
      kk_integer_t _uniq_atmost_487 = kk_integer_unbox(_box_x86, _ctx);
      kk_integer_dup(_uniq_atmost_487, _ctx);
      kk_std_core_types__optional_drop(atmost, _ctx);
      _x_x265 = _uniq_atmost_487; /*int*/
    }
    else {
      kk_std_core_types__optional_drop(atmost, _ctx);
      _x_x265 = kk_integer_from_small(-1); /*int*/
    }
    _x_x264 = kk_std_core_int_ssize__t(_x_x265, _ctx); /*ssize_t*/
    _x_x262 = kk_std_text_regex_regex_exec_all(_x_x263, s, (KK_IZ(0)), _x_x264, _ctx); /*list<list<sslice/sslice>>*/
    return kk_std_text_regex_concat_replace(_x_x262, kk_std_text_regex_string_fs_new_replace_all_fun266(repl, _ctx), kk_std_core_types__new_Nil(_ctx), _ctx);
  }
  {
    kk_std_core_types__list _x_x267;
    kk_box_t _x_x268;
    {
      kk_box_t _x_0 = regex_0.obj;
      kk_box_dup(_x_0, _ctx);
      kk_std_text_regex__regex_drop(regex_0, _ctx);
      _x_x268 = _x_0; /*any*/
    }
    kk_ssize_t _x_x269;
    kk_integer_t _x_x270;
    if (kk_std_core_types__is_Optional(atmost, _ctx)) {
      kk_box_t _box_x87 = atmost._cons._Optional.value;
      kk_integer_t _uniq_atmost_487_0 = kk_integer_unbox(_box_x87, _ctx);
      kk_integer_dup(_uniq_atmost_487_0, _ctx);
      kk_std_core_types__optional_drop(atmost, _ctx);
      _x_x270 = _uniq_atmost_487_0; /*int*/
    }
    else {
      kk_std_core_types__optional_drop(atmost, _ctx);
      _x_x270 = kk_integer_from_small(-1); /*int*/
    }
    _x_x269 = kk_std_core_int_ssize__t(_x_x270, _ctx); /*ssize_t*/
    _x_x267 = kk_std_text_regex_regex_exec_all(_x_x268, s, (KK_IZ(0)), _x_x269, _ctx); /*list<list<sslice/sslice>>*/
    return kk_std_text_regex_concat_replace(_x_x267, kk_std_text_regex_string_fs_new_replace_all_fun271(repl, _ctx), kk_std_core_types__new_Nil(_ctx), _ctx);
  }
}
 
// How many groups are captured by this regex?


// lift anonymous function
struct kk_std_text_regex_groups_count_fun283__t {
  struct kk_function_s _base;
};
static kk_string_t kk_std_text_regex_groups_count_fun283(kk_function_t _fself, kk_std_core_types__list caps, kk_context_t* _ctx);
static kk_function_t kk_std_text_regex_new_groups_count_fun283(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_text_regex_groups_count_fun283, _ctx)
  return kk_function_static_dup(_fself,kk_context());
}

static kk_string_t kk_std_text_regex_groups_count_fun283(kk_function_t _fself, kk_std_core_types__list caps, kk_context_t* _ctx) {
  kk_function_static_drop(_fself,kk_context());
  kk_string_t _x_x284 = kk_string_empty(); /*string*/
  return kk_std_text_regex_replace_captures(caps, _x_x284, _ctx);
}


// lift anonymous function
struct kk_std_text_regex_groups_count_fun292__t {
  struct kk_function_s _base;
};
static kk_string_t kk_std_text_regex_groups_count_fun292(kk_function_t _fself, kk_std_core_types__list ___wildcard_x158__35, kk_context_t* _ctx);
static kk_function_t kk_std_text_regex_new_groups_count_fun292(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_text_regex_groups_count_fun292, _ctx)
  return kk_function_static_dup(_fself,kk_context());
}

static kk_string_t kk_std_text_regex_groups_count_fun292(kk_function_t _fself, kk_std_core_types__list ___wildcard_x158__35, kk_context_t* _ctx) {
  kk_function_static_drop(_fself,kk_context());
  kk_std_core_types__list_drop(___wildcard_x158__35, _ctx);
  return kk_string_empty();
}

kk_integer_t kk_std_text_regex_groups_count(kk_std_text_regex__regex r, kk_context_t* _ctx) { /* (r : regex) -> int */ 
  kk_string_t _x_x272;
  bool _match_x137;
  kk_string_t _x_x273 = kk_string_empty(); /*string*/
  kk_string_t _x_x275;
  kk_define_string_literal(, _s_x276, 1, "$", _ctx)
  _x_x275 = kk_string_dup(_s_x276, _ctx); /*string*/
  _match_x137 = kk_string_contains(_x_x273,_x_x275,kk_context()); /*bool*/
  if (_match_x137) {
    kk_std_core_types__list _x_x277;
    kk_box_t _x_x278;
    kk_std_text_regex__regex _x_x279 = kk_std_text_regex_rx_nongroup; /*std/text/regex/regex*/
    {
      kk_box_t _x = _x_x279.obj;
      _x_x278 = kk_box_dup(_x, _ctx); /*any*/
    }
    kk_string_t _x_x280;
    {
      kk_string_t _x_0 = r.src;
      kk_string_dup(_x_0, _ctx);
      kk_std_text_regex__regex_drop(r, _ctx);
      _x_x280 = _x_0; /*string*/
    }
    kk_ssize_t _x_x281;
    kk_integer_t _x_x282;
    kk_std_core_types__optional _match_x139 = kk_std_core_types__new_None(_ctx); /*forall<a> ? a*/;
    if (kk_std_core_types__is_Optional(_match_x139, _ctx)) {
      kk_box_t _box_x88 = _match_x139._cons._Optional.value;
      kk_integer_t _uniq_atmost_487 = kk_integer_unbox(_box_x88, _ctx);
      kk_integer_dup(_uniq_atmost_487, _ctx);
      kk_std_core_types__optional_drop(_match_x139, _ctx);
      _x_x282 = _uniq_atmost_487; /*int*/
    }
    else {
      kk_std_core_types__optional_drop(_match_x139, _ctx);
      _x_x282 = kk_integer_from_small(-1); /*int*/
    }
    _x_x281 = kk_std_core_int_ssize__t(_x_x282, _ctx); /*ssize_t*/
    _x_x277 = kk_std_text_regex_regex_exec_all(_x_x278, _x_x280, (KK_IZ(0)), _x_x281, _ctx); /*list<list<sslice/sslice>>*/
    _x_x272 = kk_std_text_regex_concat_replace(_x_x277, kk_std_text_regex_new_groups_count_fun283(_ctx), kk_std_core_types__new_Nil(_ctx), _ctx); /*string*/
  }
  else {
    kk_std_core_types__list _x_x286;
    kk_box_t _x_x287;
    kk_std_text_regex__regex _x_x288 = kk_std_text_regex_rx_nongroup; /*std/text/regex/regex*/
    {
      kk_box_t _x_1 = _x_x288.obj;
      _x_x287 = kk_box_dup(_x_1, _ctx); /*any*/
    }
    kk_string_t _x_x289;
    {
      kk_string_t _x_0_0 = r.src;
      kk_string_dup(_x_0_0, _ctx);
      kk_std_text_regex__regex_drop(r, _ctx);
      _x_x289 = _x_0_0; /*string*/
    }
    kk_ssize_t _x_x290;
    kk_integer_t _x_x291;
    kk_std_core_types__optional _match_x138 = kk_std_core_types__new_None(_ctx); /*forall<a> ? a*/;
    if (kk_std_core_types__is_Optional(_match_x138, _ctx)) {
      kk_box_t _box_x89 = _match_x138._cons._Optional.value;
      kk_integer_t _uniq_atmost_487_0 = kk_integer_unbox(_box_x89, _ctx);
      kk_integer_dup(_uniq_atmost_487_0, _ctx);
      kk_std_core_types__optional_drop(_match_x138, _ctx);
      _x_x291 = _uniq_atmost_487_0; /*int*/
    }
    else {
      kk_std_core_types__optional_drop(_match_x138, _ctx);
      _x_x291 = kk_integer_from_small(-1); /*int*/
    }
    _x_x290 = kk_std_core_int_ssize__t(_x_x291, _ctx); /*ssize_t*/
    _x_x286 = kk_std_text_regex_regex_exec_all(_x_x287, _x_x289, (KK_IZ(0)), _x_x290, _ctx); /*list<list<sslice/sslice>>*/
    _x_x272 = kk_std_text_regex_concat_replace(_x_x286, kk_std_text_regex_new_groups_count_fun292(_ctx), kk_std_core_types__new_Nil(_ctx), _ctx); /*string*/
  }
  return kk_std_core_string_chars_fs_count(_x_x272, _ctx);
}
 
// Replace the first occurrence of `regex` by the result of the replacement fun `repl` in a string `s`.

kk_string_t kk_std_text_regex_replace(kk_string_t s, kk_std_text_regex__regex r, kk_function_t repl, kk_context_t* _ctx) { /* forall<e> (s : string, r : regex, repl : (list<sslice/sslice>) -> e string) -> e string */ 
  kk_std_core_types__list _x_x294;
  kk_box_t _x_x295;
  {
    kk_box_t _x = r.obj;
    kk_box_dup(_x, _ctx);
    kk_std_text_regex__regex_drop(r, _ctx);
    _x_x295 = _x; /*any*/
  }
  _x_x294 = kk_std_text_regex_regex_exec_all(_x_x295, s, (KK_IZ(0)), (KK_IZ(1)), _ctx); /*list<list<sslice/sslice>>*/
  return kk_std_text_regex_concat_replace(_x_x294, repl, kk_std_core_types__new_Nil(_ctx), _ctx);
}
 
// Replace the first occurrence of `regex` with a replacement string `repl` in a string `s`.
// The replacement string can contain `$$` for a `$` sign, `$n` for a capture group,
// `$&` for the entire match `==$0`.


// lift anonymous function
struct kk_std_text_regex_string_fs_replace_fun301__t {
  struct kk_function_s _base;
  kk_string_t repl;
};
static kk_string_t kk_std_text_regex_string_fs_replace_fun301(kk_function_t _fself, kk_std_core_types__list caps, kk_context_t* _ctx);
static kk_function_t kk_std_text_regex_string_fs_new_replace_fun301(kk_string_t repl, kk_context_t* _ctx) {
  struct kk_std_text_regex_string_fs_replace_fun301__t* _self = kk_function_alloc_as(struct kk_std_text_regex_string_fs_replace_fun301__t, 2, _ctx);
  _self->_base.fun = kk_kkfun_ptr_box(&kk_std_text_regex_string_fs_replace_fun301, kk_context());
  _self->repl = repl;
  return kk_datatype_from_base(&_self->_base, kk_context());
}

static kk_string_t kk_std_text_regex_string_fs_replace_fun301(kk_function_t _fself, kk_std_core_types__list caps, kk_context_t* _ctx) {
  struct kk_std_text_regex_string_fs_replace_fun301__t* _self = kk_function_as(struct kk_std_text_regex_string_fs_replace_fun301__t*, _fself, _ctx);
  kk_string_t repl = _self->repl; /* string */
  kk_drop_match(_self, {kk_string_dup(repl, _ctx);}, {}, _ctx)
  return kk_std_text_regex_replace_captures(caps, repl, _ctx);
}


// lift anonymous function
struct kk_std_text_regex_string_fs_replace_fun304__t {
  struct kk_function_s _base;
  kk_string_t repl;
};
static kk_string_t kk_std_text_regex_string_fs_replace_fun304(kk_function_t _fself, kk_std_core_types__list ___wildcard_x158__35, kk_context_t* _ctx);
static kk_function_t kk_std_text_regex_string_fs_new_replace_fun304(kk_string_t repl, kk_context_t* _ctx) {
  struct kk_std_text_regex_string_fs_replace_fun304__t* _self = kk_function_alloc_as(struct kk_std_text_regex_string_fs_replace_fun304__t, 2, _ctx);
  _self->_base.fun = kk_kkfun_ptr_box(&kk_std_text_regex_string_fs_replace_fun304, kk_context());
  _self->repl = repl;
  return kk_datatype_from_base(&_self->_base, kk_context());
}

static kk_string_t kk_std_text_regex_string_fs_replace_fun304(kk_function_t _fself, kk_std_core_types__list ___wildcard_x158__35, kk_context_t* _ctx) {
  struct kk_std_text_regex_string_fs_replace_fun304__t* _self = kk_function_as(struct kk_std_text_regex_string_fs_replace_fun304__t*, _fself, _ctx);
  kk_string_t repl = _self->repl; /* string */
  kk_drop_match(_self, {kk_string_dup(repl, _ctx);}, {}, _ctx)
  kk_std_core_types__list_drop(___wildcard_x158__35, _ctx);
  return repl;
}

kk_string_t kk_std_text_regex_string_fs_replace(kk_string_t s, kk_std_text_regex__regex regex_0, kk_string_t repl, kk_context_t* _ctx) { /* (s : string, regex : regex, repl : string) -> string */ 
  bool _match_x134;
  kk_string_t _x_x296 = kk_string_dup(repl, _ctx); /*string*/
  kk_string_t _x_x297;
  kk_define_string_literal(, _s_x298, 1, "$", _ctx)
  _x_x297 = kk_string_dup(_s_x298, _ctx); /*string*/
  _match_x134 = kk_string_contains(_x_x296,_x_x297,kk_context()); /*bool*/
  if (_match_x134) {
    kk_std_core_types__list _x_x299;
    kk_box_t _x_x300;
    {
      kk_box_t _x = regex_0.obj;
      kk_box_dup(_x, _ctx);
      kk_std_text_regex__regex_drop(regex_0, _ctx);
      _x_x300 = _x; /*any*/
    }
    _x_x299 = kk_std_text_regex_regex_exec_all(_x_x300, s, (KK_IZ(0)), (KK_IZ(1)), _ctx); /*list<list<sslice/sslice>>*/
    return kk_std_text_regex_concat_replace(_x_x299, kk_std_text_regex_string_fs_new_replace_fun301(repl, _ctx), kk_std_core_types__new_Nil(_ctx), _ctx);
  }
  {
    kk_std_core_types__list _x_x302;
    kk_box_t _x_x303;
    {
      kk_box_t _x_0 = regex_0.obj;
      kk_box_dup(_x_0, _ctx);
      kk_std_text_regex__regex_drop(regex_0, _ctx);
      _x_x303 = _x_0; /*any*/
    }
    _x_x302 = kk_std_text_regex_regex_exec_all(_x_x303, s, (KK_IZ(0)), (KK_IZ(1)), _ctx); /*list<list<sslice/sslice>>*/
    return kk_std_text_regex_concat_replace(_x_x302, kk_std_text_regex_string_fs_new_replace_fun304(repl, _ctx), kk_std_core_types__new_Nil(_ctx), _ctx);
  }
}
 
// Filter only for the non-matched parts.

kk_std_core_types__list kk_std_text_regex__trmc_filter_non_matches(kk_std_core_types__list xs, kk_std_core_types__cctx _acc, kk_context_t* _ctx) { /* (xs : list<list<sslice/sslice>>, ctx<list<list<sslice/sslice>>>) -> list<list<sslice/sslice>> */ 
  kk__tailcall: ;
  if (kk_std_core_types__is_Cons(xs, _ctx)) {
    struct kk_std_core_types_Cons* _con_x305 = kk_std_core_types__as_Cons(xs, _ctx);
    kk_box_t _box_x90 = _con_x305->head;
    kk_std_core_types__list _pat_0 = _con_x305->tail;
    if (kk_std_core_types__is_Cons(_pat_0, _ctx)) {
      struct kk_std_core_types_Cons* _con_x306 = kk_std_core_types__as_Cons(_pat_0, _ctx);
      kk_box_t _box_x91 = _con_x306->head;
      kk_std_core_types__list s = kk_std_core_types__list_unbox(_box_x90, KK_BORROWED, _ctx);
      kk_std_core_types__list xx = _con_x306->tail;
      kk_std_core_types__list _pat_1 = kk_std_core_types__list_unbox(_box_x91, KK_BORROWED, _ctx);
      kk_reuse_t _ru_x176 = kk_reuse_null; /*@reuse*/;
      if kk_likely(kk_datatype_ptr_is_unique(xs, _ctx)) {
        if kk_likely(kk_datatype_ptr_is_unique(_pat_0, _ctx)) {
          kk_std_core_types__list_drop(_pat_1, _ctx);
          kk_datatype_ptr_free(_pat_0, _ctx);
        }
        else {
          kk_std_core_types__list_dup(xx, _ctx);
          kk_datatype_ptr_decref(_pat_0, _ctx);
        }
        _ru_x176 = (kk_datatype_ptr_reuse(xs, _ctx));
      }
      else {
        kk_std_core_types__list_dup(s, _ctx);
        kk_std_core_types__list_dup(xx, _ctx);
        kk_datatype_ptr_decref(xs, _ctx);
      }
      kk_std_core_types__list _trmc_x10057 = kk_datatype_null(); /*list<list<sslice/sslice>>*/;
      kk_std_core_types__list _trmc_x10058 = kk_std_core_types__new_Cons(_ru_x176, 0, kk_std_core_types__list_box(s, _ctx), _trmc_x10057, _ctx); /*list<list<sslice/sslice>>*/;
      kk_field_addr_t _b_x101_111 = kk_field_addr_create(&kk_std_core_types__as_Cons(_trmc_x10058, _ctx)->tail, _ctx); /*@field-addr<list<list<sslice/sslice>>>*/;
      { // tailcall
        kk_std_core_types__cctx _x_x307 = kk_cctx_extend_linear(_acc,(kk_std_core_types__list_box(_trmc_x10058, _ctx)),_b_x101_111,kk_context()); /*ctx<0>*/
        xs = xx;
        _acc = _x_x307;
        goto kk__tailcall;
      }
    }
  }
  if (kk_std_core_types__is_Cons(xs, _ctx)) {
    struct kk_std_core_types_Cons* _con_x308 = kk_std_core_types__as_Cons(xs, _ctx);
    kk_box_t _box_x102 = _con_x308->head;
    kk_std_core_types__list _pat_3 = _con_x308->tail;
    kk_std_core_types__list s_0 = kk_std_core_types__list_unbox(_box_x102, KK_BORROWED, _ctx);
    kk_reuse_t _ru_x177 = kk_reuse_null; /*@reuse*/;
    if kk_likely(kk_datatype_ptr_is_unique(xs, _ctx)) {
      _ru_x177 = (kk_datatype_ptr_reuse(xs, _ctx));
    }
    else {
      kk_std_core_types__list_dup(s_0, _ctx);
      kk_datatype_ptr_decref(xs, _ctx);
    }
    kk_box_t _x_x309;
    kk_box_t _x_x310;
    kk_std_core_types__list _x_x311 = kk_std_core_types__new_Cons(_ru_x177, 0, kk_std_core_types__list_box(s_0, _ctx), kk_std_core_types__new_Nil(_ctx), _ctx); /*list<82>*/
    _x_x310 = kk_std_core_types__list_box(_x_x311, _ctx); /*-1*/
    _x_x309 = kk_cctx_apply_linear(_acc,_x_x310,kk_context()); /*-1*/
    return kk_std_core_types__list_unbox(_x_x309, KK_OWNED, _ctx);
  }
  {
    kk_box_t _x_x312 = kk_cctx_apply_linear(_acc,(kk_std_core_types__list_box(kk_std_core_types__new_Nil(_ctx), _ctx)),kk_context()); /*-1*/
    return kk_std_core_types__list_unbox(_x_x312, KK_OWNED, _ctx);
  }
}
 
// Filter only for the non-matched parts.

kk_std_core_types__list kk_std_text_regex_filter_non_matches(kk_std_core_types__list xs_0, kk_context_t* _ctx) { /* (xs : list<list<sslice/sslice>>) -> list<list<sslice/sslice>> */ 
  kk_std_core_types__cctx _x_x313 = kk_cctx_empty(kk_context()); /*ctx<0>*/
  return kk_std_text_regex__trmc_filter_non_matches(xs_0, _x_x313, _ctx);
}
 
// Split a string `s` in at most `atmost` parts using a regular expression `r` as separator.


// lift anonymous function
struct kk_std_text_regex_split_fun318__t {
  struct kk_function_s _base;
};
static kk_box_t kk_std_text_regex_split_fun318(kk_function_t _fself, kk_box_t _b_x124, kk_context_t* _ctx);
static kk_function_t kk_std_text_regex_new_split_fun318(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_text_regex_split_fun318, _ctx)
  return kk_function_static_dup(_fself,kk_context());
}

static kk_box_t kk_std_text_regex_split_fun318(kk_function_t _fself, kk_box_t _b_x124, kk_context_t* _ctx) {
  kk_function_static_drop(_fself,kk_context());
  kk_string_t _x_x319;
  kk_std_core_types__list _x_x320 = kk_std_core_types__list_unbox(_b_x124, KK_OWNED, _ctx); /*list<sslice/sslice>*/
  _x_x319 = kk_std_text_regex_captured(_x_x320, _ctx); /*string*/
  return kk_string_box(_x_x319);
}

kk_std_core_types__list kk_std_text_regex_split(kk_string_t s, kk_std_text_regex__regex r, kk_std_core_types__optional atmost, kk_context_t* _ctx) { /* (s : string, r : regex, atmost : ? int) -> list<string> */ 
  kk_std_core_types__list xs_10106;
  kk_box_t _x_x314;
  {
    kk_box_t _x = r.obj;
    kk_box_dup(_x, _ctx);
    kk_std_text_regex__regex_drop(r, _ctx);
    _x_x314 = _x; /*any*/
  }
  kk_ssize_t _x_x315;
  kk_integer_t _x_x316;
  if (kk_std_core_types__is_Optional(atmost, _ctx)) {
    kk_box_t _box_x121 = atmost._cons._Optional.value;
    kk_integer_t _uniq_atmost_822 = kk_integer_unbox(_box_x121, _ctx);
    kk_integer_dup(_uniq_atmost_822, _ctx);
    kk_std_core_types__optional_drop(atmost, _ctx);
    _x_x316 = _uniq_atmost_822; /*int*/
  }
  else {
    kk_std_core_types__optional_drop(atmost, _ctx);
    _x_x316 = kk_integer_from_small(-1); /*int*/
  }
  _x_x315 = kk_std_core_int_ssize__t(_x_x316, _ctx); /*ssize_t*/
  xs_10106 = kk_std_text_regex_regex_exec_all(_x_x314, s, (KK_IZ(0)), _x_x315, _ctx); /*list<list<sslice/sslice>>*/
  kk_std_core_types__list xs_10041;
  kk_std_core_types__cctx _x_x317 = kk_cctx_empty(kk_context()); /*ctx<0>*/
  xs_10041 = kk_std_text_regex__trmc_filter_non_matches(xs_10106, _x_x317, _ctx); /*list<list<sslice/sslice>>*/
  kk_function_t _brw_x132 = kk_std_text_regex_new_split_fun318(_ctx); /*(2105) -> 2107 2106*/;
  kk_std_core_types__list _brw_x133 = kk_std_core_list_map(xs_10041, _brw_x132, _ctx); /*list<2106>*/;
  kk_function_drop(_brw_x132, _ctx);
  return _brw_x133;
}

bool kk_std_text_regex_testabc(kk_string_t s, kk_context_t* _ctx) { /* (s : string) -> bool */ 
  kk_std_text_regex__regex r_10047;
  kk_box_t _x_x321;
  kk_string_t _x_x322;
  kk_define_string_literal(, _s_x323, 6, "[ab]+c", _ctx)
  _x_x322 = kk_string_dup(_s_x323, _ctx); /*string*/
  bool _x_x324;
  kk_std_core_types__optional _match_x131 = kk_std_core_types__new_None(_ctx); /*forall<a> ? a*/;
  if (kk_std_core_types__is_Optional(_match_x131, _ctx)) {
    kk_box_t _box_x127 = _match_x131._cons._Optional.value;
    bool _uniq_ignorecase_161 = kk_bool_unbox(_box_x127);
    kk_std_core_types__optional_drop(_match_x131, _ctx);
    _x_x324 = _uniq_ignorecase_161; /*bool*/
  }
  else {
    kk_std_core_types__optional_drop(_match_x131, _ctx);
    _x_x324 = false; /*bool*/
  }
  bool _x_x325;
  kk_std_core_types__optional _match_x130 = kk_std_core_types__new_None(_ctx); /*forall<a> ? a*/;
  if (kk_std_core_types__is_Optional(_match_x130, _ctx)) {
    kk_box_t _box_x128 = _match_x130._cons._Optional.value;
    bool _uniq_multiline_165 = kk_bool_unbox(_box_x128);
    kk_std_core_types__optional_drop(_match_x130, _ctx);
    _x_x325 = _uniq_multiline_165; /*bool*/
  }
  else {
    kk_std_core_types__optional_drop(_match_x130, _ctx);
    _x_x325 = false; /*bool*/
  }
  _x_x321 = kk_std_text_regex_regex_create(_x_x322, _x_x324, _x_x325, _ctx); /*any*/
  kk_string_t _x_x326;
  kk_define_string_literal(, _s_x327, 6, "[ab]+c", _ctx)
  _x_x326 = kk_string_dup(_s_x327, _ctx); /*string*/
  r_10047 = kk_std_text_regex__new_Regex(_x_x321, _x_x326, _ctx); /*std/text/regex/regex*/
  kk_std_core_types__list list_10051;
  kk_box_t _x_x328;
  {
    kk_box_t _x = r_10047.obj;
    kk_box_dup(_x, _ctx);
    kk_std_text_regex__regex_drop(r_10047, _ctx);
    _x_x328 = _x; /*any*/
  }
  list_10051 = kk_std_text_regex_regex_exec(_x_x328, s, (KK_IZ(0)), _ctx); /*list<sslice/sslice>*/
  if (kk_std_core_types__is_Cons(list_10051, _ctx)) {
    struct kk_std_core_types_Cons* _con_x329 = kk_std_core_types__as_Cons(list_10051, _ctx);
    kk_box_t _box_x129 = _con_x329->head;
    kk_std_core_types__list _pat_1 = _con_x329->tail;
    if kk_likely(kk_datatype_ptr_is_unique(list_10051, _ctx)) {
      kk_std_core_types__list_drop(_pat_1, _ctx);
      kk_box_drop(_box_x129, _ctx);
      kk_datatype_ptr_free(list_10051, _ctx);
    }
    else {
      kk_datatype_ptr_decref(list_10051, _ctx);
    }
    return true;
  }
  {
    return false;
  }
}

// initialization
void kk_std_text_regex__init(kk_context_t* _ctx){
  static bool _kk_initialized = false;
  if (_kk_initialized) return;
  _kk_initialized = true;
  kk_std_core_types__init(_ctx);
  kk_std_core_hnd__init(_ctx);
  kk_std_core_exn__init(_ctx);
  kk_std_core_bool__init(_ctx);
  kk_std_core_order__init(_ctx);
  kk_std_core_char__init(_ctx);
  kk_std_core_int__init(_ctx);
  kk_std_core_vector__init(_ctx);
  kk_std_core_string__init(_ctx);
  kk_std_core_sslice__init(_ctx);
  kk_std_core_list__init(_ctx);
  kk_std_core_maybe__init(_ctx);
  kk_std_core_maybe2__init(_ctx);
  kk_std_core_either__init(_ctx);
  kk_std_core_tuple__init(_ctx);
  kk_std_core_lazy__init(_ctx);
  kk_std_core_show__init(_ctx);
  kk_std_core_debug__init(_ctx);
  kk_std_core_delayed__init(_ctx);
  kk_std_core_console__init(_ctx);
  kk_std_core__init(_ctx);
  kk_std_num_int32__init(_ctx);
  #if defined(KK_CUSTOM_INIT)
    KK_CUSTOM_INIT (_ctx);
  #endif
  {
    kk_box_t _x_x185;
    kk_string_t _x_x186;
    kk_define_string_literal(, _s_x187, 45, "[^\\\\\\[(]+|\\\\[\\s\\S]\?|\\(\\\?|\\[(\?:[^\\\\\\]]|\\\\.)*\\]", _ctx)
    _x_x186 = kk_string_dup(_s_x187, _ctx); /*string*/
    bool _x_x188;
    kk_std_core_types__optional _match_x159 = kk_std_core_types__new_None(_ctx); /*forall<a> ? a*/;
    if (kk_std_core_types__is_Optional(_match_x159, _ctx)) {
      kk_box_t _box_x4 = _match_x159._cons._Optional.value;
      bool _uniq_ignorecase_161 = kk_bool_unbox(_box_x4);
      kk_std_core_types__optional_drop(_match_x159, _ctx);
      _x_x188 = _uniq_ignorecase_161; /*bool*/
    }
    else {
      kk_std_core_types__optional_drop(_match_x159, _ctx);
      _x_x188 = false; /*bool*/
    }
    bool _x_x189;
    kk_std_core_types__optional _match_x158 = kk_std_core_types__new_None(_ctx); /*forall<a> ? a*/;
    if (kk_std_core_types__is_Optional(_match_x158, _ctx)) {
      kk_box_t _box_x5 = _match_x158._cons._Optional.value;
      bool _uniq_multiline_165 = kk_bool_unbox(_box_x5);
      kk_std_core_types__optional_drop(_match_x158, _ctx);
      _x_x189 = _uniq_multiline_165; /*bool*/
    }
    else {
      kk_std_core_types__optional_drop(_match_x158, _ctx);
      _x_x189 = false; /*bool*/
    }
    _x_x185 = kk_std_text_regex_regex_create(_x_x186, _x_x188, _x_x189, _ctx); /*any*/
    kk_string_t _x_x190;
    kk_define_string_literal(, _s_x191, 45, "[^\\\\\\[(]+|\\\\[\\s\\S]\?|\\(\\\?|\\[(\?:[^\\\\\\]]|\\\\.)*\\]", _ctx)
    _x_x190 = kk_string_dup(_s_x191, _ctx); /*string*/
    kk_std_text_regex_rx_nongroup = kk_std_text_regex__new_Regex(_x_x185, _x_x190, _ctx); /*std/text/regex/regex*/
  }
}

// termination
void kk_std_text_regex__done(kk_context_t* _ctx){
  static bool _kk_done = false;
  if (_kk_done) return;
  _kk_done = true;
  #if defined(KK_CUSTOM_DONE)
    KK_CUSTOM_DONE (_ctx);
  #endif
  kk_std_text_regex__regex_drop(kk_std_text_regex_rx_nongroup, _ctx);
  kk_std_num_int32__done(_ctx);
  kk_std_core__done(_ctx);
  kk_std_core_console__done(_ctx);
  kk_std_core_delayed__done(_ctx);
  kk_std_core_debug__done(_ctx);
  kk_std_core_show__done(_ctx);
  kk_std_core_lazy__done(_ctx);
  kk_std_core_tuple__done(_ctx);
  kk_std_core_either__done(_ctx);
  kk_std_core_maybe2__done(_ctx);
  kk_std_core_maybe__done(_ctx);
  kk_std_core_list__done(_ctx);
  kk_std_core_sslice__done(_ctx);
  kk_std_core_string__done(_ctx);
  kk_std_core_vector__done(_ctx);
  kk_std_core_int__done(_ctx);
  kk_std_core_char__done(_ctx);
  kk_std_core_order__done(_ctx);
  kk_std_core_bool__done(_ctx);
  kk_std_core_exn__done(_ctx);
  kk_std_core_hnd__done(_ctx);
  kk_std_core_types__done(_ctx);
}
