// Koka generated module: std/text/regex, koka version: 3.1.3, platform: 64-bit
#include "std_text_regex.h"
/*---------------------------------------------------------------------------
  Copyright 2021, Microsoft Research, Daan Leijen.

  This is free software; you can redistribute it and/or modify it under the
  terms of the Apache License, Version 2.0. A copy of the License can be
  found in the LICENSE file at the root of this distribution.
---------------------------------------------------------------------------*/

#pragma clang diagnostic ignored "-Wdisabled-macro-expansion"
#define PCRE2_CODE_UNIT_WIDTH 8
#define PCRE2_STATIC
#include <pcre2.h>

/* -----------------------------------------------------------------------
  Init/Done
------------------------------------------------------------------------*/
#define KK_CUSTOM_INIT  kk_regex_custom_init
#define KK_CUSTOM_DONE  kk_regex_custom_done

static pcre2_general_context* gen_ctx;
static pcre2_compile_context* cmp_ctx;
static pcre2_match_context*   match_ctx;

static void* kk_pcre2_malloc( PCRE2_SIZE size, void* data ) {
  return kk_malloc( kk_to_ssize_t(size), kk_get_context() );
}
static void kk_pcre2_free( void* p, void* data ) {
  kk_free(p,kk_get_context());
}

static void kk_regex_custom_init( kk_context_t* ctx ) {
  gen_ctx = pcre2_general_context_create( &kk_pcre2_malloc, &kk_pcre2_free, NULL );
  if (gen_ctx != NULL) {
    match_ctx = pcre2_match_context_create( gen_ctx );
    cmp_ctx = pcre2_compile_context_create( gen_ctx );
    if (cmp_ctx != NULL) {
      pcre2_set_newline( cmp_ctx, PCRE2_NEWLINE_ANYCRLF );
      pcre2_set_bsr( cmp_ctx, PCRE2_BSR_ANYCRLF );
    }
  }
}

static void kk_regex_custom_done( kk_context_t* ctx ) {
  if (cmp_ctx != NULL) {
    pcre2_compile_context_free(cmp_ctx);
    cmp_ctx = NULL;
  }
  if (match_ctx != NULL) {
    pcre2_match_context_free(match_ctx);
    match_ctx = NULL;
  }
  if (gen_ctx != NULL) {
    pcre2_general_context_free(gen_ctx);
    gen_ctx = NULL;
  }
}


/* -----------------------------------------------------------------------
  Compile
------------------------------------------------------------------------*/

static void kk_regex_free( void* pre, kk_block_t* b, kk_context_t* ctx ) {
  kk_unused(ctx);
  pcre2_code* re = (pcre2_code*)pre;
  //kk_info_message( "free regex at %p\n", re );
  if (re != NULL) { pcre2_code_free(re); }
}

#define KK_REGEX_OPTIONS  (PCRE2_ALT_BSUX | PCRE2_EXTRA_ALT_BSUX | PCRE2_MATCH_UNSET_BACKREF /* javascript compat */ \
                          | PCRE2_NEVER_BACKSLASH_C | PCRE2_NEVER_UCP | PCRE2_UTF /* utf-8 safety */ \
                          )

static kk_box_t kk_regex_create( kk_string_t pat, bool ignore_case, bool multi_line, kk_context_t* ctx ) {
  kk_ssize_t len;
  const uint8_t* cpat = kk_string_buf_borrow( pat, &len, ctx );
  PCRE2_SIZE errofs = 0;
  int        errnum = 0;
  uint32_t   options = KK_REGEX_OPTIONS;
  if (ignore_case) options |= PCRE2_CASELESS;
  if (multi_line)  options |= PCRE2_MULTILINE;
  pcre2_code* re = pcre2_compile( cpat, PCRE2_ZERO_TERMINATED, options, &errnum, &errofs, cmp_ctx);
  //kk_info_message( "create regex: err:%i, at %p\n", (re==NULL ? 0 : errnum), re );
  kk_string_drop(pat,ctx);
  return kk_cptr_raw_box( &kk_regex_free, re, ctx );
}


/* -----------------------------------------------------------------------
  Match
------------------------------------------------------------------------*/

/*
static void kk_match_data_free( void* pmd, kk_block_t* b ) {
  kk_info_message( "free match_data\n" );
  pcre2_match_data* md = (pcre2_match_data*)pmd;
  if (md != NULL) pcre2_match_data_free(md);
}
*/

static kk_std_core_types__list kk_regex_exec_ex( pcre2_code* re, pcre2_match_data* match_data,
                                           kk_string_t str_borrow, const uint8_t* cstr, kk_ssize_t len, bool allow_empty,
                                           kk_ssize_t start, kk_ssize_t* mstart, kk_ssize_t* end, int* res, kk_context_t* ctx )
{
  // match
  kk_std_core_types__list hd  = kk_std_core_types__new_Nil(ctx);
  uint32_t options = 0;
  if (!allow_empty) options |= (PCRE2_NOTEMPTY_ATSTART | PCRE2_ANCHORED);
  int rc = pcre2_match( re, cstr, len, start, options, match_data, match_ctx );
  if (res != NULL) *res = rc;
  if (rc > 0) {
    // extract captures
    uint32_t    gcount = pcre2_get_ovector_count(match_data);
    PCRE2_SIZE* groups = pcre2_get_ovector_pointer(match_data);
    for( uint32_t i = gcount; i > 0; ) {
      i--;
      kk_ssize_t sstart = groups[i*2];       // on no-match, sstart and send == -1.
      kk_ssize_t send   = groups[i*2 + 1];
      kk_assert(send >= sstart);
      kk_std_core_sslice__sslice sslice = kk_std_core_sslice__new_Sslice( kk_string_dup(str_borrow,ctx), kk_integer_from_ssize_t(sstart,ctx), kk_integer_from_ssize_t(send - sstart,ctx), ctx );
      hd = kk_std_core_types__new_Cons(kk_reuse_null,0,kk_std_core_sslice__sslice_box(sslice,ctx), hd, ctx);
      if (i == 0) {
        if (mstart != NULL) { *mstart = sstart; }
        if (end    != NULL) { *end = send; }
      }
    }
  }
  return hd;
}


static kk_std_core_types__list kk_regex_exec( kk_box_t bre, kk_string_t str, kk_ssize_t start, kk_context_t* ctx )
{
  // unpack
  pcre2_match_data* match_data = NULL;
  kk_std_core_types__list res = kk_std_core_types__new_Nil(ctx);
  pcre2_code* re = (pcre2_code*)kk_cptr_raw_unbox_borrowed(bre,ctx);
  kk_ssize_t len = 0;
  const uint8_t* cstr = NULL;
  if (re == NULL) goto done;
  match_data = pcre2_match_data_create_from_pattern(re, gen_ctx);
  if (match_data==NULL) goto done;
  cstr = kk_string_buf_borrow(str, &len, ctx );

  // and match
  res = kk_regex_exec_ex( re, match_data, str, cstr, len, true, start, NULL, NULL, NULL, ctx );

done:
  if (match_data != NULL) {
    //kk_info_message( "free match data: %p\n", match_data );
    pcre2_match_data_free(match_data);
  }
  kk_string_drop(str,ctx);
  kk_box_drop(bre,ctx);
  return res;
}

static kk_std_core_types__list kk_regex_exec_all( kk_box_t bre, kk_string_t str, kk_ssize_t start, kk_ssize_t atmost, kk_context_t* ctx ) {
  // unpack
  if (atmost < 0) atmost = KK_SSIZE_MAX;
  pcre2_match_data* match_data = NULL;
  kk_std_core_types__list res = kk_std_core_types__new_Nil(ctx);
  pcre2_code* re = (pcre2_code*)kk_cptr_raw_unbox_borrowed(bre,ctx);
  if (re == NULL) goto done;
  match_data = pcre2_match_data_create_from_pattern(re, gen_ctx);
  if (match_data==NULL) goto done;
  {
    kk_ssize_t len;
    const uint8_t* cstr = kk_string_buf_borrow(str, &len, ctx );

    // and match
    kk_std_core_types__list* tail = NULL;
    bool allow_empty = true;
    int rc = 1;
    kk_ssize_t next = start;
    while( rc > 0 && start < len && atmost > 0) {
      atmost--;
      rc = 0;
      kk_ssize_t mstart = start;
      kk_std_core_types__list cap = kk_regex_exec_ex( re, match_data, str, cstr, len, allow_empty, start, &mstart, &next, &rc, ctx );
      if (rc > 0) {
        // found a match;
        // push string up to match, and the actual matched regex
        kk_std_core_sslice__sslice pre = kk_std_core_sslice__new_Sslice( kk_string_dup(str,ctx), kk_integer_from_ssize_t(start,ctx), kk_integer_from_ssize_t(mstart - start,ctx), ctx );
        kk_std_core_types__list   prelist = kk_std_core_types__new_Cons( kk_reuse_null, 0, kk_std_core_sslice__sslice_box(pre,ctx), kk_std_core_types__new_Nil(ctx), ctx );
        kk_std_core_types__list   capcons = kk_std_core_types__new_Cons( kk_reuse_null, 0, kk_std_core_types__list_box(cap,ctx), kk_std_core_types__new_Nil(ctx) /*tail*/, ctx );
        kk_std_core_types__list   cons = kk_std_core_types__new_Cons( kk_reuse_null, 0, kk_std_core_types__list_box(prelist,ctx), capcons, ctx );
        if (tail==NULL) res = cons;
                  else *tail = cons;
        tail = &kk_std_core_types__as_Cons(capcons,ctx)->tail;
        allow_empty = (next > start);
        start = next;
      }
      else if (rc <= 0 && !allow_empty) {
        // skip one character and try again
        // todo: handle cr/lf pairs better?
        const uint8_t* p = kk_utf8_next( cstr + start );
        start = (p - cstr);
        allow_empty = true;
        rc = 1;
      }
    }

    // push final string part as well and end the list
    kk_std_core_sslice__sslice post    = kk_std_core_sslice__new_Sslice( kk_string_dup(str,ctx), kk_integer_from_ssize_t(next,ctx), kk_integer_from_ssize_t(len - next,ctx), ctx );
    kk_std_core_types__list   postlist= kk_std_core_types__new_Cons( kk_reuse_null, 0, kk_std_core_sslice__sslice_box(post,ctx), kk_std_core_types__new_Nil(ctx), ctx );
    kk_std_core_types__list   cons    = kk_std_core_types__new_Cons( kk_reuse_null, 0, kk_std_core_types__list_box(postlist,ctx), kk_std_core_types__new_Nil(ctx), ctx );
    if (tail==NULL) res = cons;
              else *tail = cons;
  }

done:
  if (match_data != NULL) {
    //kk_info_message( "free match data: %p\n", match_data );
    pcre2_match_data_free(match_data);
  }
  kk_string_drop(str,ctx);
  kk_box_drop(bre,ctx);
  return res;
}

/*
kk_std_core_sslice__sslice kk_slice_upto( struct kk_std_core_Sslice slice1, struct kk_std_core_Sslice slice2, kk_context_t* ctx ) {
  kk_ssize_t start = slice1.start;
  kk_ssize_t len   = (slice1.start <= slice2.start ? slice2.start - slice1.start : -1);
  return kk_std_core_sslice__new_Sslice(slice1.str, start, len, ctx);
}
*/

kk_std_text_regex__regex kk_std_text_regex_regex_fs__copy(kk_std_text_regex__regex _this, kk_std_core_types__optional obj, kk_std_core_types__optional src, kk_context_t* _ctx) { /* (regex, obj : ? any, src : ? string) -> regex */ 
  kk_box_t _x_x456;
  if (kk_std_core_types__is_Optional(obj, _ctx)) {
    kk_box_t _box_x0 = obj._cons._Optional.value;
    kk_box_t _uniq_obj_107 = kk_box_unbox(_box_x0, _ctx);
    kk_box_dup(_uniq_obj_107, _ctx);
    kk_std_core_types__optional_drop(obj, _ctx);
    _x_x456 = _uniq_obj_107; /*any*/
  }
  else {
    kk_std_core_types__optional_drop(obj, _ctx);
    {
      kk_box_t _x = _this.obj;
      kk_box_dup(_x, _ctx);
      _x_x456 = _x; /*any*/
    }
  }
  kk_string_t _x_x457;
  if (kk_std_core_types__is_Optional(src, _ctx)) {
    kk_box_t _box_x1 = src._cons._Optional.value;
    kk_string_t _uniq_src_114 = kk_string_unbox(_box_x1);
    kk_string_dup(_uniq_src_114, _ctx);
    kk_std_core_types__optional_drop(src, _ctx);
    kk_std_text_regex__regex_drop(_this, _ctx);
    _x_x457 = _uniq_src_114; /*string*/
  }
  else {
    kk_std_core_types__optional_drop(src, _ctx);
    {
      kk_string_t _x_0 = _this.src;
      kk_string_dup(_x_0, _ctx);
      kk_std_text_regex__regex_drop(_this, _ctx);
      _x_x457 = _x_0; /*string*/
    }
  }
  return kk_std_text_regex__new_Regex(_x_x456, _x_x457, _ctx);
}

kk_box_t kk_std_text_regex_regex_create(kk_string_t source_0, bool ignore_case, bool multi_line, kk_context_t* _ctx) { /* (source : string, ignore-case : bool, multi-line : bool) -> any */ 
  return kk_regex_create(source_0,ignore_case,multi_line,kk_context());
}

kk_std_core_types__list kk_std_text_regex_regex_exec(kk_box_t regex_0, kk_string_t str, kk_ssize_t start, kk_context_t* _ctx) { /* (regex : any, str : string, start : ssize_t) -> list<sslice/sslice> */ 
  return kk_regex_exec(regex_0,str,start,kk_context());
}

kk_std_core_types__list kk_std_text_regex_regex_exec_all(kk_box_t regex_0, kk_string_t str, kk_ssize_t start, kk_ssize_t atmost, kk_context_t* _ctx) { /* (regex : any, str : string, start : ssize_t, atmost : ssize_t) -> list<list<sslice/sslice>> */ 
  return kk_regex_exec_all(regex_0,str,start,atmost,kk_context());
}
 
// Create a new regular expression. Takes two optional parameters. Set `ignoreCase` to `True`
// to ignore uppercase/lowercase distinction. If  `multiline` is set to `True`, then `^` and `$`
// match also the beginning and end of every line (instead of the entire input).

kk_std_text_regex__regex kk_std_text_regex_regex(kk_string_t regex_0, kk_std_core_types__optional ignorecase, kk_std_core_types__optional multiline, kk_context_t* _ctx) { /* (regex : string, ignorecase : ? bool, multiline : ? bool) -> regex */ 
  kk_box_t _x_x458;
  kk_string_t _x_x459 = kk_string_dup(regex_0, _ctx); /*string*/
  bool _x_x460;
  if (kk_std_core_types__is_Optional(ignorecase, _ctx)) {
    kk_box_t _box_x2 = ignorecase._cons._Optional.value;
    bool _uniq_ignorecase_161 = kk_bool_unbox(_box_x2);
    kk_std_core_types__optional_drop(ignorecase, _ctx);
    _x_x460 = _uniq_ignorecase_161; /*bool*/
  }
  else {
    kk_std_core_types__optional_drop(ignorecase, _ctx);
    _x_x460 = false; /*bool*/
  }
  bool _x_x461;
  if (kk_std_core_types__is_Optional(multiline, _ctx)) {
    kk_box_t _box_x3 = multiline._cons._Optional.value;
    bool _uniq_multiline_165 = kk_bool_unbox(_box_x3);
    kk_std_core_types__optional_drop(multiline, _ctx);
    _x_x461 = _uniq_multiline_165; /*bool*/
  }
  else {
    kk_std_core_types__optional_drop(multiline, _ctx);
    _x_x461 = false; /*bool*/
  }
  _x_x458 = kk_std_text_regex_regex_create(_x_x459, _x_x460, _x_x461, _ctx); /*any*/
  return kk_std_text_regex__new_Regex(_x_x458, regex_0, _ctx);
}

kk_std_text_regex__regex kk_std_text_regex_rx_nongroup;
 
// Match a regular expression `regex` over a string `s`.
// Matches at most `atmost` times (and matches all by default).
// Returns always an odd number of elements where every even
// element is a match and the odd ones the string parts between the
// matches.
// See also `find-all` and `strings`.

kk_std_core_types__list kk_std_text_regex_exec_all(kk_std_text_regex__regex regex_0, kk_string_t s, kk_std_core_types__optional atmost, kk_context_t* _ctx) { /* (regex : regex, s : string, atmost : ? int) -> list<list<sslice/sslice>> */ 
  kk_box_t _x_x470;
  {
    kk_box_t _x = regex_0.obj;
    kk_box_dup(_x, _ctx);
    kk_std_text_regex__regex_drop(regex_0, _ctx);
    _x_x470 = _x; /*any*/
  }
  kk_ssize_t _x_x471;
  kk_integer_t _x_x472;
  if (kk_std_core_types__is_Optional(atmost, _ctx)) {
    kk_box_t _box_x6 = atmost._cons._Optional.value;
    kk_integer_t _uniq_atmost_207 = kk_integer_unbox(_box_x6, _ctx);
    kk_integer_dup(_uniq_atmost_207, _ctx);
    kk_std_core_types__optional_drop(atmost, _ctx);
    _x_x472 = _uniq_atmost_207; /*int*/
  }
  else {
    kk_std_core_types__optional_drop(atmost, _ctx);
    _x_x472 = kk_integer_from_small(-1); /*int*/
  }
  _x_x471 = kk_std_core_int_ssize__t(_x_x472, _ctx); /*ssize_t*/
  return kk_std_text_regex_regex_exec_all(_x_x470, s, (KK_IZ(0)), _x_x471, _ctx);
}
 
// Return the full matched string of a capture group

kk_string_t kk_std_text_regex_captured(kk_std_core_types__list matched_0, kk_context_t* _ctx) { /* (matched : list<sslice/sslice>) -> string */ 
  if (kk_std_core_types__is_Cons(matched_0, _ctx)) {
    struct kk_std_core_types_Cons* _con_x473 = kk_std_core_types__as_Cons(matched_0, _ctx);
    kk_box_t _box_x7 = _con_x473->head;
    kk_std_core_sslice__sslice s = kk_std_core_sslice__sslice_unbox(_box_x7, KK_BORROWED, _ctx);
    kk_std_core_types__list _pat_3 = _con_x473->tail;
    if kk_likely(kk_datatype_ptr_is_unique(matched_0, _ctx)) {
      kk_std_core_types__list_drop(_pat_3, _ctx);
      kk_std_core_sslice__sslice_dup(s, _ctx);
      kk_box_drop(_box_x7, _ctx);
      kk_datatype_ptr_free(matched_0, _ctx);
    }
    else {
      kk_std_core_sslice__sslice_dup(s, _ctx);
      kk_datatype_ptr_decref(matched_0, _ctx);
    }
    return kk_std_core_sslice_string(s, _ctx);
  }
  {
    return kk_string_empty();
  }
}
 
// lifted local: @lift-captures@10521, captures, @spec-x10188
// specialized: std/core/list/@unroll-map@10010, on parameters @uniq-f@10183, using:
// @uniq-f@10183 = std/text/regex/captured

kk_std_core_types__list kk_std_text_regex__trmc_lift_captures_10522(kk_std_core_types__list _uniq_xs_10182, kk_std_core_types__cctx _acc, kk_context_t* _ctx) { /* (list<list<sslice/sslice>>, ctx<list<string>>) -> list<string> */ 
  if (kk_std_core_types__is_Cons(_uniq_xs_10182, _ctx)) {
    struct kk_std_core_types_Cons* _con_x475 = kk_std_core_types__as_Cons(_uniq_xs_10182, _ctx);
    kk_box_t _box_x8 = _con_x475->head;
    kk_std_core_types__list _uniq_xx_10186 = _con_x475->tail;
    kk_std_core_types__list _uniq_x_10185 = kk_std_core_types__list_unbox(_box_x8, KK_BORROWED, _ctx);
    kk_reuse_t _ru_x373 = kk_reuse_null; /*@reuse*/;
    if kk_likely(kk_datatype_ptr_is_unique(_uniq_xs_10182, _ctx)) {
      _ru_x373 = (kk_datatype_ptr_reuse(_uniq_xs_10182, _ctx));
    }
    else {
      kk_std_core_types__list_dup(_uniq_x_10185, _ctx);
      kk_std_core_types__list_dup(_uniq_xx_10186, _ctx);
      kk_datatype_ptr_decref(_uniq_xs_10182, _ctx);
    }
    kk_string_t _trmc_x10546;
    if (kk_std_core_types__is_Cons(_uniq_x_10185, _ctx)) {
      struct kk_std_core_types_Cons* _con_x476 = kk_std_core_types__as_Cons(_uniq_x_10185, _ctx);
      kk_box_t _box_x9 = _con_x476->head;
      kk_std_core_sslice__sslice s = kk_std_core_sslice__sslice_unbox(_box_x9, KK_BORROWED, _ctx);
      kk_std_core_types__list _pat_3 = _con_x476->tail;
      if kk_likely(kk_datatype_ptr_is_unique(_uniq_x_10185, _ctx)) {
        kk_std_core_types__list_drop(_pat_3, _ctx);
        kk_std_core_sslice__sslice_dup(s, _ctx);
        kk_box_drop(_box_x9, _ctx);
        kk_datatype_ptr_free(_uniq_x_10185, _ctx);
      }
      else {
        kk_std_core_sslice__sslice_dup(s, _ctx);
        kk_datatype_ptr_decref(_uniq_x_10185, _ctx);
      }
      _trmc_x10546 = kk_std_core_sslice_string(s, _ctx); /*string*/
    }
    else {
      _trmc_x10546 = kk_string_empty(); /*string*/
    }
    kk_std_core_types__list _trmc_x10547 = kk_datatype_null(); /*list<string>*/;
    kk_std_core_types__list _trmc_x10548 = kk_std_core_types__new_Cons(_ru_x373, 0, kk_string_box(_trmc_x10546), _trmc_x10547, _ctx); /*list<string>*/;
    kk_field_addr_t _b_x19_24 = kk_field_addr_create(&kk_std_core_types__as_Cons(_trmc_x10548, _ctx)->tail, _ctx); /*@field-addr<list<string>>*/;
    kk_std_core_types__cctx _brw_x368;
    kk_std_core_types__cctx _x_x478 = kk_std_core_types__cctx_dup(_acc, _ctx); /*ctx<list<string>>*/
    _brw_x368 = kk_cctx_extend_linear(_x_x478,(kk_std_core_types__list_box(_trmc_x10548, _ctx)),_b_x19_24,kk_context()); /*ctx<0>*/
    kk_std_core_types__list _brw_x369 = kk_std_text_regex__trmc_lift_captures_10522(_uniq_xx_10186, _brw_x368, _ctx); /*list<string>*/;
    kk_std_core_types__cctx_drop(_brw_x368, _ctx);
    return _brw_x369;
  }
  {
    kk_box_t _x_x479;
    kk_std_core_types__cctx _x_x480 = kk_std_core_types__cctx_dup(_acc, _ctx); /*ctx<list<string>>*/
    _x_x479 = kk_cctx_apply_linear(_x_x480,(kk_std_core_types__list_box(kk_std_core_types__new_Nil(_ctx), _ctx)),kk_context()); /*-1*/
    return kk_std_core_types__list_unbox(_x_x479, KK_OWNED, _ctx);
  }
}
 
// lifted local: @lift-captures@10521, captures, @spec-x10188
// specialized: std/core/list/@unroll-map@10010, on parameters @uniq-f@10183, using:
// @uniq-f@10183 = std/text/regex/captured

kk_std_core_types__list kk_std_text_regex__lift_captures_10522(kk_std_core_types__list _uniq_xs_10182_0, kk_context_t* _ctx) { /* (list<list<sslice/sslice>>) -> list<string> */ 
  kk_std_core_types__cctx _brw_x366 = kk_cctx_empty(kk_context()); /*ctx<0>*/;
  kk_std_core_types__list _brw_x367 = kk_std_text_regex__trmc_lift_captures_10522(_uniq_xs_10182_0, _brw_x366, _ctx); /*list<string>*/;
  kk_std_core_types__cctx_drop(_brw_x366, _ctx);
  return _brw_x367;
}
 
// lifted local: captures, @spec-x10181
// specialized: std/core/list/map, on parameters @uniq-f@10178, using:
// @uniq-f@10178 = std/text/regex/captured

kk_std_core_types__list kk_std_text_regex__lift_captures_10521(kk_std_core_types__list _uniq_xs_10177, kk_context_t* _ctx) { /* (list<list<sslice/sslice>>) -> list<string> */ 
  if (kk_std_core_types__is_Nil(_uniq_xs_10177, _ctx)) {
    return kk_std_core_types__new_Nil(_ctx);
  }
  {
    kk_std_core_types__cctx _brw_x364 = kk_cctx_empty(kk_context()); /*ctx<0>*/;
    kk_std_core_types__list _brw_x365 = kk_std_text_regex__trmc_lift_captures_10522(_uniq_xs_10177, _brw_x364, _ctx); /*list<string>*/;
    kk_std_core_types__cctx_drop(_brw_x364, _ctx);
    return _brw_x365;
  }
}
 
// Find a match for a regular expression.
// See also `exec`

kk_std_core_types__maybe kk_std_text_regex_find(kk_string_t s, kk_std_text_regex__regex r, kk_context_t* _ctx) { /* (s : string, r : regex) -> maybe<string> */ 
  kk_std_core_types__list _match_x363;
  kk_box_t _x_x481;
  {
    kk_box_t _x = r.obj;
    kk_box_dup(_x, _ctx);
    kk_std_text_regex__regex_drop(r, _ctx);
    _x_x481 = _x; /*any*/
  }
  _match_x363 = kk_std_text_regex_regex_exec(_x_x481, s, (KK_IZ(0)), _ctx); /*list<sslice/sslice>*/
  if (kk_std_core_types__is_Cons(_match_x363, _ctx)) {
    struct kk_std_core_types_Cons* _con_x482 = kk_std_core_types__as_Cons(_match_x363, _ctx);
    kk_box_t _box_x30 = _con_x482->head;
    kk_std_core_sslice__sslice m = kk_std_core_sslice__sslice_unbox(_box_x30, KK_BORROWED, _ctx);
    kk_std_core_types__list _pat_3 = _con_x482->tail;
    if kk_likely(kk_datatype_ptr_is_unique(_match_x363, _ctx)) {
      kk_std_core_types__list_drop(_pat_3, _ctx);
      kk_std_core_sslice__sslice_dup(m, _ctx);
      kk_box_drop(_box_x30, _ctx);
      kk_datatype_ptr_free(_match_x363, _ctx);
    }
    else {
      kk_std_core_sslice__sslice_dup(m, _ctx);
      kk_datatype_ptr_decref(_match_x363, _ctx);
    }
    kk_string_t _b_x31_32 = kk_std_core_sslice_string(m, _ctx); /*string*/;
    return kk_std_core_types__new_Just(kk_string_box(_b_x31_32), _ctx);
  }
  {
    return kk_std_core_types__new_Nothing(_ctx);
  }
}
 
// Does a regular expression pattern occur in a string `s`?
// (note: called `test` in javascript)

bool kk_std_text_regex_contains(kk_string_t s, kk_std_text_regex__regex r, kk_context_t* _ctx) { /* (s : string, r : regex) -> bool */ 
  kk_std_core_types__list list_10011;
  kk_box_t _x_x483;
  {
    kk_box_t _x = r.obj;
    kk_box_dup(_x, _ctx);
    kk_std_text_regex__regex_drop(r, _ctx);
    _x_x483 = _x; /*any*/
  }
  list_10011 = kk_std_text_regex_regex_exec(_x_x483, s, (KK_IZ(0)), _ctx); /*list<sslice/sslice>*/
  if (kk_std_core_types__is_Cons(list_10011, _ctx)) {
    struct kk_std_core_types_Cons* _con_x484 = kk_std_core_types__as_Cons(list_10011, _ctx);
    kk_box_t _box_x33 = _con_x484->head;
    kk_std_core_types__list _pat_1 = _con_x484->tail;
    if kk_likely(kk_datatype_ptr_is_unique(list_10011, _ctx)) {
      kk_std_core_types__list_drop(_pat_1, _ctx);
      kk_box_drop(_box_x33, _ctx);
      kk_datatype_ptr_free(list_10011, _ctx);
    }
    else {
      kk_datatype_ptr_decref(list_10011, _ctx);
    }
    return true;
  }
  {
    return false;
  }
}
 
// Filter only for the matched parts.

kk_std_core_types__list kk_std_text_regex__trmc_filter_matches(kk_std_core_types__list xs, kk_std_core_types__cctx _acc, kk_context_t* _ctx) { /* (xs : list<list<sslice/sslice>>, ctx<list<list<sslice/sslice>>>) -> list<list<sslice/sslice>> */ 
  kk__tailcall: ;
  if (kk_std_core_types__is_Cons(xs, _ctx)) {
    struct kk_std_core_types_Cons* _con_x485 = kk_std_core_types__as_Cons(xs, _ctx);
    kk_box_t _box_x34 = _con_x485->head;
    kk_std_core_types__list _pat_1 = _con_x485->tail;
    if (kk_std_core_types__is_Cons(_pat_1, _ctx)) {
      struct kk_std_core_types_Cons* _con_x486 = kk_std_core_types__as_Cons(_pat_1, _ctx);
      kk_box_t _box_x35 = _con_x486->head;
      kk_std_core_types__list _pat_0 = kk_std_core_types__list_unbox(_box_x34, KK_BORROWED, _ctx);
      kk_std_core_types__list mm = _con_x486->tail;
      kk_std_core_types__list m = kk_std_core_types__list_unbox(_box_x35, KK_BORROWED, _ctx);
      kk_reuse_t _ru_x378 = kk_reuse_null; /*@reuse*/;
      if kk_likely(kk_datatype_ptr_is_unique(xs, _ctx)) {
        if kk_likely(kk_datatype_ptr_is_unique(_pat_1, _ctx)) {
          kk_datatype_ptr_free(_pat_1, _ctx);
        }
        else {
          kk_std_core_types__list_dup(m, _ctx);
          kk_std_core_types__list_dup(mm, _ctx);
          kk_datatype_ptr_decref(_pat_1, _ctx);
        }
        kk_std_core_types__list_drop(_pat_0, _ctx);
        _ru_x378 = (kk_datatype_ptr_reuse(xs, _ctx));
      }
      else {
        kk_std_core_types__list_dup(m, _ctx);
        kk_std_core_types__list_dup(mm, _ctx);
        kk_datatype_ptr_decref(xs, _ctx);
      }
      kk_std_core_types__list _trmc_x10549 = kk_datatype_null(); /*list<list<sslice/sslice>>*/;
      kk_std_core_types__list _trmc_x10550 = kk_std_core_types__new_Cons(_ru_x378, 0, kk_std_core_types__list_box(m, _ctx), _trmc_x10549, _ctx); /*list<list<sslice/sslice>>*/;
      kk_field_addr_t _b_x45_50 = kk_field_addr_create(&kk_std_core_types__as_Cons(_trmc_x10550, _ctx)->tail, _ctx); /*@field-addr<list<list<sslice/sslice>>>*/;
      { // tailcall
        kk_std_core_types__cctx _x_x487 = kk_cctx_extend_linear(_acc,(kk_std_core_types__list_box(_trmc_x10550, _ctx)),_b_x45_50,kk_context()); /*ctx<0>*/
        xs = mm;
        _acc = _x_x487;
        goto kk__tailcall;
      }
    }
  }
  {
    kk_std_core_types__list_drop(xs, _ctx);
    kk_box_t _x_x488 = kk_cctx_apply_linear(_acc,(kk_std_core_types__list_box(kk_std_core_types__new_Nil(_ctx), _ctx)),kk_context()); /*-1*/
    return kk_std_core_types__list_unbox(_x_x488, KK_OWNED, _ctx);
  }
}
 
// Filter only for the matched parts.

kk_std_core_types__list kk_std_text_regex_filter_matches(kk_std_core_types__list xs_0, kk_context_t* _ctx) { /* (xs : list<list<sslice/sslice>>) -> list<list<sslice/sslice>> */ 
  kk_std_core_types__cctx _x_x489 = kk_cctx_empty(kk_context()); /*ctx<0>*/
  return kk_std_text_regex__trmc_filter_matches(xs_0, _x_x489, _ctx);
}
 
// lifted local: @lift-find-all@10523, find-all, @spec-x10200
// specialized: std/core/list/@unroll-map@10010, on parameters @uniq-f@10195, using:
// @uniq-f@10195 = std/text/regex/captured

kk_std_core_types__list kk_std_text_regex__trmc_lift_find_all_10524(kk_std_core_types__list _uniq_xs_10194, kk_std_core_types__cctx _acc, kk_context_t* _ctx) { /* (list<list<sslice/sslice>>, ctx<list<string>>) -> list<string> */ 
  if (kk_std_core_types__is_Cons(_uniq_xs_10194, _ctx)) {
    struct kk_std_core_types_Cons* _con_x490 = kk_std_core_types__as_Cons(_uniq_xs_10194, _ctx);
    kk_box_t _box_x56 = _con_x490->head;
    kk_std_core_types__list _uniq_xx_10198 = _con_x490->tail;
    kk_std_core_types__list _uniq_x_10197 = kk_std_core_types__list_unbox(_box_x56, KK_BORROWED, _ctx);
    kk_reuse_t _ru_x379 = kk_reuse_null; /*@reuse*/;
    if kk_likely(kk_datatype_ptr_is_unique(_uniq_xs_10194, _ctx)) {
      _ru_x379 = (kk_datatype_ptr_reuse(_uniq_xs_10194, _ctx));
    }
    else {
      kk_std_core_types__list_dup(_uniq_x_10197, _ctx);
      kk_std_core_types__list_dup(_uniq_xx_10198, _ctx);
      kk_datatype_ptr_decref(_uniq_xs_10194, _ctx);
    }
    kk_string_t _trmc_x10551;
    if (kk_std_core_types__is_Cons(_uniq_x_10197, _ctx)) {
      struct kk_std_core_types_Cons* _con_x491 = kk_std_core_types__as_Cons(_uniq_x_10197, _ctx);
      kk_box_t _box_x57 = _con_x491->head;
      kk_std_core_sslice__sslice s = kk_std_core_sslice__sslice_unbox(_box_x57, KK_BORROWED, _ctx);
      kk_std_core_types__list _pat_3 = _con_x491->tail;
      if kk_likely(kk_datatype_ptr_is_unique(_uniq_x_10197, _ctx)) {
        kk_std_core_types__list_drop(_pat_3, _ctx);
        kk_std_core_sslice__sslice_dup(s, _ctx);
        kk_box_drop(_box_x57, _ctx);
        kk_datatype_ptr_free(_uniq_x_10197, _ctx);
      }
      else {
        kk_std_core_sslice__sslice_dup(s, _ctx);
        kk_datatype_ptr_decref(_uniq_x_10197, _ctx);
      }
      _trmc_x10551 = kk_std_core_sslice_string(s, _ctx); /*string*/
    }
    else {
      _trmc_x10551 = kk_string_empty(); /*string*/
    }
    kk_std_core_types__list _trmc_x10552 = kk_datatype_null(); /*list<string>*/;
    kk_std_core_types__list _trmc_x10553 = kk_std_core_types__new_Cons(_ru_x379, 0, kk_string_box(_trmc_x10551), _trmc_x10552, _ctx); /*list<string>*/;
    kk_field_addr_t _b_x67_72 = kk_field_addr_create(&kk_std_core_types__as_Cons(_trmc_x10553, _ctx)->tail, _ctx); /*@field-addr<list<string>>*/;
    kk_std_core_types__cctx _brw_x361;
    kk_std_core_types__cctx _x_x493 = kk_std_core_types__cctx_dup(_acc, _ctx); /*ctx<list<string>>*/
    _brw_x361 = kk_cctx_extend_linear(_x_x493,(kk_std_core_types__list_box(_trmc_x10553, _ctx)),_b_x67_72,kk_context()); /*ctx<0>*/
    kk_std_core_types__list _brw_x362 = kk_std_text_regex__trmc_lift_find_all_10524(_uniq_xx_10198, _brw_x361, _ctx); /*list<string>*/;
    kk_std_core_types__cctx_drop(_brw_x361, _ctx);
    return _brw_x362;
  }
  {
    kk_box_t _x_x494;
    kk_std_core_types__cctx _x_x495 = kk_std_core_types__cctx_dup(_acc, _ctx); /*ctx<list<string>>*/
    _x_x494 = kk_cctx_apply_linear(_x_x495,(kk_std_core_types__list_box(kk_std_core_types__new_Nil(_ctx), _ctx)),kk_context()); /*-1*/
    return kk_std_core_types__list_unbox(_x_x494, KK_OWNED, _ctx);
  }
}
 
// lifted local: @lift-find-all@10523, find-all, @spec-x10200
// specialized: std/core/list/@unroll-map@10010, on parameters @uniq-f@10195, using:
// @uniq-f@10195 = std/text/regex/captured

kk_std_core_types__list kk_std_text_regex__lift_find_all_10524(kk_std_core_types__list _uniq_xs_10194_0, kk_context_t* _ctx) { /* (list<list<sslice/sslice>>) -> list<string> */ 
  kk_std_core_types__cctx _brw_x359 = kk_cctx_empty(kk_context()); /*ctx<0>*/;
  kk_std_core_types__list _brw_x360 = kk_std_text_regex__trmc_lift_find_all_10524(_uniq_xs_10194_0, _brw_x359, _ctx); /*list<string>*/;
  kk_std_core_types__cctx_drop(_brw_x359, _ctx);
  return _brw_x360;
}
 
// lifted local: find-all, @spec-x10193
// specialized: std/core/list/map, on parameters @uniq-f@10190, using:
// @uniq-f@10190 = std/text/regex/captured

kk_std_core_types__list kk_std_text_regex__lift_find_all_10523(kk_std_core_types__list _uniq_xs_10189, kk_context_t* _ctx) { /* (list<list<sslice/sslice>>) -> list<string> */ 
  if (kk_std_core_types__is_Nil(_uniq_xs_10189, _ctx)) {
    return kk_std_core_types__new_Nil(_ctx);
  }
  {
    kk_std_core_types__cctx _brw_x357 = kk_cctx_empty(kk_context()); /*ctx<0>*/;
    kk_std_core_types__list _brw_x358 = kk_std_text_regex__trmc_lift_find_all_10524(_uniq_xs_10189, _brw_x357, _ctx); /*list<string>*/;
    kk_std_core_types__cctx_drop(_brw_x357, _ctx);
    return _brw_x358;
  }
}
 
// Find all matches for a regular expression in a string.

kk_std_core_types__list kk_std_text_regex_find_all(kk_string_t s, kk_std_text_regex__regex r, kk_std_core_types__optional atmost, kk_context_t* _ctx) { /* (s : string, r : regex, atmost : ? int) -> list<string> */ 
  kk_std_core_types__list xs_10578;
  kk_box_t _x_x496;
  {
    kk_box_t _x = r.obj;
    kk_box_dup(_x, _ctx);
    kk_std_text_regex__regex_drop(r, _ctx);
    _x_x496 = _x; /*any*/
  }
  kk_ssize_t _x_x497;
  kk_integer_t _x_x498;
  if (kk_std_core_types__is_Optional(atmost, _ctx)) {
    kk_box_t _box_x78 = atmost._cons._Optional.value;
    kk_integer_t _uniq_atmost_355 = kk_integer_unbox(_box_x78, _ctx);
    kk_integer_dup(_uniq_atmost_355, _ctx);
    kk_std_core_types__optional_drop(atmost, _ctx);
    _x_x498 = _uniq_atmost_355; /*int*/
  }
  else {
    kk_std_core_types__optional_drop(atmost, _ctx);
    _x_x498 = kk_integer_from_small(-1); /*int*/
  }
  _x_x497 = kk_std_core_int_ssize__t(_x_x498, _ctx); /*ssize_t*/
  xs_10578 = kk_std_text_regex_regex_exec_all(_x_x496, s, (KK_IZ(0)), _x_x497, _ctx); /*list<list<sslice/sslice>>*/
  kk_std_core_types__list xs_10015;
  kk_std_core_types__cctx _x_x499 = kk_cctx_empty(kk_context()); /*ctx<0>*/
  xs_10015 = kk_std_text_regex__trmc_filter_matches(xs_10578, _x_x499, _ctx); /*list<list<sslice/sslice>>*/
  return kk_std_text_regex__lift_find_all_10523(xs_10015, _ctx);
}
 
// monadic lift

kk_string_t kk_std_text_regex__mlift_concat_replace_10573(kk_std_core_types__list acc, kk_std_core_types__list mm, kk_std_core_types__list pre, kk_function_t repl, kk_string_t _y_x10559, kk_context_t* _ctx) { /* forall<e> (acc : list<string>, mm : list<list<sslice/sslice>>, pre : list<sslice/sslice>, repl : (list<sslice/sslice>) -> e string, string) -> e string */ 
  kk_string_t _b_x80_86;
  if (kk_std_core_types__is_Cons(pre, _ctx)) {
    struct kk_std_core_types_Cons* _con_x500 = kk_std_core_types__as_Cons(pre, _ctx);
    kk_box_t _box_x79 = _con_x500->head;
    kk_std_core_sslice__sslice s = kk_std_core_sslice__sslice_unbox(_box_x79, KK_BORROWED, _ctx);
    kk_std_core_types__list _pat_3 = _con_x500->tail;
    if kk_likely(kk_datatype_ptr_is_unique(pre, _ctx)) {
      kk_std_core_types__list_drop(_pat_3, _ctx);
      kk_std_core_sslice__sslice_dup(s, _ctx);
      kk_box_drop(_box_x79, _ctx);
      kk_datatype_ptr_free(pre, _ctx);
    }
    else {
      kk_std_core_sslice__sslice_dup(s, _ctx);
      kk_datatype_ptr_decref(pre, _ctx);
    }
    _b_x80_86 = kk_std_core_sslice_string(s, _ctx); /*string*/
  }
  else {
    _b_x80_86 = kk_string_empty(); /*string*/
  }
  kk_std_core_types__list _x_x502;
  kk_std_core_types__list _x_x503 = kk_std_core_types__new_Cons(kk_reuse_null, 0, kk_string_box(_b_x80_86), acc, _ctx); /*list<82>*/
  _x_x502 = kk_std_core_types__new_Cons(kk_reuse_null, 0, kk_string_box(_y_x10559), _x_x503, _ctx); /*list<82>*/
  return kk_std_text_regex_concat_replace(mm, repl, _x_x502, _ctx);
}


// lift anonymous function
struct kk_std_text_regex_concat_replace_fun508__t {
  struct kk_function_s _base;
  kk_std_core_types__list acc_0;
  kk_std_core_types__list mm_0;
  kk_std_core_types__list pre_0;
  kk_function_t repl_0;
};
static kk_box_t kk_std_text_regex_concat_replace_fun508(kk_function_t _fself, kk_box_t _b_x91, kk_context_t* _ctx);
static kk_function_t kk_std_text_regex_new_concat_replace_fun508(kk_std_core_types__list acc_0, kk_std_core_types__list mm_0, kk_std_core_types__list pre_0, kk_function_t repl_0, kk_context_t* _ctx) {
  struct kk_std_text_regex_concat_replace_fun508__t* _self = kk_function_alloc_as(struct kk_std_text_regex_concat_replace_fun508__t, 5, _ctx);
  _self->_base.fun = kk_kkfun_ptr_box(&kk_std_text_regex_concat_replace_fun508, kk_context());
  _self->acc_0 = acc_0;
  _self->mm_0 = mm_0;
  _self->pre_0 = pre_0;
  _self->repl_0 = repl_0;
  return kk_datatype_from_base(&_self->_base, kk_context());
}

static kk_box_t kk_std_text_regex_concat_replace_fun508(kk_function_t _fself, kk_box_t _b_x91, kk_context_t* _ctx) {
  struct kk_std_text_regex_concat_replace_fun508__t* _self = kk_function_as(struct kk_std_text_regex_concat_replace_fun508__t*, _fself, _ctx);
  kk_std_core_types__list acc_0 = _self->acc_0; /* list<string> */
  kk_std_core_types__list mm_0 = _self->mm_0; /* list<list<sslice/sslice>> */
  kk_std_core_types__list pre_0 = _self->pre_0; /* list<sslice/sslice> */
  kk_function_t repl_0 = _self->repl_0; /* (list<sslice/sslice>) -> 446 string */
  kk_drop_match(_self, {kk_std_core_types__list_dup(acc_0, _ctx);kk_std_core_types__list_dup(mm_0, _ctx);kk_std_core_types__list_dup(pre_0, _ctx);kk_function_dup(repl_0, _ctx);}, {}, _ctx)
  kk_string_t _y_x10559_0_111 = kk_string_unbox(_b_x91); /*string*/;
  kk_string_t _x_x509 = kk_std_text_regex__mlift_concat_replace_10573(acc_0, mm_0, pre_0, repl_0, _y_x10559_0_111, _ctx); /*string*/
  return kk_string_box(_x_x509);
}

kk_string_t kk_std_text_regex_concat_replace(kk_std_core_types__list matches, kk_function_t repl_0, kk_std_core_types__list acc_0, kk_context_t* _ctx) { /* forall<e> (matches : list<list<sslice/sslice>>, repl : (list<sslice/sslice>) -> e string, acc : list<string>) -> e string */ 
  kk__tailcall: ;
  if (kk_std_core_types__is_Cons(matches, _ctx)) {
    struct kk_std_core_types_Cons* _con_x504 = kk_std_core_types__as_Cons(matches, _ctx);
    kk_box_t _box_x88 = _con_x504->head;
    kk_std_core_types__list _pat_0 = _con_x504->tail;
    if (kk_std_core_types__is_Cons(_pat_0, _ctx)) {
      struct kk_std_core_types_Cons* _con_x505 = kk_std_core_types__as_Cons(_pat_0, _ctx);
      kk_box_t _box_x89 = _con_x505->head;
      kk_std_core_types__list pre_0 = kk_std_core_types__list_unbox(_box_x88, KK_BORROWED, _ctx);
      kk_std_core_types__list mm_0 = _con_x505->tail;
      kk_std_core_types__list m = kk_std_core_types__list_unbox(_box_x89, KK_BORROWED, _ctx);
      kk_reuse_t _ru_x383 = kk_reuse_null; /*@reuse*/;
      kk_reuse_t _ru_x382 = kk_reuse_null; /*@reuse*/;
      if kk_likely(kk_datatype_ptr_is_unique(matches, _ctx)) {
        if kk_likely(kk_datatype_ptr_is_unique(_pat_0, _ctx)) {
          _ru_x382 = (kk_datatype_ptr_reuse(_pat_0, _ctx));
        }
        else {
          kk_std_core_types__list_dup(m, _ctx);
          kk_std_core_types__list_dup(mm_0, _ctx);
          kk_datatype_ptr_decref(_pat_0, _ctx);
        }
        _ru_x383 = (kk_datatype_ptr_reuse(matches, _ctx));
      }
      else {
        kk_std_core_types__list_dup(m, _ctx);
        kk_std_core_types__list_dup(mm_0, _ctx);
        kk_std_core_types__list_dup(pre_0, _ctx);
        kk_datatype_ptr_decref(matches, _ctx);
      }
      kk_string_t x_10579;
      kk_function_t _x_x506 = kk_function_dup(repl_0, _ctx); /*(list<sslice/sslice>) -> 446 string*/
      x_10579 = kk_function_call(kk_string_t, (kk_function_t, kk_std_core_types__list, kk_context_t*), _x_x506, (_x_x506, m, _ctx), _ctx); /*string*/
      if (kk_yielding(kk_context())) {
        kk_reuse_drop(_ru_x382,kk_context());
        kk_reuse_drop(_ru_x383,kk_context());
        kk_string_drop(x_10579, _ctx);
        kk_box_t _x_x507 = kk_std_core_hnd_yield_extend(kk_std_text_regex_new_concat_replace_fun508(acc_0, mm_0, pre_0, repl_0, _ctx), _ctx); /*2401*/
        return kk_string_unbox(_x_x507);
      }
      {
        kk_string_t _b_x93_109;
        if (kk_std_core_types__is_Cons(pre_0, _ctx)) {
          struct kk_std_core_types_Cons* _con_x510 = kk_std_core_types__as_Cons(pre_0, _ctx);
          kk_box_t _box_x92 = _con_x510->head;
          kk_std_core_sslice__sslice s_0 = kk_std_core_sslice__sslice_unbox(_box_x92, KK_BORROWED, _ctx);
          kk_std_core_types__list _pat_3_0 = _con_x510->tail;
          if kk_likely(kk_datatype_ptr_is_unique(pre_0, _ctx)) {
            kk_std_core_types__list_drop(_pat_3_0, _ctx);
            kk_std_core_sslice__sslice_dup(s_0, _ctx);
            kk_box_drop(_box_x92, _ctx);
            kk_datatype_ptr_free(pre_0, _ctx);
          }
          else {
            kk_std_core_sslice__sslice_dup(s_0, _ctx);
            kk_datatype_ptr_decref(pre_0, _ctx);
          }
          _b_x93_109 = kk_std_core_sslice_string(s_0, _ctx); /*string*/
        }
        else {
          _b_x93_109 = kk_string_empty(); /*string*/
        }
        { // tailcall
          kk_std_core_types__list _x_x512;
          kk_std_core_types__list _x_x513 = kk_std_core_types__new_Cons(_ru_x383, 0, kk_string_box(_b_x93_109), acc_0, _ctx); /*list<82>*/
          _x_x512 = kk_std_core_types__new_Cons(_ru_x382, 0, kk_string_box(x_10579), _x_x513, _ctx); /*list<82>*/
          matches = mm_0;
          acc_0 = _x_x512;
          goto kk__tailcall;
        }
      }
    }
  }
  if (kk_std_core_types__is_Cons(matches, _ctx)) {
    struct kk_std_core_types_Cons* _con_x514 = kk_std_core_types__as_Cons(matches, _ctx);
    kk_box_t _box_x97 = _con_x514->head;
    kk_std_core_types__list _pat_2_0_0 = _con_x514->tail;
    kk_std_core_types__list post = kk_std_core_types__list_unbox(_box_x97, KK_BORROWED, _ctx);
    kk_function_drop(repl_0, _ctx);
    kk_reuse_t _ru_x385 = kk_reuse_null; /*@reuse*/;
    if kk_likely(kk_datatype_ptr_is_unique(matches, _ctx)) {
      _ru_x385 = (kk_datatype_ptr_reuse(matches, _ctx));
    }
    else {
      kk_std_core_types__list_dup(post, _ctx);
      kk_datatype_ptr_decref(matches, _ctx);
    }
    kk_string_t _b_x99_101;
    if (kk_std_core_types__is_Cons(post, _ctx)) {
      struct kk_std_core_types_Cons* _con_x515 = kk_std_core_types__as_Cons(post, _ctx);
      kk_box_t _box_x98 = _con_x515->head;
      kk_std_core_sslice__sslice s_0_0 = kk_std_core_sslice__sslice_unbox(_box_x98, KK_BORROWED, _ctx);
      kk_std_core_types__list _pat_3_1 = _con_x515->tail;
      if kk_likely(kk_datatype_ptr_is_unique(post, _ctx)) {
        kk_std_core_types__list_drop(_pat_3_1, _ctx);
        kk_std_core_sslice__sslice_dup(s_0_0, _ctx);
        kk_box_drop(_box_x98, _ctx);
        kk_datatype_ptr_free(post, _ctx);
      }
      else {
        kk_std_core_sslice__sslice_dup(s_0_0, _ctx);
        kk_datatype_ptr_decref(post, _ctx);
      }
      _b_x99_101 = kk_std_core_sslice_string(s_0_0, _ctx); /*string*/
    }
    else {
      _b_x99_101 = kk_string_empty(); /*string*/
    }
    kk_std_core_types__list xs_10565 = kk_std_core_types__new_Cons(_ru_x385, 0, kk_string_box(_b_x99_101), acc_0, _ctx); /*list<string>*/;
    kk_std_core_types__list xs_0_10122;
    if (kk_std_core_types__is_Nil(xs_10565, _ctx)) {
      xs_0_10122 = kk_std_core_types__new_Nil(_ctx); /*list<string>*/
    }
    else {
      xs_0_10122 = kk_std_core_list__unroll_reverse_acc_10003(kk_std_core_types__new_Nil(_ctx), xs_10565, _ctx); /*list<string>*/
    }
    if (kk_std_core_types__is_Nil(xs_0_10122, _ctx)) {
      return kk_string_empty();
    }
    {
      struct kk_std_core_types_Cons* _con_x518 = kk_std_core_types__as_Cons(xs_0_10122, _ctx);
      kk_box_t _box_x103 = _con_x518->head;
      kk_std_core_types__list _pat_1_2 = _con_x518->tail;
      if (kk_std_core_types__is_Nil(_pat_1_2, _ctx)) {
        kk_string_t x_0 = kk_string_unbox(_box_x103);
        if kk_likely(kk_datatype_ptr_is_unique(xs_0_10122, _ctx)) {
          kk_datatype_ptr_free(xs_0_10122, _ctx);
        }
        else {
          kk_string_dup(x_0, _ctx);
          kk_datatype_ptr_decref(xs_0_10122, _ctx);
        }
        return x_0;
      }
    }
    {
      struct kk_std_core_types_Cons* _con_x519 = kk_std_core_types__as_Cons(xs_0_10122, _ctx);
      kk_box_t _box_x104 = _con_x519->head;
      kk_std_core_types__list _pat_3_0_0 = _con_x519->tail;
      struct kk_std_core_types_Cons* _con_x520 = kk_std_core_types__as_Cons(_pat_3_0_0, _ctx);
      kk_box_t _box_x105 = _con_x520->head;
      kk_std_core_types__list _pat_4_0_0 = _con_x520->tail;
      if (kk_std_core_types__is_Nil(_pat_4_0_0, _ctx)) {
        kk_string_t x_0_0 = kk_string_unbox(_box_x104);
        kk_string_t y = kk_string_unbox(_box_x105);
        if kk_likely(kk_datatype_ptr_is_unique(xs_0_10122, _ctx)) {
          if kk_likely(kk_datatype_ptr_is_unique(_pat_3_0_0, _ctx)) {
            kk_datatype_ptr_free(_pat_3_0_0, _ctx);
          }
          else {
            kk_string_dup(y, _ctx);
            kk_datatype_ptr_decref(_pat_3_0_0, _ctx);
          }
          kk_datatype_ptr_free(xs_0_10122, _ctx);
        }
        else {
          kk_string_dup(x_0_0, _ctx);
          kk_string_dup(y, _ctx);
          kk_datatype_ptr_decref(xs_0_10122, _ctx);
        }
        return kk_std_core_types__lp__plus__plus__rp_(x_0_0, y, _ctx);
      }
    }
    {
      kk_vector_t _x_x521 = kk_std_core_vector_unvlist(xs_0_10122, _ctx); /*vector<911>*/
      return kk_string_join(_x_x521,kk_context());
    }
  }
  {
    kk_function_drop(repl_0, _ctx);
    return kk_std_core_list_reverse_join(acc_0, _ctx);
  }
}
 
// Replace the all occurrences of `regex` by the result of the replacement fun `repl` in a string `s`.

kk_string_t kk_std_text_regex_replace_all(kk_string_t s, kk_std_text_regex__regex r, kk_function_t repl, kk_std_core_types__optional atmost, kk_context_t* _ctx) { /* forall<e> (s : string, r : regex, repl : (list<sslice/sslice>) -> e string, atmost : ? int) -> e string */ 
  kk_std_core_types__list _x_x522;
  kk_box_t _x_x523;
  {
    kk_box_t _x = r.obj;
    kk_box_dup(_x, _ctx);
    kk_std_text_regex__regex_drop(r, _ctx);
    _x_x523 = _x; /*any*/
  }
  kk_ssize_t _x_x524;
  kk_integer_t _x_x525;
  if (kk_std_core_types__is_Optional(atmost, _ctx)) {
    kk_box_t _box_x112 = atmost._cons._Optional.value;
    kk_integer_t _uniq_atmost_454 = kk_integer_unbox(_box_x112, _ctx);
    kk_integer_dup(_uniq_atmost_454, _ctx);
    kk_std_core_types__optional_drop(atmost, _ctx);
    _x_x525 = _uniq_atmost_454; /*int*/
  }
  else {
    kk_std_core_types__optional_drop(atmost, _ctx);
    _x_x525 = kk_integer_from_small(-1); /*int*/
  }
  _x_x524 = kk_std_core_int_ssize__t(_x_x525, _ctx); /*ssize_t*/
  _x_x522 = kk_std_text_regex_regex_exec_all(_x_x523, s, (KK_IZ(0)), _x_x524, _ctx); /*list<list<sslice/sslice>>*/
  return kk_std_text_regex_concat_replace(_x_x522, repl, kk_std_core_types__new_Nil(_ctx), _ctx);
}
 
// lifted local: replace-captures, @spec-x10243
// specialized: std/text/regex/concat-replace, on parameters @uniq-repl@10202, using:
// @uniq-repl@10202 = fn(cap: (list :: V -> V)<sslice/sslice>){
//   match (cap) {
//     ((std/core/types/Cons(((@skip std/core/sslice/Sslice((@pat@1@1: string) : string, (@pat@2@1: int) : int, (@pat@3@0: int) : int) : sslice/sslice ) as @pat@0@3: sslice/sslice) : sslice/sslice, ((std/core/types/Cons(((@skip std/core/sslice/Sslice((@pat@5: string) : string, (@pat@6: int) : int, (@pat@7: int) : int) : sslice/sslice ) as digit: sslice/sslice) : sslice/sslice, ((std/core/types/Cons(((@skip std/core/sslice/Sslice((@pat@9: string) : string, (@pat@10: int) : int, (@pat@11: int) : int) : sslice/sslice ) as amp: sslice/sslice) : sslice/sslice, ((std/core/types/Cons(((@skip std/core/sslice/Sslice((@pat@13: string) : string, (@pat@14: int) : int, (@pat@15: int) : int) : sslice/sslice ) as dollar: sslice/sslice) : sslice/sslice, ((std/core/types/Nil() : (list :: V -> V)<sslice/sslice> ) as @pat@16: (list :: V -> V)<sslice/sslice>) : (list :: V -> V)<sslice/sslice>) : (list :: V -> V)<sslice/sslice> ) as @pat@12: (list :: V -> V)<sslice/sslice>) : (list :: V -> V)<sslice/sslice>) : (list :: V -> V)<sslice/sslice> ) as @pat@8: (list :: V -> V)<sslice/sslice>) : (list :: V -> V)<sslice/sslice>) : (list :: V -> V)<sslice/sslice> ) as @pat@4@0: (list :: V -> V)<sslice/sslice>) : (list :: V -> V)<sslice/sslice>) : (list :: V -> V)<sslice/sslice> ) as @pat@4: ((list :: V -> V)<sslice/sslice>))
//        -> (match ((std/core/int/(>=)((match (dollar) {
//           ((@skip std/core/sslice/Sslice((@pat@0@4: string) : string, (@x@0: int) : int, (@pat@1@2: int) : int) : sslice/sslice ) as @pat@17: sslice/sslice)
//              -> @x@0;
//         }), 0))) {
//         ((std/core/types/True() : bool ) as @pat@17@0: bool)
//            -> "$";
//         ((@skip std/core/types/False() : bool ) as @pat@18: bool)
//            -> val grp : int
//                     = (match ((std/core/int/(>=)((match (amp) {
//                         ((@skip std/core/sslice/Sslice((@pat@0@5: string) : string, (@x@1: int) : int, (@pat@1@3: int) : int) : sslice/sslice ) as @pat@19: sslice/sslice)
//                            -> @x@1;
//                       }), 0))) {
//                       ((std/core/types/True() : bool ) as @pat@19@0: bool)
//                          -> 0;
//                       ((@skip std/core/types/False() : bool ) as @pat@20: bool)
//                          -> val s@1@10032 : string;
//                             val m@10030 : (maybe :: V -> V)<int>;
//                         (match (m@10030) {
//                           ((std/core/types/Nothing() : (maybe :: V -> V)<int> ) as @pat@21: ((maybe :: V -> V)<int>))
//                              -> 0;
//                           ((@skip std/core/types/Just((x: int) : int) : (maybe :: V -> V)<int> ) as @pat@0@6: ((maybe :: V -> V)<int>))
//                              -> x;
//                         });
//                     });
//           (match ((match (caps) {
//             ((std/core/types/Nil() : (list :: V -> V)<sslice/sslice> ) as @pat@4@1: ((list :: V -> V)<sslice/sslice>))
//                -> std/core/types/Nothing<sslice/sslice>;
//             (@pat@23: ((list :: V -> V)<sslice/sslice>))
//                -> std/core/list/@unroll-index@10029<sslice/sslice>(caps, grp);
//           })) {
//             ((std/core/types/Nothing() : (maybe :: V -> V)<sslice/sslice> ) as @pat@21@0: ((maybe :: V -> V)<sslice/sslice>))
//                -> "";
//             ((@skip std/core/types/Just(((@skip std/core/sslice/Sslice((@pat@23@0: string) : string, (@pat@24: int) : int, (@pat@25: int) : int) : sslice/sslice ) as s@2: sslice/sslice) : sslice/sslice) : (maybe :: V -> V)<sslice/sslice> ) as @pat@22@0: ((maybe :: V -> V)<sslice/sslice>))
//                -> std/core/sslice/string(s@2);
//           });
//       });
//     (@pat@26: ((list :: V -> V)<sslice/sslice>))
//        -> "$";
//   };
// }

kk_string_t kk_std_text_regex__lift_replace_captures_10525(kk_std_core_types__list caps, kk_std_core_types__list _uniq_matches_10201, kk_std_core_types__list _uniq_acc_10203, kk_context_t* _ctx) { /* (caps : list<sslice/sslice>, list<list<sslice/sslice>>, list<string>) -> string */ 
  kk__tailcall: ;
  if (kk_std_core_types__is_Cons(_uniq_matches_10201, _ctx)) {
    struct kk_std_core_types_Cons* _con_x526 = kk_std_core_types__as_Cons(_uniq_matches_10201, _ctx);
    kk_box_t _box_x113 = _con_x526->head;
    kk_std_core_types__list _uniq_pat_0_10206 = _con_x526->tail;
    if (kk_std_core_types__is_Cons(_uniq_pat_0_10206, _ctx)) {
      struct kk_std_core_types_Cons* _con_x527 = kk_std_core_types__as_Cons(_uniq_pat_0_10206, _ctx);
      kk_box_t _box_x114 = _con_x527->head;
      kk_std_core_types__list _uniq_pre_10205 = kk_std_core_types__list_unbox(_box_x113, KK_BORROWED, _ctx);
      kk_std_core_types__list _uniq_mm_10208 = _con_x527->tail;
      kk_std_core_types__list _uniq_m_10207 = kk_std_core_types__list_unbox(_box_x114, KK_BORROWED, _ctx);
      kk_reuse_t _ru_x391 = kk_reuse_null; /*@reuse*/;
      kk_reuse_t _ru_x390 = kk_reuse_null; /*@reuse*/;
      if kk_likely(kk_datatype_ptr_is_unique(_uniq_matches_10201, _ctx)) {
        if kk_likely(kk_datatype_ptr_is_unique(_uniq_pat_0_10206, _ctx)) {
          _ru_x390 = (kk_datatype_ptr_reuse(_uniq_pat_0_10206, _ctx));
        }
        else {
          kk_std_core_types__list_dup(_uniq_m_10207, _ctx);
          kk_std_core_types__list_dup(_uniq_mm_10208, _ctx);
          kk_datatype_ptr_decref(_uniq_pat_0_10206, _ctx);
        }
        _ru_x391 = (kk_datatype_ptr_reuse(_uniq_matches_10201, _ctx));
      }
      else {
        kk_std_core_types__list_dup(_uniq_m_10207, _ctx);
        kk_std_core_types__list_dup(_uniq_mm_10208, _ctx);
        kk_std_core_types__list_dup(_uniq_pre_10205, _ctx);
        kk_datatype_ptr_decref(_uniq_matches_10201, _ctx);
      }
      kk_string_t _b_x125_127;
      if (kk_std_core_types__is_Cons(_uniq_m_10207, _ctx)) {
        struct kk_std_core_types_Cons* _con_x529 = kk_std_core_types__as_Cons(_uniq_m_10207, _ctx);
        kk_box_t _box_x115 = _con_x529->head;
        kk_std_core_types__list _pat_4_0 = _con_x529->tail;
        if (kk_std_core_types__is_Cons(_pat_4_0, _ctx)) {
          struct kk_std_core_types_Cons* _con_x530 = kk_std_core_types__as_Cons(_pat_4_0, _ctx);
          kk_std_core_sslice__sslice _pat_0_3 = kk_std_core_sslice__sslice_unbox(_box_x115, KK_BORROWED, _ctx);
          kk_box_t _box_x116 = _con_x530->head;
          kk_std_core_types__list _pat_8 = _con_x530->tail;
          if (kk_std_core_types__is_Cons(_pat_8, _ctx)) {
            struct kk_std_core_types_Cons* _con_x531 = kk_std_core_types__as_Cons(_pat_8, _ctx);
            kk_std_core_sslice__sslice digit = kk_std_core_sslice__sslice_unbox(_box_x116, KK_BORROWED, _ctx);
            kk_box_t _box_x117 = _con_x531->head;
            kk_std_core_types__list _pat_12 = _con_x531->tail;
            if (kk_std_core_types__is_Cons(_pat_12, _ctx)) {
              struct kk_std_core_types_Cons* _con_x532 = kk_std_core_types__as_Cons(_pat_12, _ctx);
              kk_std_core_sslice__sslice amp = kk_std_core_sslice__sslice_unbox(_box_x117, KK_BORROWED, _ctx);
              kk_box_t _box_x118 = _con_x532->head;
              kk_std_core_types__list _pat_16 = _con_x532->tail;
              if (kk_std_core_types__is_Nil(_pat_16, _ctx)) {
                kk_std_core_sslice__sslice dollar = kk_std_core_sslice__sslice_unbox(_box_x118, KK_BORROWED, _ctx);
                if kk_likely(kk_datatype_ptr_is_unique(_uniq_m_10207, _ctx)) {
                  if kk_likely(kk_datatype_ptr_is_unique(_pat_4_0, _ctx)) {
                    if kk_likely(kk_datatype_ptr_is_unique(_pat_8, _ctx)) {
                      if kk_likely(kk_datatype_ptr_is_unique(_pat_12, _ctx)) {
                        kk_std_core_sslice__sslice_dup(dollar, _ctx);
                        kk_box_drop(_box_x118, _ctx);
                        kk_datatype_ptr_free(_pat_12, _ctx);
                      }
                      else {
                        kk_std_core_sslice__sslice_dup(dollar, _ctx);
                        kk_datatype_ptr_decref(_pat_12, _ctx);
                      }
                      kk_std_core_sslice__sslice_dup(amp, _ctx);
                      kk_box_drop(_box_x117, _ctx);
                      kk_datatype_ptr_free(_pat_8, _ctx);
                    }
                    else {
                      kk_std_core_sslice__sslice_dup(amp, _ctx);
                      kk_std_core_sslice__sslice_dup(dollar, _ctx);
                      kk_datatype_ptr_decref(_pat_8, _ctx);
                    }
                    kk_std_core_sslice__sslice_dup(digit, _ctx);
                    kk_box_drop(_box_x116, _ctx);
                    kk_datatype_ptr_free(_pat_4_0, _ctx);
                  }
                  else {
                    kk_std_core_sslice__sslice_dup(amp, _ctx);
                    kk_std_core_sslice__sslice_dup(digit, _ctx);
                    kk_std_core_sslice__sslice_dup(dollar, _ctx);
                    kk_datatype_ptr_decref(_pat_4_0, _ctx);
                  }
                  kk_box_drop(_box_x115, _ctx);
                  kk_datatype_ptr_free(_uniq_m_10207, _ctx);
                }
                else {
                  kk_std_core_sslice__sslice_dup(amp, _ctx);
                  kk_std_core_sslice__sslice_dup(digit, _ctx);
                  kk_std_core_sslice__sslice_dup(dollar, _ctx);
                  kk_datatype_ptr_decref(_uniq_m_10207, _ctx);
                }
                bool _match_x348;
                kk_integer_t _brw_x354;
                {
                  kk_integer_t _x_0 = dollar.start;
                  kk_integer_dup(_x_0, _ctx);
                  kk_std_core_sslice__sslice_drop(dollar, _ctx);
                  _brw_x354 = _x_0; /*int*/
                }
                bool _brw_x355 = kk_integer_gte_borrow(_brw_x354,(kk_integer_from_small(0)),kk_context()); /*bool*/;
                kk_integer_drop(_brw_x354, _ctx);
                _match_x348 = _brw_x355; /*bool*/
                if (_match_x348) {
                  kk_std_core_sslice__sslice_drop(digit, _ctx);
                  kk_std_core_sslice__sslice_drop(amp, _ctx);
                  kk_define_string_literal(, _s_x533, 1, "$", _ctx)
                  _b_x125_127 = kk_string_dup(_s_x533, _ctx); /*string*/
                  goto _match_x528;
                }
                {
                  kk_integer_t grp;
                  bool _match_x350;
                  kk_integer_t _brw_x352;
                  {
                    kk_integer_t _x_1 = amp.start;
                    kk_integer_dup(_x_1, _ctx);
                    kk_std_core_sslice__sslice_drop(amp, _ctx);
                    _brw_x352 = _x_1; /*int*/
                  }
                  bool _brw_x353 = kk_integer_gte_borrow(_brw_x352,(kk_integer_from_small(0)),kk_context()); /*bool*/;
                  kk_integer_drop(_brw_x352, _ctx);
                  _match_x350 = _brw_x353; /*bool*/
                  if (_match_x350) {
                    kk_std_core_sslice__sslice_drop(digit, _ctx);
                    grp = kk_integer_from_small(0); /*int*/
                  }
                  else {
                    kk_string_t s_1_10032 = kk_std_core_sslice_string(digit, _ctx); /*string*/;
                    kk_std_core_types__maybe m_10030;
                    bool _x_x534;
                    kk_std_core_types__optional _match_x351 = kk_std_core_types__new_None(_ctx); /*forall<a> ? a*/;
                    if (kk_std_core_types__is_Optional(_match_x351, _ctx)) {
                      kk_box_t _box_x119 = _match_x351._cons._Optional.value;
                      bool _uniq_hex_533 = kk_bool_unbox(_box_x119);
                      kk_std_core_types__optional_drop(_match_x351, _ctx);
                      _x_x534 = _uniq_hex_533; /*bool*/
                    }
                    else {
                      kk_std_core_types__optional_drop(_match_x351, _ctx);
                      _x_x534 = false; /*bool*/
                    }
                    m_10030 = kk_std_core_int_xparse(s_1_10032, _x_x534, _ctx); /*maybe<int>*/
                    if (kk_std_core_types__is_Nothing(m_10030, _ctx)) {
                      grp = kk_integer_from_small(0); /*int*/
                    }
                    else {
                      kk_box_t _box_x120 = m_10030._cons.Just.value;
                      kk_integer_t x = kk_integer_unbox(_box_x120, _ctx);
                      kk_integer_dup(x, _ctx);
                      kk_std_core_types__maybe_drop(m_10030, _ctx);
                      grp = x; /*int*/
                    }
                  }
                  if (kk_std_core_types__is_Nil(caps, _ctx)) {
                    kk_integer_drop(grp, _ctx);
                    _b_x125_127 = kk_string_empty(); /*string*/
                    goto _match_x528;
                  }
                  {
                    kk_std_core_types__maybe _match_x349;
                    kk_std_core_types__list _x_x536 = kk_std_core_types__list_dup(caps, _ctx); /*list<sslice/sslice>*/
                    _match_x349 = kk_std_core_list__unroll_index_10029(_x_x536, grp, _ctx); /*maybe<5214>*/
                    if (kk_std_core_types__is_Nothing(_match_x349, _ctx)) {
                      _b_x125_127 = kk_string_empty(); /*string*/
                      goto _match_x528;
                    }
                    {
                      kk_box_t _box_x121 = _match_x349._cons.Just.value;
                      kk_std_core_sslice__sslice s_2 = kk_std_core_sslice__sslice_unbox(_box_x121, KK_BORROWED, _ctx);
                      kk_std_core_sslice__sslice_dup(s_2, _ctx);
                      kk_std_core_types__maybe_drop(_match_x349, _ctx);
                      _b_x125_127 = kk_std_core_sslice_string(s_2, _ctx); /*string*/
                      goto _match_x528;
                    }
                  }
                }
              }
            }
          }
        }
      }
      {
        kk_std_core_types__list_drop(_uniq_m_10207, _ctx);
        kk_define_string_literal(, _s_x538, 1, "$", _ctx)
        _b_x125_127 = kk_string_dup(_s_x538, _ctx); /*string*/
      }
      _match_x528: ;
      kk_string_t _b_x123_129;
      if (kk_std_core_types__is_Cons(_uniq_pre_10205, _ctx)) {
        struct kk_std_core_types_Cons* _con_x539 = kk_std_core_types__as_Cons(_uniq_pre_10205, _ctx);
        kk_box_t _box_x122 = _con_x539->head;
        kk_std_core_sslice__sslice _uniq_s_10211 = kk_std_core_sslice__sslice_unbox(_box_x122, KK_BORROWED, _ctx);
        kk_std_core_types__list _uniq_pat_3_10215 = _con_x539->tail;
        if kk_likely(kk_datatype_ptr_is_unique(_uniq_pre_10205, _ctx)) {
          kk_std_core_types__list_drop(_uniq_pat_3_10215, _ctx);
          kk_std_core_sslice__sslice_dup(_uniq_s_10211, _ctx);
          kk_box_drop(_box_x122, _ctx);
          kk_datatype_ptr_free(_uniq_pre_10205, _ctx);
        }
        else {
          kk_std_core_sslice__sslice_dup(_uniq_s_10211, _ctx);
          kk_datatype_ptr_decref(_uniq_pre_10205, _ctx);
        }
        _b_x123_129 = kk_std_core_sslice_string(_uniq_s_10211, _ctx); /*string*/
      }
      else {
        _b_x123_129 = kk_string_empty(); /*string*/
      }
      { // tailcall
        kk_std_core_types__list _x_x541;
        kk_std_core_types__list _x_x542 = kk_std_core_types__new_Cons(_ru_x391, 0, kk_string_box(_b_x123_129), _uniq_acc_10203, _ctx); /*list<82>*/
        _x_x541 = kk_std_core_types__new_Cons(_ru_x390, 0, kk_string_box(_b_x125_127), _x_x542, _ctx); /*list<82>*/
        _uniq_matches_10201 = _uniq_mm_10208;
        _uniq_acc_10203 = _x_x541;
        goto kk__tailcall;
      }
    }
  }
  if (kk_std_core_types__is_Cons(_uniq_matches_10201, _ctx)) {
    struct kk_std_core_types_Cons* _con_x543 = kk_std_core_types__as_Cons(_uniq_matches_10201, _ctx);
    kk_box_t _box_x131 = _con_x543->head;
    kk_std_core_types__list _uniq_pat_2_0_10219 = _con_x543->tail;
    kk_std_core_types__list _uniq_post_10218 = kk_std_core_types__list_unbox(_box_x131, KK_BORROWED, _ctx);
    kk_std_core_types__list_drop(caps, _ctx);
    kk_reuse_t _ru_x397 = kk_reuse_null; /*@reuse*/;
    if kk_likely(kk_datatype_ptr_is_unique(_uniq_matches_10201, _ctx)) {
      _ru_x397 = (kk_datatype_ptr_reuse(_uniq_matches_10201, _ctx));
    }
    else {
      kk_std_core_types__list_dup(_uniq_post_10218, _ctx);
      kk_datatype_ptr_decref(_uniq_matches_10201, _ctx);
    }
    kk_string_t _b_x133_135;
    if (kk_std_core_types__is_Cons(_uniq_post_10218, _ctx)) {
      struct kk_std_core_types_Cons* _con_x544 = kk_std_core_types__as_Cons(_uniq_post_10218, _ctx);
      kk_box_t _box_x132 = _con_x544->head;
      kk_std_core_sslice__sslice _uniq_s_0_10236 = kk_std_core_sslice__sslice_unbox(_box_x132, KK_BORROWED, _ctx);
      kk_std_core_types__list _uniq_pat_3_1_10240 = _con_x544->tail;
      if kk_likely(kk_datatype_ptr_is_unique(_uniq_post_10218, _ctx)) {
        kk_std_core_types__list_drop(_uniq_pat_3_1_10240, _ctx);
        kk_std_core_sslice__sslice_dup(_uniq_s_0_10236, _ctx);
        kk_box_drop(_box_x132, _ctx);
        kk_datatype_ptr_free(_uniq_post_10218, _ctx);
      }
      else {
        kk_std_core_sslice__sslice_dup(_uniq_s_0_10236, _ctx);
        kk_datatype_ptr_decref(_uniq_post_10218, _ctx);
      }
      _b_x133_135 = kk_std_core_sslice_string(_uniq_s_0_10236, _ctx); /*string*/
    }
    else {
      _b_x133_135 = kk_string_empty(); /*string*/
    }
    kk_std_core_types__list _uniq_xs_10220 = kk_std_core_types__new_Cons(_ru_x397, 0, kk_string_box(_b_x133_135), _uniq_acc_10203, _ctx); /*list<string>*/;
    kk_std_core_types__list _uniq_xs_0_10122_10223;
    if (kk_std_core_types__is_Nil(_uniq_xs_10220, _ctx)) {
      _uniq_xs_0_10122_10223 = kk_std_core_types__new_Nil(_ctx); /*list<string>*/
    }
    else {
      _uniq_xs_0_10122_10223 = kk_std_core_list__unroll_reverse_acc_10003(kk_std_core_types__new_Nil(_ctx), _uniq_xs_10220, _ctx); /*list<string>*/
    }
    if (kk_std_core_types__is_Nil(_uniq_xs_0_10122_10223, _ctx)) {
      return kk_string_empty();
    }
    {
      struct kk_std_core_types_Cons* _con_x547 = kk_std_core_types__as_Cons(_uniq_xs_0_10122_10223, _ctx);
      kk_box_t _box_x137 = _con_x547->head;
      kk_std_core_types__list _uniq_pat_1_2_10227 = _con_x547->tail;
      if (kk_std_core_types__is_Nil(_uniq_pat_1_2_10227, _ctx)) {
        kk_string_t _uniq_x_10226 = kk_string_unbox(_box_x137);
        if kk_likely(kk_datatype_ptr_is_unique(_uniq_xs_0_10122_10223, _ctx)) {
          kk_datatype_ptr_free(_uniq_xs_0_10122_10223, _ctx);
        }
        else {
          kk_string_dup(_uniq_x_10226, _ctx);
          kk_datatype_ptr_decref(_uniq_xs_0_10122_10223, _ctx);
        }
        return _uniq_x_10226;
      }
    }
    {
      struct kk_std_core_types_Cons* _con_x548 = kk_std_core_types__as_Cons(_uniq_xs_0_10122_10223, _ctx);
      kk_box_t _box_x138 = _con_x548->head;
      kk_std_core_types__list _uniq_pat_3_0_10230 = _con_x548->tail;
      struct kk_std_core_types_Cons* _con_x549 = kk_std_core_types__as_Cons(_uniq_pat_3_0_10230, _ctx);
      kk_box_t _box_x139 = _con_x549->head;
      kk_std_core_types__list _uniq_pat_4_0_10232 = _con_x549->tail;
      if (kk_std_core_types__is_Nil(_uniq_pat_4_0_10232, _ctx)) {
        kk_string_t _uniq_x_0_10229 = kk_string_unbox(_box_x138);
        kk_string_t _uniq_y_10231 = kk_string_unbox(_box_x139);
        if kk_likely(kk_datatype_ptr_is_unique(_uniq_xs_0_10122_10223, _ctx)) {
          if kk_likely(kk_datatype_ptr_is_unique(_uniq_pat_3_0_10230, _ctx)) {
            kk_datatype_ptr_free(_uniq_pat_3_0_10230, _ctx);
          }
          else {
            kk_string_dup(_uniq_y_10231, _ctx);
            kk_datatype_ptr_decref(_uniq_pat_3_0_10230, _ctx);
          }
          kk_datatype_ptr_free(_uniq_xs_0_10122_10223, _ctx);
        }
        else {
          kk_string_dup(_uniq_x_0_10229, _ctx);
          kk_string_dup(_uniq_y_10231, _ctx);
          kk_datatype_ptr_decref(_uniq_xs_0_10122_10223, _ctx);
        }
        return kk_std_core_types__lp__plus__plus__rp_(_uniq_x_0_10229, _uniq_y_10231, _ctx);
      }
    }
    {
      kk_vector_t _x_x550 = kk_std_core_vector_unvlist(_uniq_xs_0_10122_10223, _ctx); /*vector<911>*/
      return kk_string_join(_x_x550,kk_context());
    }
  }
  {
    kk_std_core_types__list_drop(caps, _ctx);
    return kk_std_core_list_reverse_join(_uniq_acc_10203, _ctx);
  }
}
 
// Replace using a replacement string that can contain `$$` for a `$` sign, `$n` for a capture group,
// `$&` for the entire match `==$0`.

kk_string_t kk_std_text_regex_replace_captures(kk_std_core_types__list caps, kk_string_t repl, kk_context_t* _ctx) { /* (caps : list<sslice/sslice>, repl : string) -> string */ 
  kk_std_text_regex__regex r_10022;
  kk_box_t _x_x551;
  kk_string_t _x_x552;
  kk_define_string_literal(, _s_x553, 20, "\\$(\?:(\\d)|(\\&)|(\\$))", _ctx)
  _x_x552 = kk_string_dup(_s_x553, _ctx); /*string*/
  bool _x_x554;
  kk_std_core_types__optional _match_x347 = kk_std_core_types__new_None(_ctx); /*forall<a> ? a*/;
  if (kk_std_core_types__is_Optional(_match_x347, _ctx)) {
    kk_box_t _box_x140 = _match_x347._cons._Optional.value;
    bool _uniq_ignorecase_161 = kk_bool_unbox(_box_x140);
    kk_std_core_types__optional_drop(_match_x347, _ctx);
    _x_x554 = _uniq_ignorecase_161; /*bool*/
  }
  else {
    kk_std_core_types__optional_drop(_match_x347, _ctx);
    _x_x554 = false; /*bool*/
  }
  bool _x_x555;
  kk_std_core_types__optional _match_x346 = kk_std_core_types__new_None(_ctx); /*forall<a> ? a*/;
  if (kk_std_core_types__is_Optional(_match_x346, _ctx)) {
    kk_box_t _box_x141 = _match_x346._cons._Optional.value;
    bool _uniq_multiline_165 = kk_bool_unbox(_box_x141);
    kk_std_core_types__optional_drop(_match_x346, _ctx);
    _x_x555 = _uniq_multiline_165; /*bool*/
  }
  else {
    kk_std_core_types__optional_drop(_match_x346, _ctx);
    _x_x555 = false; /*bool*/
  }
  _x_x551 = kk_std_text_regex_regex_create(_x_x552, _x_x554, _x_x555, _ctx); /*any*/
  kk_string_t _x_x556;
  kk_define_string_literal(, _s_x557, 20, "\\$(\?:(\\d)|(\\&)|(\\$))", _ctx)
  _x_x556 = kk_string_dup(_s_x557, _ctx); /*string*/
  r_10022 = kk_std_text_regex__new_Regex(_x_x551, _x_x556, _ctx); /*std/text/regex/regex*/
  kk_std_core_types__list _uniq_matches_10201_10536;
  kk_box_t _x_x558;
  {
    kk_box_t _x = r_10022.obj;
    kk_box_dup(_x, _ctx);
    kk_std_text_regex__regex_drop(r_10022, _ctx);
    _x_x558 = _x; /*any*/
  }
  kk_ssize_t _x_x559;
  kk_integer_t _x_x560;
  kk_std_core_types__optional _match_x345 = kk_std_core_types__new_None(_ctx); /*forall<a> ? a*/;
  if (kk_std_core_types__is_Optional(_match_x345, _ctx)) {
    kk_box_t _box_x142 = _match_x345._cons._Optional.value;
    kk_integer_t _uniq_atmost_454 = kk_integer_unbox(_box_x142, _ctx);
    kk_integer_dup(_uniq_atmost_454, _ctx);
    kk_std_core_types__optional_drop(_match_x345, _ctx);
    _x_x560 = _uniq_atmost_454; /*int*/
  }
  else {
    kk_std_core_types__optional_drop(_match_x345, _ctx);
    _x_x560 = kk_integer_from_small(-1); /*int*/
  }
  _x_x559 = kk_std_core_int_ssize__t(_x_x560, _ctx); /*ssize_t*/
  _uniq_matches_10201_10536 = kk_std_text_regex_regex_exec_all(_x_x558, repl, (KK_IZ(0)), _x_x559, _ctx); /*list<list<sslice/sslice>>*/
  return kk_std_text_regex__lift_replace_captures_10525(caps, _uniq_matches_10201_10536, kk_std_core_types__new_Nil(_ctx), _ctx);
}
 
// lifted local: string/replace-all, @spec-x10287
// specialized: std/text/regex/concat-replace, on parameters @uniq-repl@10246, using:
// @uniq-repl@10246 = fn(caps: (list :: V -> V)<sslice/sslice>){
//   std/text/regex/replace-captures(caps, repl);
// }

kk_string_t kk_std_text_regex_string_fs__lift_replace_all_10526(kk_string_t repl, kk_std_core_types__list _uniq_matches_10245, kk_std_core_types__list _uniq_acc_10247, kk_context_t* _ctx) { /* (repl : string, list<list<sslice/sslice>>, list<string>) -> string */ 
  kk__tailcall: ;
  if (kk_std_core_types__is_Cons(_uniq_matches_10245, _ctx)) {
    struct kk_std_core_types_Cons* _con_x561 = kk_std_core_types__as_Cons(_uniq_matches_10245, _ctx);
    kk_box_t _box_x143 = _con_x561->head;
    kk_std_core_types__list _uniq_pat_0_10250 = _con_x561->tail;
    if (kk_std_core_types__is_Cons(_uniq_pat_0_10250, _ctx)) {
      struct kk_std_core_types_Cons* _con_x562 = kk_std_core_types__as_Cons(_uniq_pat_0_10250, _ctx);
      kk_box_t _box_x144 = _con_x562->head;
      kk_std_core_types__list _uniq_pre_10249 = kk_std_core_types__list_unbox(_box_x143, KK_BORROWED, _ctx);
      kk_std_core_types__list _uniq_mm_10252 = _con_x562->tail;
      kk_std_core_types__list _uniq_m_10251 = kk_std_core_types__list_unbox(_box_x144, KK_BORROWED, _ctx);
      kk_reuse_t _ru_x403 = kk_reuse_null; /*@reuse*/;
      kk_reuse_t _ru_x402 = kk_reuse_null; /*@reuse*/;
      if kk_likely(kk_datatype_ptr_is_unique(_uniq_matches_10245, _ctx)) {
        if kk_likely(kk_datatype_ptr_is_unique(_uniq_pat_0_10250, _ctx)) {
          _ru_x402 = (kk_datatype_ptr_reuse(_uniq_pat_0_10250, _ctx));
        }
        else {
          kk_std_core_types__list_dup(_uniq_m_10251, _ctx);
          kk_std_core_types__list_dup(_uniq_mm_10252, _ctx);
          kk_datatype_ptr_decref(_uniq_pat_0_10250, _ctx);
        }
        _ru_x403 = (kk_datatype_ptr_reuse(_uniq_matches_10245, _ctx));
      }
      else {
        kk_std_core_types__list_dup(_uniq_m_10251, _ctx);
        kk_std_core_types__list_dup(_uniq_mm_10252, _ctx);
        kk_std_core_types__list_dup(_uniq_pre_10249, _ctx);
        kk_datatype_ptr_decref(_uniq_matches_10245, _ctx);
      }
      kk_string_t _b_x148_150;
      kk_string_t _x_x563 = kk_string_dup(repl, _ctx); /*string*/
      _b_x148_150 = kk_std_text_regex_replace_captures(_uniq_m_10251, _x_x563, _ctx); /*string*/
      kk_string_t _b_x146_152;
      if (kk_std_core_types__is_Cons(_uniq_pre_10249, _ctx)) {
        struct kk_std_core_types_Cons* _con_x564 = kk_std_core_types__as_Cons(_uniq_pre_10249, _ctx);
        kk_box_t _box_x145 = _con_x564->head;
        kk_std_core_sslice__sslice _uniq_s_10255 = kk_std_core_sslice__sslice_unbox(_box_x145, KK_BORROWED, _ctx);
        kk_std_core_types__list _uniq_pat_3_10259 = _con_x564->tail;
        if kk_likely(kk_datatype_ptr_is_unique(_uniq_pre_10249, _ctx)) {
          kk_std_core_types__list_drop(_uniq_pat_3_10259, _ctx);
          kk_std_core_sslice__sslice_dup(_uniq_s_10255, _ctx);
          kk_box_drop(_box_x145, _ctx);
          kk_datatype_ptr_free(_uniq_pre_10249, _ctx);
        }
        else {
          kk_std_core_sslice__sslice_dup(_uniq_s_10255, _ctx);
          kk_datatype_ptr_decref(_uniq_pre_10249, _ctx);
        }
        _b_x146_152 = kk_std_core_sslice_string(_uniq_s_10255, _ctx); /*string*/
      }
      else {
        _b_x146_152 = kk_string_empty(); /*string*/
      }
      { // tailcall
        kk_std_core_types__list _x_x566;
        kk_std_core_types__list _x_x567 = kk_std_core_types__new_Cons(_ru_x403, 0, kk_string_box(_b_x146_152), _uniq_acc_10247, _ctx); /*list<82>*/
        _x_x566 = kk_std_core_types__new_Cons(_ru_x402, 0, kk_string_box(_b_x148_150), _x_x567, _ctx); /*list<82>*/
        _uniq_matches_10245 = _uniq_mm_10252;
        _uniq_acc_10247 = _x_x566;
        goto kk__tailcall;
      }
    }
  }
  if (kk_std_core_types__is_Cons(_uniq_matches_10245, _ctx)) {
    struct kk_std_core_types_Cons* _con_x568 = kk_std_core_types__as_Cons(_uniq_matches_10245, _ctx);
    kk_box_t _box_x154 = _con_x568->head;
    kk_std_core_types__list _uniq_pat_2_0_10263 = _con_x568->tail;
    kk_std_core_types__list _uniq_post_10262 = kk_std_core_types__list_unbox(_box_x154, KK_BORROWED, _ctx);
    kk_string_drop(repl, _ctx);
    kk_reuse_t _ru_x405 = kk_reuse_null; /*@reuse*/;
    if kk_likely(kk_datatype_ptr_is_unique(_uniq_matches_10245, _ctx)) {
      _ru_x405 = (kk_datatype_ptr_reuse(_uniq_matches_10245, _ctx));
    }
    else {
      kk_std_core_types__list_dup(_uniq_post_10262, _ctx);
      kk_datatype_ptr_decref(_uniq_matches_10245, _ctx);
    }
    kk_string_t _b_x156_158;
    if (kk_std_core_types__is_Cons(_uniq_post_10262, _ctx)) {
      struct kk_std_core_types_Cons* _con_x569 = kk_std_core_types__as_Cons(_uniq_post_10262, _ctx);
      kk_box_t _box_x155 = _con_x569->head;
      kk_std_core_sslice__sslice _uniq_s_0_10280 = kk_std_core_sslice__sslice_unbox(_box_x155, KK_BORROWED, _ctx);
      kk_std_core_types__list _uniq_pat_3_1_10284 = _con_x569->tail;
      if kk_likely(kk_datatype_ptr_is_unique(_uniq_post_10262, _ctx)) {
        kk_std_core_types__list_drop(_uniq_pat_3_1_10284, _ctx);
        kk_std_core_sslice__sslice_dup(_uniq_s_0_10280, _ctx);
        kk_box_drop(_box_x155, _ctx);
        kk_datatype_ptr_free(_uniq_post_10262, _ctx);
      }
      else {
        kk_std_core_sslice__sslice_dup(_uniq_s_0_10280, _ctx);
        kk_datatype_ptr_decref(_uniq_post_10262, _ctx);
      }
      _b_x156_158 = kk_std_core_sslice_string(_uniq_s_0_10280, _ctx); /*string*/
    }
    else {
      _b_x156_158 = kk_string_empty(); /*string*/
    }
    kk_std_core_types__list _uniq_xs_10264 = kk_std_core_types__new_Cons(_ru_x405, 0, kk_string_box(_b_x156_158), _uniq_acc_10247, _ctx); /*list<string>*/;
    kk_std_core_types__list _uniq_xs_0_10122_10267;
    if (kk_std_core_types__is_Nil(_uniq_xs_10264, _ctx)) {
      _uniq_xs_0_10122_10267 = kk_std_core_types__new_Nil(_ctx); /*list<string>*/
    }
    else {
      _uniq_xs_0_10122_10267 = kk_std_core_list__unroll_reverse_acc_10003(kk_std_core_types__new_Nil(_ctx), _uniq_xs_10264, _ctx); /*list<string>*/
    }
    if (kk_std_core_types__is_Nil(_uniq_xs_0_10122_10267, _ctx)) {
      return kk_string_empty();
    }
    {
      struct kk_std_core_types_Cons* _con_x572 = kk_std_core_types__as_Cons(_uniq_xs_0_10122_10267, _ctx);
      kk_box_t _box_x160 = _con_x572->head;
      kk_std_core_types__list _uniq_pat_1_2_10271 = _con_x572->tail;
      if (kk_std_core_types__is_Nil(_uniq_pat_1_2_10271, _ctx)) {
        kk_string_t _uniq_x_10270 = kk_string_unbox(_box_x160);
        if kk_likely(kk_datatype_ptr_is_unique(_uniq_xs_0_10122_10267, _ctx)) {
          kk_datatype_ptr_free(_uniq_xs_0_10122_10267, _ctx);
        }
        else {
          kk_string_dup(_uniq_x_10270, _ctx);
          kk_datatype_ptr_decref(_uniq_xs_0_10122_10267, _ctx);
        }
        return _uniq_x_10270;
      }
    }
    {
      struct kk_std_core_types_Cons* _con_x573 = kk_std_core_types__as_Cons(_uniq_xs_0_10122_10267, _ctx);
      kk_box_t _box_x161 = _con_x573->head;
      kk_std_core_types__list _uniq_pat_3_0_10274 = _con_x573->tail;
      struct kk_std_core_types_Cons* _con_x574 = kk_std_core_types__as_Cons(_uniq_pat_3_0_10274, _ctx);
      kk_box_t _box_x162 = _con_x574->head;
      kk_std_core_types__list _uniq_pat_4_0_10276 = _con_x574->tail;
      if (kk_std_core_types__is_Nil(_uniq_pat_4_0_10276, _ctx)) {
        kk_string_t _uniq_x_0_10273 = kk_string_unbox(_box_x161);
        kk_string_t _uniq_y_10275 = kk_string_unbox(_box_x162);
        if kk_likely(kk_datatype_ptr_is_unique(_uniq_xs_0_10122_10267, _ctx)) {
          if kk_likely(kk_datatype_ptr_is_unique(_uniq_pat_3_0_10274, _ctx)) {
            kk_datatype_ptr_free(_uniq_pat_3_0_10274, _ctx);
          }
          else {
            kk_string_dup(_uniq_y_10275, _ctx);
            kk_datatype_ptr_decref(_uniq_pat_3_0_10274, _ctx);
          }
          kk_datatype_ptr_free(_uniq_xs_0_10122_10267, _ctx);
        }
        else {
          kk_string_dup(_uniq_x_0_10273, _ctx);
          kk_string_dup(_uniq_y_10275, _ctx);
          kk_datatype_ptr_decref(_uniq_xs_0_10122_10267, _ctx);
        }
        return kk_std_core_types__lp__plus__plus__rp_(_uniq_x_0_10273, _uniq_y_10275, _ctx);
      }
    }
    {
      kk_vector_t _x_x575 = kk_std_core_vector_unvlist(_uniq_xs_0_10122_10267, _ctx); /*vector<911>*/
      return kk_string_join(_x_x575,kk_context());
    }
  }
  {
    kk_string_drop(repl, _ctx);
    return kk_std_core_list_reverse_join(_uniq_acc_10247, _ctx);
  }
}
 
// lifted local: string/replace-all, @spec-x10331
// specialized: std/text/regex/concat-replace, on parameters @uniq-repl@10290, using:
// @uniq-repl@10290 = fn(@_wildcard-x158_35: (list :: V -> V)<sslice/sslice>){
//   repl;
// }

kk_string_t kk_std_text_regex_string_fs__lift_replace_all_10527(kk_string_t repl, kk_std_core_types__list _uniq_matches_10289, kk_std_core_types__list _uniq_acc_10291, kk_context_t* _ctx) { /* (repl : string, list<list<sslice/sslice>>, list<string>) -> string */ 
  kk__tailcall: ;
  if (kk_std_core_types__is_Cons(_uniq_matches_10289, _ctx)) {
    struct kk_std_core_types_Cons* _con_x576 = kk_std_core_types__as_Cons(_uniq_matches_10289, _ctx);
    kk_box_t _box_x163 = _con_x576->head;
    kk_std_core_types__list _uniq_pat_0_10294 = _con_x576->tail;
    if (kk_std_core_types__is_Cons(_uniq_pat_0_10294, _ctx)) {
      struct kk_std_core_types_Cons* _con_x577 = kk_std_core_types__as_Cons(_uniq_pat_0_10294, _ctx);
      kk_box_t _box_x164 = _con_x577->head;
      kk_std_core_types__list _uniq_pre_10293 = kk_std_core_types__list_unbox(_box_x163, KK_BORROWED, _ctx);
      kk_std_core_types__list _uniq_mm_10296 = _con_x577->tail;
      kk_std_core_types__list _uniq_m_10295 = kk_std_core_types__list_unbox(_box_x164, KK_BORROWED, _ctx);
      kk_reuse_t _ru_x411 = kk_reuse_null; /*@reuse*/;
      kk_reuse_t _ru_x410 = kk_reuse_null; /*@reuse*/;
      if kk_likely(kk_datatype_ptr_is_unique(_uniq_matches_10289, _ctx)) {
        if kk_likely(kk_datatype_ptr_is_unique(_uniq_pat_0_10294, _ctx)) {
          kk_std_core_types__list_drop(_uniq_m_10295, _ctx);
          _ru_x410 = (kk_datatype_ptr_reuse(_uniq_pat_0_10294, _ctx));
        }
        else {
          kk_std_core_types__list_dup(_uniq_mm_10296, _ctx);
          kk_datatype_ptr_decref(_uniq_pat_0_10294, _ctx);
        }
        _ru_x411 = (kk_datatype_ptr_reuse(_uniq_matches_10289, _ctx));
      }
      else {
        kk_std_core_types__list_dup(_uniq_mm_10296, _ctx);
        kk_std_core_types__list_dup(_uniq_pre_10293, _ctx);
        kk_datatype_ptr_decref(_uniq_matches_10289, _ctx);
      }
      kk_string_t _b_x166_172;
      if (kk_std_core_types__is_Cons(_uniq_pre_10293, _ctx)) {
        struct kk_std_core_types_Cons* _con_x578 = kk_std_core_types__as_Cons(_uniq_pre_10293, _ctx);
        kk_box_t _box_x165 = _con_x578->head;
        kk_std_core_sslice__sslice _uniq_s_10299 = kk_std_core_sslice__sslice_unbox(_box_x165, KK_BORROWED, _ctx);
        kk_std_core_types__list _uniq_pat_3_10303 = _con_x578->tail;
        if kk_likely(kk_datatype_ptr_is_unique(_uniq_pre_10293, _ctx)) {
          kk_std_core_types__list_drop(_uniq_pat_3_10303, _ctx);
          kk_std_core_sslice__sslice_dup(_uniq_s_10299, _ctx);
          kk_box_drop(_box_x165, _ctx);
          kk_datatype_ptr_free(_uniq_pre_10293, _ctx);
        }
        else {
          kk_std_core_sslice__sslice_dup(_uniq_s_10299, _ctx);
          kk_datatype_ptr_decref(_uniq_pre_10293, _ctx);
        }
        _b_x166_172 = kk_std_core_sslice_string(_uniq_s_10299, _ctx); /*string*/
      }
      else {
        _b_x166_172 = kk_string_empty(); /*string*/
      }
      { // tailcall
        kk_string_t _x_x580 = kk_string_dup(repl, _ctx); /*string*/
        kk_std_core_types__list _x_x581;
        kk_std_core_types__list _x_x582 = kk_std_core_types__new_Cons(_ru_x411, 0, kk_string_box(_b_x166_172), _uniq_acc_10291, _ctx); /*list<82>*/
        _x_x581 = kk_std_core_types__new_Cons(_ru_x410, 0, kk_string_box(repl), _x_x582, _ctx); /*list<82>*/
        repl = _x_x580;
        _uniq_matches_10289 = _uniq_mm_10296;
        _uniq_acc_10291 = _x_x581;
        goto kk__tailcall;
      }
    }
  }
  if (kk_std_core_types__is_Cons(_uniq_matches_10289, _ctx)) {
    struct kk_std_core_types_Cons* _con_x583 = kk_std_core_types__as_Cons(_uniq_matches_10289, _ctx);
    kk_box_t _box_x174 = _con_x583->head;
    kk_std_core_types__list _uniq_pat_2_0_10307 = _con_x583->tail;
    kk_std_core_types__list _uniq_post_10306 = kk_std_core_types__list_unbox(_box_x174, KK_BORROWED, _ctx);
    kk_string_drop(repl, _ctx);
    kk_reuse_t _ru_x413 = kk_reuse_null; /*@reuse*/;
    if kk_likely(kk_datatype_ptr_is_unique(_uniq_matches_10289, _ctx)) {
      _ru_x413 = (kk_datatype_ptr_reuse(_uniq_matches_10289, _ctx));
    }
    else {
      kk_std_core_types__list_dup(_uniq_post_10306, _ctx);
      kk_datatype_ptr_decref(_uniq_matches_10289, _ctx);
    }
    kk_string_t _b_x176_178;
    if (kk_std_core_types__is_Cons(_uniq_post_10306, _ctx)) {
      struct kk_std_core_types_Cons* _con_x584 = kk_std_core_types__as_Cons(_uniq_post_10306, _ctx);
      kk_box_t _box_x175 = _con_x584->head;
      kk_std_core_sslice__sslice _uniq_s_0_10324 = kk_std_core_sslice__sslice_unbox(_box_x175, KK_BORROWED, _ctx);
      kk_std_core_types__list _uniq_pat_3_1_10328 = _con_x584->tail;
      if kk_likely(kk_datatype_ptr_is_unique(_uniq_post_10306, _ctx)) {
        kk_std_core_types__list_drop(_uniq_pat_3_1_10328, _ctx);
        kk_std_core_sslice__sslice_dup(_uniq_s_0_10324, _ctx);
        kk_box_drop(_box_x175, _ctx);
        kk_datatype_ptr_free(_uniq_post_10306, _ctx);
      }
      else {
        kk_std_core_sslice__sslice_dup(_uniq_s_0_10324, _ctx);
        kk_datatype_ptr_decref(_uniq_post_10306, _ctx);
      }
      _b_x176_178 = kk_std_core_sslice_string(_uniq_s_0_10324, _ctx); /*string*/
    }
    else {
      _b_x176_178 = kk_string_empty(); /*string*/
    }
    kk_std_core_types__list _uniq_xs_10308 = kk_std_core_types__new_Cons(_ru_x413, 0, kk_string_box(_b_x176_178), _uniq_acc_10291, _ctx); /*list<string>*/;
    kk_std_core_types__list _uniq_xs_0_10122_10311;
    if (kk_std_core_types__is_Nil(_uniq_xs_10308, _ctx)) {
      _uniq_xs_0_10122_10311 = kk_std_core_types__new_Nil(_ctx); /*list<string>*/
    }
    else {
      _uniq_xs_0_10122_10311 = kk_std_core_list__unroll_reverse_acc_10003(kk_std_core_types__new_Nil(_ctx), _uniq_xs_10308, _ctx); /*list<string>*/
    }
    if (kk_std_core_types__is_Nil(_uniq_xs_0_10122_10311, _ctx)) {
      return kk_string_empty();
    }
    {
      struct kk_std_core_types_Cons* _con_x587 = kk_std_core_types__as_Cons(_uniq_xs_0_10122_10311, _ctx);
      kk_box_t _box_x180 = _con_x587->head;
      kk_std_core_types__list _uniq_pat_1_2_10315 = _con_x587->tail;
      if (kk_std_core_types__is_Nil(_uniq_pat_1_2_10315, _ctx)) {
        kk_string_t _uniq_x_10314 = kk_string_unbox(_box_x180);
        if kk_likely(kk_datatype_ptr_is_unique(_uniq_xs_0_10122_10311, _ctx)) {
          kk_datatype_ptr_free(_uniq_xs_0_10122_10311, _ctx);
        }
        else {
          kk_string_dup(_uniq_x_10314, _ctx);
          kk_datatype_ptr_decref(_uniq_xs_0_10122_10311, _ctx);
        }
        return _uniq_x_10314;
      }
    }
    {
      struct kk_std_core_types_Cons* _con_x588 = kk_std_core_types__as_Cons(_uniq_xs_0_10122_10311, _ctx);
      kk_box_t _box_x181 = _con_x588->head;
      kk_std_core_types__list _uniq_pat_3_0_10318 = _con_x588->tail;
      struct kk_std_core_types_Cons* _con_x589 = kk_std_core_types__as_Cons(_uniq_pat_3_0_10318, _ctx);
      kk_box_t _box_x182 = _con_x589->head;
      kk_std_core_types__list _uniq_pat_4_0_10320 = _con_x589->tail;
      if (kk_std_core_types__is_Nil(_uniq_pat_4_0_10320, _ctx)) {
        kk_string_t _uniq_x_0_10317 = kk_string_unbox(_box_x181);
        kk_string_t _uniq_y_10319 = kk_string_unbox(_box_x182);
        if kk_likely(kk_datatype_ptr_is_unique(_uniq_xs_0_10122_10311, _ctx)) {
          if kk_likely(kk_datatype_ptr_is_unique(_uniq_pat_3_0_10318, _ctx)) {
            kk_datatype_ptr_free(_uniq_pat_3_0_10318, _ctx);
          }
          else {
            kk_string_dup(_uniq_y_10319, _ctx);
            kk_datatype_ptr_decref(_uniq_pat_3_0_10318, _ctx);
          }
          kk_datatype_ptr_free(_uniq_xs_0_10122_10311, _ctx);
        }
        else {
          kk_string_dup(_uniq_x_0_10317, _ctx);
          kk_string_dup(_uniq_y_10319, _ctx);
          kk_datatype_ptr_decref(_uniq_xs_0_10122_10311, _ctx);
        }
        return kk_std_core_types__lp__plus__plus__rp_(_uniq_x_0_10317, _uniq_y_10319, _ctx);
      }
    }
    {
      kk_vector_t _x_x590 = kk_std_core_vector_unvlist(_uniq_xs_0_10122_10311, _ctx); /*vector<911>*/
      return kk_string_join(_x_x590,kk_context());
    }
  }
  {
    kk_string_drop(repl, _ctx);
    return kk_std_core_list_reverse_join(_uniq_acc_10291, _ctx);
  }
}
 
// Replace all occurrences of `regex` with the replacement string `repl` in a string `s`.
// The replacement string can contain `$$` for a `$` sign, `$n` for a capture group,
// `$&` for the entire match `==$0`.

kk_string_t kk_std_text_regex_string_fs_replace_all(kk_string_t s, kk_std_text_regex__regex regex_0, kk_string_t repl, kk_std_core_types__optional atmost, kk_context_t* _ctx) { /* (s : string, regex : regex, repl : string, atmost : ? int) -> string */ 
  bool _match_x344;
  kk_string_t _x_x591 = kk_string_dup(repl, _ctx); /*string*/
  kk_string_t _x_x592;
  kk_define_string_literal(, _s_x593, 1, "$", _ctx)
  _x_x592 = kk_string_dup(_s_x593, _ctx); /*string*/
  _match_x344 = kk_string_contains(_x_x591,_x_x592,kk_context()); /*bool*/
  if (_match_x344) {
    kk_std_core_types__list _uniq_matches_10245;
    kk_box_t _x_x594;
    {
      kk_box_t _x = regex_0.obj;
      kk_box_dup(_x, _ctx);
      kk_std_text_regex__regex_drop(regex_0, _ctx);
      _x_x594 = _x; /*any*/
    }
    kk_ssize_t _x_x595;
    kk_integer_t _x_x596;
    if (kk_std_core_types__is_Optional(atmost, _ctx)) {
      kk_box_t _box_x183 = atmost._cons._Optional.value;
      kk_integer_t _uniq_atmost_487 = kk_integer_unbox(_box_x183, _ctx);
      kk_integer_dup(_uniq_atmost_487, _ctx);
      kk_std_core_types__optional_drop(atmost, _ctx);
      _x_x596 = _uniq_atmost_487; /*int*/
    }
    else {
      kk_std_core_types__optional_drop(atmost, _ctx);
      _x_x596 = kk_integer_from_small(-1); /*int*/
    }
    _x_x595 = kk_std_core_int_ssize__t(_x_x596, _ctx); /*ssize_t*/
    _uniq_matches_10245 = kk_std_text_regex_regex_exec_all(_x_x594, s, (KK_IZ(0)), _x_x595, _ctx); /*list<list<sslice/sslice>>*/
    return kk_std_text_regex_string_fs__lift_replace_all_10526(repl, _uniq_matches_10245, kk_std_core_types__new_Nil(_ctx), _ctx);
  }
  {
    kk_std_core_types__list _uniq_matches_10289;
    kk_box_t _x_x597;
    {
      kk_box_t _x_0 = regex_0.obj;
      kk_box_dup(_x_0, _ctx);
      kk_std_text_regex__regex_drop(regex_0, _ctx);
      _x_x597 = _x_0; /*any*/
    }
    kk_ssize_t _x_x598;
    kk_integer_t _x_x599;
    if (kk_std_core_types__is_Optional(atmost, _ctx)) {
      kk_box_t _box_x184 = atmost._cons._Optional.value;
      kk_integer_t _uniq_atmost_487_0 = kk_integer_unbox(_box_x184, _ctx);
      kk_integer_dup(_uniq_atmost_487_0, _ctx);
      kk_std_core_types__optional_drop(atmost, _ctx);
      _x_x599 = _uniq_atmost_487_0; /*int*/
    }
    else {
      kk_std_core_types__optional_drop(atmost, _ctx);
      _x_x599 = kk_integer_from_small(-1); /*int*/
    }
    _x_x598 = kk_std_core_int_ssize__t(_x_x599, _ctx); /*ssize_t*/
    _uniq_matches_10289 = kk_std_text_regex_regex_exec_all(_x_x597, s, (KK_IZ(0)), _x_x598, _ctx); /*list<list<sslice/sslice>>*/
    return kk_std_text_regex_string_fs__lift_replace_all_10527(repl, _uniq_matches_10289, kk_std_core_types__new_Nil(_ctx), _ctx);
  }
}
 
// lifted local: groups-count, @spec-x10375
// specialized: std/text/regex/concat-replace, on parameters @uniq-repl@10334, using:
// @uniq-repl@10334 = fn(caps: (list :: V -> V)<sslice/sslice>){
//   std/text/regex/replace-captures(caps, "");
// }

kk_string_t kk_std_text_regex__lift_groups_count_10528(kk_std_core_types__list _uniq_matches_10333, kk_std_core_types__list _uniq_acc_10335, kk_context_t* _ctx) { /* (list<list<sslice/sslice>>, list<string>) -> string */ 
  kk__tailcall: ;
  if (kk_std_core_types__is_Cons(_uniq_matches_10333, _ctx)) {
    struct kk_std_core_types_Cons* _con_x600 = kk_std_core_types__as_Cons(_uniq_matches_10333, _ctx);
    kk_box_t _box_x185 = _con_x600->head;
    kk_std_core_types__list _uniq_pat_0_10338 = _con_x600->tail;
    if (kk_std_core_types__is_Cons(_uniq_pat_0_10338, _ctx)) {
      struct kk_std_core_types_Cons* _con_x601 = kk_std_core_types__as_Cons(_uniq_pat_0_10338, _ctx);
      kk_box_t _box_x186 = _con_x601->head;
      kk_std_core_types__list _uniq_pre_10337 = kk_std_core_types__list_unbox(_box_x185, KK_BORROWED, _ctx);
      kk_std_core_types__list _uniq_mm_10340 = _con_x601->tail;
      kk_std_core_types__list _uniq_m_10339 = kk_std_core_types__list_unbox(_box_x186, KK_BORROWED, _ctx);
      kk_reuse_t _ru_x419 = kk_reuse_null; /*@reuse*/;
      kk_reuse_t _ru_x418 = kk_reuse_null; /*@reuse*/;
      if kk_likely(kk_datatype_ptr_is_unique(_uniq_matches_10333, _ctx)) {
        if kk_likely(kk_datatype_ptr_is_unique(_uniq_pat_0_10338, _ctx)) {
          _ru_x418 = (kk_datatype_ptr_reuse(_uniq_pat_0_10338, _ctx));
        }
        else {
          kk_std_core_types__list_dup(_uniq_m_10339, _ctx);
          kk_std_core_types__list_dup(_uniq_mm_10340, _ctx);
          kk_datatype_ptr_decref(_uniq_pat_0_10338, _ctx);
        }
        _ru_x419 = (kk_datatype_ptr_reuse(_uniq_matches_10333, _ctx));
      }
      else {
        kk_std_core_types__list_dup(_uniq_m_10339, _ctx);
        kk_std_core_types__list_dup(_uniq_mm_10340, _ctx);
        kk_std_core_types__list_dup(_uniq_pre_10337, _ctx);
        kk_datatype_ptr_decref(_uniq_matches_10333, _ctx);
      }
      kk_std_text_regex__regex r_10022;
      kk_box_t _x_x602;
      kk_string_t _x_x603;
      kk_define_string_literal(, _s_x604, 20, "\\$(\?:(\\d)|(\\&)|(\\$))", _ctx)
      _x_x603 = kk_string_dup(_s_x604, _ctx); /*string*/
      bool _x_x605;
      kk_std_core_types__optional _match_x343 = kk_std_core_types__new_None(_ctx); /*forall<a> ? a*/;
      if (kk_std_core_types__is_Optional(_match_x343, _ctx)) {
        kk_box_t _box_x187 = _match_x343._cons._Optional.value;
        bool _uniq_ignorecase_161 = kk_bool_unbox(_box_x187);
        kk_std_core_types__optional_drop(_match_x343, _ctx);
        _x_x605 = _uniq_ignorecase_161; /*bool*/
      }
      else {
        kk_std_core_types__optional_drop(_match_x343, _ctx);
        _x_x605 = false; /*bool*/
      }
      bool _x_x606;
      kk_std_core_types__optional _match_x342 = kk_std_core_types__new_None(_ctx); /*forall<a> ? a*/;
      if (kk_std_core_types__is_Optional(_match_x342, _ctx)) {
        kk_box_t _box_x188 = _match_x342._cons._Optional.value;
        bool _uniq_multiline_165 = kk_bool_unbox(_box_x188);
        kk_std_core_types__optional_drop(_match_x342, _ctx);
        _x_x606 = _uniq_multiline_165; /*bool*/
      }
      else {
        kk_std_core_types__optional_drop(_match_x342, _ctx);
        _x_x606 = false; /*bool*/
      }
      _x_x602 = kk_std_text_regex_regex_create(_x_x603, _x_x605, _x_x606, _ctx); /*any*/
      kk_string_t _x_x607;
      kk_define_string_literal(, _s_x608, 20, "\\$(\?:(\\d)|(\\&)|(\\$))", _ctx)
      _x_x607 = kk_string_dup(_s_x608, _ctx); /*string*/
      r_10022 = kk_std_text_regex__new_Regex(_x_x602, _x_x607, _ctx); /*std/text/regex/regex*/
      kk_std_core_types__list _uniq_matches_10201_10536;
      kk_box_t _x_x609;
      {
        kk_box_t _x = r_10022.obj;
        kk_box_dup(_x, _ctx);
        kk_std_text_regex__regex_drop(r_10022, _ctx);
        _x_x609 = _x; /*any*/
      }
      kk_string_t _x_x610 = kk_string_empty(); /*string*/
      kk_ssize_t _x_x612;
      kk_integer_t _x_x613;
      kk_std_core_types__optional _match_x341 = kk_std_core_types__new_None(_ctx); /*forall<a> ? a*/;
      if (kk_std_core_types__is_Optional(_match_x341, _ctx)) {
        kk_box_t _box_x189 = _match_x341._cons._Optional.value;
        kk_integer_t _uniq_atmost_454 = kk_integer_unbox(_box_x189, _ctx);
        kk_integer_dup(_uniq_atmost_454, _ctx);
        kk_std_core_types__optional_drop(_match_x341, _ctx);
        _x_x613 = _uniq_atmost_454; /*int*/
      }
      else {
        kk_std_core_types__optional_drop(_match_x341, _ctx);
        _x_x613 = kk_integer_from_small(-1); /*int*/
      }
      _x_x612 = kk_std_core_int_ssize__t(_x_x613, _ctx); /*ssize_t*/
      _uniq_matches_10201_10536 = kk_std_text_regex_regex_exec_all(_x_x609, _x_x610, (KK_IZ(0)), _x_x612, _ctx); /*list<list<sslice/sslice>>*/
      kk_string_t _b_x193_204 = kk_std_text_regex__lift_replace_captures_10525(_uniq_m_10339, _uniq_matches_10201_10536, kk_std_core_types__new_Nil(_ctx), _ctx); /*string*/;
      kk_string_t _b_x191_206;
      if (kk_std_core_types__is_Cons(_uniq_pre_10337, _ctx)) {
        struct kk_std_core_types_Cons* _con_x614 = kk_std_core_types__as_Cons(_uniq_pre_10337, _ctx);
        kk_box_t _box_x190 = _con_x614->head;
        kk_std_core_sslice__sslice _uniq_s_10343 = kk_std_core_sslice__sslice_unbox(_box_x190, KK_BORROWED, _ctx);
        kk_std_core_types__list _uniq_pat_3_10347 = _con_x614->tail;
        if kk_likely(kk_datatype_ptr_is_unique(_uniq_pre_10337, _ctx)) {
          kk_std_core_types__list_drop(_uniq_pat_3_10347, _ctx);
          kk_std_core_sslice__sslice_dup(_uniq_s_10343, _ctx);
          kk_box_drop(_box_x190, _ctx);
          kk_datatype_ptr_free(_uniq_pre_10337, _ctx);
        }
        else {
          kk_std_core_sslice__sslice_dup(_uniq_s_10343, _ctx);
          kk_datatype_ptr_decref(_uniq_pre_10337, _ctx);
        }
        _b_x191_206 = kk_std_core_sslice_string(_uniq_s_10343, _ctx); /*string*/
      }
      else {
        _b_x191_206 = kk_string_empty(); /*string*/
      }
      { // tailcall
        kk_std_core_types__list _x_x616;
        kk_std_core_types__list _x_x617 = kk_std_core_types__new_Cons(_ru_x419, 0, kk_string_box(_b_x191_206), _uniq_acc_10335, _ctx); /*list<82>*/
        _x_x616 = kk_std_core_types__new_Cons(_ru_x418, 0, kk_string_box(_b_x193_204), _x_x617, _ctx); /*list<82>*/
        _uniq_matches_10333 = _uniq_mm_10340;
        _uniq_acc_10335 = _x_x616;
        goto kk__tailcall;
      }
    }
  }
  if (kk_std_core_types__is_Cons(_uniq_matches_10333, _ctx)) {
    struct kk_std_core_types_Cons* _con_x618 = kk_std_core_types__as_Cons(_uniq_matches_10333, _ctx);
    kk_box_t _box_x195 = _con_x618->head;
    kk_std_core_types__list _uniq_pat_2_0_10351 = _con_x618->tail;
    kk_std_core_types__list _uniq_post_10350 = kk_std_core_types__list_unbox(_box_x195, KK_BORROWED, _ctx);
    kk_reuse_t _ru_x421 = kk_reuse_null; /*@reuse*/;
    if kk_likely(kk_datatype_ptr_is_unique(_uniq_matches_10333, _ctx)) {
      _ru_x421 = (kk_datatype_ptr_reuse(_uniq_matches_10333, _ctx));
    }
    else {
      kk_std_core_types__list_dup(_uniq_post_10350, _ctx);
      kk_datatype_ptr_decref(_uniq_matches_10333, _ctx);
    }
    kk_string_t _b_x197_199;
    if (kk_std_core_types__is_Cons(_uniq_post_10350, _ctx)) {
      struct kk_std_core_types_Cons* _con_x619 = kk_std_core_types__as_Cons(_uniq_post_10350, _ctx);
      kk_box_t _box_x196 = _con_x619->head;
      kk_std_core_sslice__sslice _uniq_s_0_10368 = kk_std_core_sslice__sslice_unbox(_box_x196, KK_BORROWED, _ctx);
      kk_std_core_types__list _uniq_pat_3_1_10372 = _con_x619->tail;
      if kk_likely(kk_datatype_ptr_is_unique(_uniq_post_10350, _ctx)) {
        kk_std_core_types__list_drop(_uniq_pat_3_1_10372, _ctx);
        kk_std_core_sslice__sslice_dup(_uniq_s_0_10368, _ctx);
        kk_box_drop(_box_x196, _ctx);
        kk_datatype_ptr_free(_uniq_post_10350, _ctx);
      }
      else {
        kk_std_core_sslice__sslice_dup(_uniq_s_0_10368, _ctx);
        kk_datatype_ptr_decref(_uniq_post_10350, _ctx);
      }
      _b_x197_199 = kk_std_core_sslice_string(_uniq_s_0_10368, _ctx); /*string*/
    }
    else {
      _b_x197_199 = kk_string_empty(); /*string*/
    }
    kk_std_core_types__list _uniq_xs_10352 = kk_std_core_types__new_Cons(_ru_x421, 0, kk_string_box(_b_x197_199), _uniq_acc_10335, _ctx); /*list<string>*/;
    kk_std_core_types__list _uniq_xs_0_10122_10355;
    if (kk_std_core_types__is_Nil(_uniq_xs_10352, _ctx)) {
      _uniq_xs_0_10122_10355 = kk_std_core_types__new_Nil(_ctx); /*list<string>*/
    }
    else {
      _uniq_xs_0_10122_10355 = kk_std_core_list__unroll_reverse_acc_10003(kk_std_core_types__new_Nil(_ctx), _uniq_xs_10352, _ctx); /*list<string>*/
    }
    if (kk_std_core_types__is_Nil(_uniq_xs_0_10122_10355, _ctx)) {
      return kk_string_empty();
    }
    {
      struct kk_std_core_types_Cons* _con_x622 = kk_std_core_types__as_Cons(_uniq_xs_0_10122_10355, _ctx);
      kk_box_t _box_x201 = _con_x622->head;
      kk_std_core_types__list _uniq_pat_1_2_10359 = _con_x622->tail;
      if (kk_std_core_types__is_Nil(_uniq_pat_1_2_10359, _ctx)) {
        kk_string_t _uniq_x_10358 = kk_string_unbox(_box_x201);
        if kk_likely(kk_datatype_ptr_is_unique(_uniq_xs_0_10122_10355, _ctx)) {
          kk_datatype_ptr_free(_uniq_xs_0_10122_10355, _ctx);
        }
        else {
          kk_string_dup(_uniq_x_10358, _ctx);
          kk_datatype_ptr_decref(_uniq_xs_0_10122_10355, _ctx);
        }
        return _uniq_x_10358;
      }
    }
    {
      struct kk_std_core_types_Cons* _con_x623 = kk_std_core_types__as_Cons(_uniq_xs_0_10122_10355, _ctx);
      kk_box_t _box_x202 = _con_x623->head;
      kk_std_core_types__list _uniq_pat_3_0_10362 = _con_x623->tail;
      struct kk_std_core_types_Cons* _con_x624 = kk_std_core_types__as_Cons(_uniq_pat_3_0_10362, _ctx);
      kk_box_t _box_x203 = _con_x624->head;
      kk_std_core_types__list _uniq_pat_4_0_10364 = _con_x624->tail;
      if (kk_std_core_types__is_Nil(_uniq_pat_4_0_10364, _ctx)) {
        kk_string_t _uniq_x_0_10361 = kk_string_unbox(_box_x202);
        kk_string_t _uniq_y_10363 = kk_string_unbox(_box_x203);
        if kk_likely(kk_datatype_ptr_is_unique(_uniq_xs_0_10122_10355, _ctx)) {
          if kk_likely(kk_datatype_ptr_is_unique(_uniq_pat_3_0_10362, _ctx)) {
            kk_datatype_ptr_free(_uniq_pat_3_0_10362, _ctx);
          }
          else {
            kk_string_dup(_uniq_y_10363, _ctx);
            kk_datatype_ptr_decref(_uniq_pat_3_0_10362, _ctx);
          }
          kk_datatype_ptr_free(_uniq_xs_0_10122_10355, _ctx);
        }
        else {
          kk_string_dup(_uniq_x_0_10361, _ctx);
          kk_string_dup(_uniq_y_10363, _ctx);
          kk_datatype_ptr_decref(_uniq_xs_0_10122_10355, _ctx);
        }
        return kk_std_core_types__lp__plus__plus__rp_(_uniq_x_0_10361, _uniq_y_10363, _ctx);
      }
    }
    {
      kk_vector_t _x_x625 = kk_std_core_vector_unvlist(_uniq_xs_0_10122_10355, _ctx); /*vector<911>*/
      return kk_string_join(_x_x625,kk_context());
    }
  }
  {
    return kk_std_core_list_reverse_join(_uniq_acc_10335, _ctx);
  }
}
 
// lifted local: groups-count, @spec-x10419
// specialized: std/text/regex/concat-replace, on parameters @uniq-repl@10378, using:
// @uniq-repl@10378 = fn(@_wildcard-x158_35: (list :: V -> V)<sslice/sslice>){
//   "";
// }

kk_string_t kk_std_text_regex__lift_groups_count_10529(kk_std_core_types__list _uniq_matches_10377, kk_std_core_types__list _uniq_acc_10379, kk_context_t* _ctx) { /* (list<list<sslice/sslice>>, list<string>) -> string */ 
  kk__tailcall: ;
  if (kk_std_core_types__is_Cons(_uniq_matches_10377, _ctx)) {
    struct kk_std_core_types_Cons* _con_x626 = kk_std_core_types__as_Cons(_uniq_matches_10377, _ctx);
    kk_box_t _box_x208 = _con_x626->head;
    kk_std_core_types__list _uniq_pat_0_10382 = _con_x626->tail;
    if (kk_std_core_types__is_Cons(_uniq_pat_0_10382, _ctx)) {
      struct kk_std_core_types_Cons* _con_x627 = kk_std_core_types__as_Cons(_uniq_pat_0_10382, _ctx);
      kk_box_t _box_x209 = _con_x627->head;
      kk_std_core_types__list _uniq_pre_10381 = kk_std_core_types__list_unbox(_box_x208, KK_BORROWED, _ctx);
      kk_std_core_types__list _uniq_mm_10384 = _con_x627->tail;
      kk_std_core_types__list _uniq_m_10383 = kk_std_core_types__list_unbox(_box_x209, KK_BORROWED, _ctx);
      kk_reuse_t _ru_x427 = kk_reuse_null; /*@reuse*/;
      kk_reuse_t _ru_x426 = kk_reuse_null; /*@reuse*/;
      if kk_likely(kk_datatype_ptr_is_unique(_uniq_matches_10377, _ctx)) {
        if kk_likely(kk_datatype_ptr_is_unique(_uniq_pat_0_10382, _ctx)) {
          kk_std_core_types__list_drop(_uniq_m_10383, _ctx);
          _ru_x426 = (kk_datatype_ptr_reuse(_uniq_pat_0_10382, _ctx));
        }
        else {
          kk_std_core_types__list_dup(_uniq_mm_10384, _ctx);
          kk_datatype_ptr_decref(_uniq_pat_0_10382, _ctx);
        }
        _ru_x427 = (kk_datatype_ptr_reuse(_uniq_matches_10377, _ctx));
      }
      else {
        kk_std_core_types__list_dup(_uniq_mm_10384, _ctx);
        kk_std_core_types__list_dup(_uniq_pre_10381, _ctx);
        kk_datatype_ptr_decref(_uniq_matches_10377, _ctx);
      }
      kk_string_t _b_x211_226;
      if (kk_std_core_types__is_Cons(_uniq_pre_10381, _ctx)) {
        struct kk_std_core_types_Cons* _con_x628 = kk_std_core_types__as_Cons(_uniq_pre_10381, _ctx);
        kk_box_t _box_x210 = _con_x628->head;
        kk_std_core_sslice__sslice _uniq_s_10387 = kk_std_core_sslice__sslice_unbox(_box_x210, KK_BORROWED, _ctx);
        kk_std_core_types__list _uniq_pat_3_10391 = _con_x628->tail;
        if kk_likely(kk_datatype_ptr_is_unique(_uniq_pre_10381, _ctx)) {
          kk_std_core_types__list_drop(_uniq_pat_3_10391, _ctx);
          kk_std_core_sslice__sslice_dup(_uniq_s_10387, _ctx);
          kk_box_drop(_box_x210, _ctx);
          kk_datatype_ptr_free(_uniq_pre_10381, _ctx);
        }
        else {
          kk_std_core_sslice__sslice_dup(_uniq_s_10387, _ctx);
          kk_datatype_ptr_decref(_uniq_pre_10381, _ctx);
        }
        _b_x211_226 = kk_std_core_sslice_string(_uniq_s_10387, _ctx); /*string*/
      }
      else {
        _b_x211_226 = kk_string_empty(); /*string*/
      }
      { // tailcall
        kk_std_core_types__list _x_x630;
        kk_box_t _x_x631;
        kk_string_t _x_x632 = kk_string_empty(); /*string*/
        _x_x631 = kk_string_box(_x_x632); /*82*/
        kk_std_core_types__list _x_x634 = kk_std_core_types__new_Cons(_ru_x427, 0, kk_string_box(_b_x211_226), _uniq_acc_10379, _ctx); /*list<82>*/
        _x_x630 = kk_std_core_types__new_Cons(_ru_x426, 0, _x_x631, _x_x634, _ctx); /*list<82>*/
        _uniq_matches_10377 = _uniq_mm_10384;
        _uniq_acc_10379 = _x_x630;
        goto kk__tailcall;
      }
    }
  }
  if (kk_std_core_types__is_Cons(_uniq_matches_10377, _ctx)) {
    struct kk_std_core_types_Cons* _con_x635 = kk_std_core_types__as_Cons(_uniq_matches_10377, _ctx);
    kk_box_t _box_x215 = _con_x635->head;
    kk_std_core_types__list _uniq_pat_2_0_10395 = _con_x635->tail;
    kk_std_core_types__list _uniq_post_10394 = kk_std_core_types__list_unbox(_box_x215, KK_BORROWED, _ctx);
    kk_reuse_t _ru_x429 = kk_reuse_null; /*@reuse*/;
    if kk_likely(kk_datatype_ptr_is_unique(_uniq_matches_10377, _ctx)) {
      _ru_x429 = (kk_datatype_ptr_reuse(_uniq_matches_10377, _ctx));
    }
    else {
      kk_std_core_types__list_dup(_uniq_post_10394, _ctx);
      kk_datatype_ptr_decref(_uniq_matches_10377, _ctx);
    }
    kk_string_t _b_x217_219;
    if (kk_std_core_types__is_Cons(_uniq_post_10394, _ctx)) {
      struct kk_std_core_types_Cons* _con_x636 = kk_std_core_types__as_Cons(_uniq_post_10394, _ctx);
      kk_box_t _box_x216 = _con_x636->head;
      kk_std_core_sslice__sslice _uniq_s_0_10412 = kk_std_core_sslice__sslice_unbox(_box_x216, KK_BORROWED, _ctx);
      kk_std_core_types__list _uniq_pat_3_1_10416 = _con_x636->tail;
      if kk_likely(kk_datatype_ptr_is_unique(_uniq_post_10394, _ctx)) {
        kk_std_core_types__list_drop(_uniq_pat_3_1_10416, _ctx);
        kk_std_core_sslice__sslice_dup(_uniq_s_0_10412, _ctx);
        kk_box_drop(_box_x216, _ctx);
        kk_datatype_ptr_free(_uniq_post_10394, _ctx);
      }
      else {
        kk_std_core_sslice__sslice_dup(_uniq_s_0_10412, _ctx);
        kk_datatype_ptr_decref(_uniq_post_10394, _ctx);
      }
      _b_x217_219 = kk_std_core_sslice_string(_uniq_s_0_10412, _ctx); /*string*/
    }
    else {
      _b_x217_219 = kk_string_empty(); /*string*/
    }
    kk_std_core_types__list _uniq_xs_10396 = kk_std_core_types__new_Cons(_ru_x429, 0, kk_string_box(_b_x217_219), _uniq_acc_10379, _ctx); /*list<string>*/;
    kk_std_core_types__list _uniq_xs_0_10122_10399;
    if (kk_std_core_types__is_Nil(_uniq_xs_10396, _ctx)) {
      _uniq_xs_0_10122_10399 = kk_std_core_types__new_Nil(_ctx); /*list<string>*/
    }
    else {
      _uniq_xs_0_10122_10399 = kk_std_core_list__unroll_reverse_acc_10003(kk_std_core_types__new_Nil(_ctx), _uniq_xs_10396, _ctx); /*list<string>*/
    }
    if (kk_std_core_types__is_Nil(_uniq_xs_0_10122_10399, _ctx)) {
      return kk_string_empty();
    }
    {
      struct kk_std_core_types_Cons* _con_x639 = kk_std_core_types__as_Cons(_uniq_xs_0_10122_10399, _ctx);
      kk_box_t _box_x221 = _con_x639->head;
      kk_std_core_types__list _uniq_pat_1_2_10403 = _con_x639->tail;
      if (kk_std_core_types__is_Nil(_uniq_pat_1_2_10403, _ctx)) {
        kk_string_t _uniq_x_10402 = kk_string_unbox(_box_x221);
        if kk_likely(kk_datatype_ptr_is_unique(_uniq_xs_0_10122_10399, _ctx)) {
          kk_datatype_ptr_free(_uniq_xs_0_10122_10399, _ctx);
        }
        else {
          kk_string_dup(_uniq_x_10402, _ctx);
          kk_datatype_ptr_decref(_uniq_xs_0_10122_10399, _ctx);
        }
        return _uniq_x_10402;
      }
    }
    {
      struct kk_std_core_types_Cons* _con_x640 = kk_std_core_types__as_Cons(_uniq_xs_0_10122_10399, _ctx);
      kk_box_t _box_x222 = _con_x640->head;
      kk_std_core_types__list _uniq_pat_3_0_10406 = _con_x640->tail;
      struct kk_std_core_types_Cons* _con_x641 = kk_std_core_types__as_Cons(_uniq_pat_3_0_10406, _ctx);
      kk_box_t _box_x223 = _con_x641->head;
      kk_std_core_types__list _uniq_pat_4_0_10408 = _con_x641->tail;
      if (kk_std_core_types__is_Nil(_uniq_pat_4_0_10408, _ctx)) {
        kk_string_t _uniq_x_0_10405 = kk_string_unbox(_box_x222);
        kk_string_t _uniq_y_10407 = kk_string_unbox(_box_x223);
        if kk_likely(kk_datatype_ptr_is_unique(_uniq_xs_0_10122_10399, _ctx)) {
          if kk_likely(kk_datatype_ptr_is_unique(_uniq_pat_3_0_10406, _ctx)) {
            kk_datatype_ptr_free(_uniq_pat_3_0_10406, _ctx);
          }
          else {
            kk_string_dup(_uniq_y_10407, _ctx);
            kk_datatype_ptr_decref(_uniq_pat_3_0_10406, _ctx);
          }
          kk_datatype_ptr_free(_uniq_xs_0_10122_10399, _ctx);
        }
        else {
          kk_string_dup(_uniq_x_0_10405, _ctx);
          kk_string_dup(_uniq_y_10407, _ctx);
          kk_datatype_ptr_decref(_uniq_xs_0_10122_10399, _ctx);
        }
        return kk_std_core_types__lp__plus__plus__rp_(_uniq_x_0_10405, _uniq_y_10407, _ctx);
      }
    }
    {
      kk_vector_t _x_x642 = kk_std_core_vector_unvlist(_uniq_xs_0_10122_10399, _ctx); /*vector<911>*/
      return kk_string_join(_x_x642,kk_context());
    }
  }
  {
    return kk_std_core_list_reverse_join(_uniq_acc_10379, _ctx);
  }
}
 
// How many groups are captured by this regex?

kk_integer_t kk_std_text_regex_groups_count(kk_std_text_regex__regex r, kk_context_t* _ctx) { /* (r : regex) -> int */ 
  kk_string_t _x_x643;
  bool _match_x338;
  kk_string_t _x_x644 = kk_string_empty(); /*string*/
  kk_string_t _x_x646;
  kk_define_string_literal(, _s_x647, 1, "$", _ctx)
  _x_x646 = kk_string_dup(_s_x647, _ctx); /*string*/
  _match_x338 = kk_string_contains(_x_x644,_x_x646,kk_context()); /*bool*/
  if (_match_x338) {
    kk_std_core_types__list _x_x648;
    kk_box_t _x_x649;
    kk_std_text_regex__regex _x_x650 = kk_std_text_regex_rx_nongroup; /*std/text/regex/regex*/
    {
      kk_box_t _x = _x_x650.obj;
      _x_x649 = kk_box_dup(_x, _ctx); /*any*/
    }
    kk_string_t _x_x651;
    {
      kk_string_t _x_1 = r.src;
      kk_string_dup(_x_1, _ctx);
      kk_std_text_regex__regex_drop(r, _ctx);
      _x_x651 = _x_1; /*string*/
    }
    kk_ssize_t _x_x652;
    kk_integer_t _x_x653;
    kk_std_core_types__optional _match_x340 = kk_std_core_types__new_None(_ctx); /*forall<a> ? a*/;
    if (kk_std_core_types__is_Optional(_match_x340, _ctx)) {
      kk_box_t _box_x228 = _match_x340._cons._Optional.value;
      kk_integer_t _uniq_atmost_487 = kk_integer_unbox(_box_x228, _ctx);
      kk_integer_dup(_uniq_atmost_487, _ctx);
      kk_std_core_types__optional_drop(_match_x340, _ctx);
      _x_x653 = _uniq_atmost_487; /*int*/
    }
    else {
      kk_std_core_types__optional_drop(_match_x340, _ctx);
      _x_x653 = kk_integer_from_small(-1); /*int*/
    }
    _x_x652 = kk_std_core_int_ssize__t(_x_x653, _ctx); /*ssize_t*/
    _x_x648 = kk_std_text_regex_regex_exec_all(_x_x649, _x_x651, (KK_IZ(0)), _x_x652, _ctx); /*list<list<sslice/sslice>>*/
    _x_x643 = kk_std_text_regex__lift_groups_count_10528(_x_x648, kk_std_core_types__new_Nil(_ctx), _ctx); /*string*/
  }
  else {
    kk_std_core_types__list _x_x654;
    kk_box_t _x_x655;
    kk_std_text_regex__regex _x_x656 = kk_std_text_regex_rx_nongroup; /*std/text/regex/regex*/
    {
      kk_box_t _x_0 = _x_x656.obj;
      _x_x655 = kk_box_dup(_x_0, _ctx); /*any*/
    }
    kk_string_t _x_x657;
    {
      kk_string_t _x_1_0 = r.src;
      kk_string_dup(_x_1_0, _ctx);
      kk_std_text_regex__regex_drop(r, _ctx);
      _x_x657 = _x_1_0; /*string*/
    }
    kk_ssize_t _x_x658;
    kk_integer_t _x_x659;
    kk_std_core_types__optional _match_x339 = kk_std_core_types__new_None(_ctx); /*forall<a> ? a*/;
    if (kk_std_core_types__is_Optional(_match_x339, _ctx)) {
      kk_box_t _box_x229 = _match_x339._cons._Optional.value;
      kk_integer_t _uniq_atmost_487_0 = kk_integer_unbox(_box_x229, _ctx);
      kk_integer_dup(_uniq_atmost_487_0, _ctx);
      kk_std_core_types__optional_drop(_match_x339, _ctx);
      _x_x659 = _uniq_atmost_487_0; /*int*/
    }
    else {
      kk_std_core_types__optional_drop(_match_x339, _ctx);
      _x_x659 = kk_integer_from_small(-1); /*int*/
    }
    _x_x658 = kk_std_core_int_ssize__t(_x_x659, _ctx); /*ssize_t*/
    _x_x654 = kk_std_text_regex_regex_exec_all(_x_x655, _x_x657, (KK_IZ(0)), _x_x658, _ctx); /*list<list<sslice/sslice>>*/
    _x_x643 = kk_std_text_regex__lift_groups_count_10529(_x_x654, kk_std_core_types__new_Nil(_ctx), _ctx); /*string*/
  }
  return kk_std_core_string_chars_fs_count(_x_x643, _ctx);
}
 
// Replace the first occurrence of `regex` by the result of the replacement fun `repl` in a string `s`.

kk_string_t kk_std_text_regex_replace(kk_string_t s, kk_std_text_regex__regex r, kk_function_t repl, kk_context_t* _ctx) { /* forall<e> (s : string, r : regex, repl : (list<sslice/sslice>) -> e string) -> e string */ 
  kk_std_core_types__list _x_x660;
  kk_box_t _x_x661;
  {
    kk_box_t _x = r.obj;
    kk_box_dup(_x, _ctx);
    kk_std_text_regex__regex_drop(r, _ctx);
    _x_x661 = _x; /*any*/
  }
  _x_x660 = kk_std_text_regex_regex_exec_all(_x_x661, s, (KK_IZ(0)), (KK_IZ(1)), _ctx); /*list<list<sslice/sslice>>*/
  return kk_std_text_regex_concat_replace(_x_x660, repl, kk_std_core_types__new_Nil(_ctx), _ctx);
}
 
// lifted local: string/replace, @spec-x10463
// specialized: std/text/regex/concat-replace, on parameters @uniq-repl@10422, using:
// @uniq-repl@10422 = fn(caps: (list :: V -> V)<sslice/sslice>){
//   std/text/regex/replace-captures(caps, repl);
// }

kk_string_t kk_std_text_regex_string_fs__lift_replace_10530(kk_string_t repl, kk_std_core_types__list _uniq_matches_10421, kk_std_core_types__list _uniq_acc_10423, kk_context_t* _ctx) { /* (repl : string, list<list<sslice/sslice>>, list<string>) -> string */ 
  kk__tailcall: ;
  if (kk_std_core_types__is_Cons(_uniq_matches_10421, _ctx)) {
    struct kk_std_core_types_Cons* _con_x662 = kk_std_core_types__as_Cons(_uniq_matches_10421, _ctx);
    kk_box_t _box_x230 = _con_x662->head;
    kk_std_core_types__list _uniq_pat_0_10426 = _con_x662->tail;
    if (kk_std_core_types__is_Cons(_uniq_pat_0_10426, _ctx)) {
      struct kk_std_core_types_Cons* _con_x663 = kk_std_core_types__as_Cons(_uniq_pat_0_10426, _ctx);
      kk_box_t _box_x231 = _con_x663->head;
      kk_std_core_types__list _uniq_pre_10425 = kk_std_core_types__list_unbox(_box_x230, KK_BORROWED, _ctx);
      kk_std_core_types__list _uniq_mm_10428 = _con_x663->tail;
      kk_std_core_types__list _uniq_m_10427 = kk_std_core_types__list_unbox(_box_x231, KK_BORROWED, _ctx);
      kk_reuse_t _ru_x435 = kk_reuse_null; /*@reuse*/;
      kk_reuse_t _ru_x434 = kk_reuse_null; /*@reuse*/;
      if kk_likely(kk_datatype_ptr_is_unique(_uniq_matches_10421, _ctx)) {
        if kk_likely(kk_datatype_ptr_is_unique(_uniq_pat_0_10426, _ctx)) {
          _ru_x434 = (kk_datatype_ptr_reuse(_uniq_pat_0_10426, _ctx));
        }
        else {
          kk_std_core_types__list_dup(_uniq_m_10427, _ctx);
          kk_std_core_types__list_dup(_uniq_mm_10428, _ctx);
          kk_datatype_ptr_decref(_uniq_pat_0_10426, _ctx);
        }
        _ru_x435 = (kk_datatype_ptr_reuse(_uniq_matches_10421, _ctx));
      }
      else {
        kk_std_core_types__list_dup(_uniq_m_10427, _ctx);
        kk_std_core_types__list_dup(_uniq_mm_10428, _ctx);
        kk_std_core_types__list_dup(_uniq_pre_10425, _ctx);
        kk_datatype_ptr_decref(_uniq_matches_10421, _ctx);
      }
      kk_string_t _b_x235_237;
      kk_string_t _x_x664 = kk_string_dup(repl, _ctx); /*string*/
      _b_x235_237 = kk_std_text_regex_replace_captures(_uniq_m_10427, _x_x664, _ctx); /*string*/
      kk_string_t _b_x233_239;
      if (kk_std_core_types__is_Cons(_uniq_pre_10425, _ctx)) {
        struct kk_std_core_types_Cons* _con_x665 = kk_std_core_types__as_Cons(_uniq_pre_10425, _ctx);
        kk_box_t _box_x232 = _con_x665->head;
        kk_std_core_sslice__sslice _uniq_s_10431 = kk_std_core_sslice__sslice_unbox(_box_x232, KK_BORROWED, _ctx);
        kk_std_core_types__list _uniq_pat_3_10435 = _con_x665->tail;
        if kk_likely(kk_datatype_ptr_is_unique(_uniq_pre_10425, _ctx)) {
          kk_std_core_types__list_drop(_uniq_pat_3_10435, _ctx);
          kk_std_core_sslice__sslice_dup(_uniq_s_10431, _ctx);
          kk_box_drop(_box_x232, _ctx);
          kk_datatype_ptr_free(_uniq_pre_10425, _ctx);
        }
        else {
          kk_std_core_sslice__sslice_dup(_uniq_s_10431, _ctx);
          kk_datatype_ptr_decref(_uniq_pre_10425, _ctx);
        }
        _b_x233_239 = kk_std_core_sslice_string(_uniq_s_10431, _ctx); /*string*/
      }
      else {
        _b_x233_239 = kk_string_empty(); /*string*/
      }
      { // tailcall
        kk_std_core_types__list _x_x667;
        kk_std_core_types__list _x_x668 = kk_std_core_types__new_Cons(_ru_x435, 0, kk_string_box(_b_x233_239), _uniq_acc_10423, _ctx); /*list<82>*/
        _x_x667 = kk_std_core_types__new_Cons(_ru_x434, 0, kk_string_box(_b_x235_237), _x_x668, _ctx); /*list<82>*/
        _uniq_matches_10421 = _uniq_mm_10428;
        _uniq_acc_10423 = _x_x667;
        goto kk__tailcall;
      }
    }
  }
  if (kk_std_core_types__is_Cons(_uniq_matches_10421, _ctx)) {
    struct kk_std_core_types_Cons* _con_x669 = kk_std_core_types__as_Cons(_uniq_matches_10421, _ctx);
    kk_box_t _box_x241 = _con_x669->head;
    kk_std_core_types__list _uniq_pat_2_0_10439 = _con_x669->tail;
    kk_std_core_types__list _uniq_post_10438 = kk_std_core_types__list_unbox(_box_x241, KK_BORROWED, _ctx);
    kk_string_drop(repl, _ctx);
    kk_reuse_t _ru_x437 = kk_reuse_null; /*@reuse*/;
    if kk_likely(kk_datatype_ptr_is_unique(_uniq_matches_10421, _ctx)) {
      _ru_x437 = (kk_datatype_ptr_reuse(_uniq_matches_10421, _ctx));
    }
    else {
      kk_std_core_types__list_dup(_uniq_post_10438, _ctx);
      kk_datatype_ptr_decref(_uniq_matches_10421, _ctx);
    }
    kk_string_t _b_x243_245;
    if (kk_std_core_types__is_Cons(_uniq_post_10438, _ctx)) {
      struct kk_std_core_types_Cons* _con_x670 = kk_std_core_types__as_Cons(_uniq_post_10438, _ctx);
      kk_box_t _box_x242 = _con_x670->head;
      kk_std_core_sslice__sslice _uniq_s_0_10456 = kk_std_core_sslice__sslice_unbox(_box_x242, KK_BORROWED, _ctx);
      kk_std_core_types__list _uniq_pat_3_1_10460 = _con_x670->tail;
      if kk_likely(kk_datatype_ptr_is_unique(_uniq_post_10438, _ctx)) {
        kk_std_core_types__list_drop(_uniq_pat_3_1_10460, _ctx);
        kk_std_core_sslice__sslice_dup(_uniq_s_0_10456, _ctx);
        kk_box_drop(_box_x242, _ctx);
        kk_datatype_ptr_free(_uniq_post_10438, _ctx);
      }
      else {
        kk_std_core_sslice__sslice_dup(_uniq_s_0_10456, _ctx);
        kk_datatype_ptr_decref(_uniq_post_10438, _ctx);
      }
      _b_x243_245 = kk_std_core_sslice_string(_uniq_s_0_10456, _ctx); /*string*/
    }
    else {
      _b_x243_245 = kk_string_empty(); /*string*/
    }
    kk_std_core_types__list _uniq_xs_10440 = kk_std_core_types__new_Cons(_ru_x437, 0, kk_string_box(_b_x243_245), _uniq_acc_10423, _ctx); /*list<string>*/;
    kk_std_core_types__list _uniq_xs_0_10122_10443;
    if (kk_std_core_types__is_Nil(_uniq_xs_10440, _ctx)) {
      _uniq_xs_0_10122_10443 = kk_std_core_types__new_Nil(_ctx); /*list<string>*/
    }
    else {
      _uniq_xs_0_10122_10443 = kk_std_core_list__unroll_reverse_acc_10003(kk_std_core_types__new_Nil(_ctx), _uniq_xs_10440, _ctx); /*list<string>*/
    }
    if (kk_std_core_types__is_Nil(_uniq_xs_0_10122_10443, _ctx)) {
      return kk_string_empty();
    }
    {
      struct kk_std_core_types_Cons* _con_x673 = kk_std_core_types__as_Cons(_uniq_xs_0_10122_10443, _ctx);
      kk_box_t _box_x247 = _con_x673->head;
      kk_std_core_types__list _uniq_pat_1_2_10447 = _con_x673->tail;
      if (kk_std_core_types__is_Nil(_uniq_pat_1_2_10447, _ctx)) {
        kk_string_t _uniq_x_10446 = kk_string_unbox(_box_x247);
        if kk_likely(kk_datatype_ptr_is_unique(_uniq_xs_0_10122_10443, _ctx)) {
          kk_datatype_ptr_free(_uniq_xs_0_10122_10443, _ctx);
        }
        else {
          kk_string_dup(_uniq_x_10446, _ctx);
          kk_datatype_ptr_decref(_uniq_xs_0_10122_10443, _ctx);
        }
        return _uniq_x_10446;
      }
    }
    {
      struct kk_std_core_types_Cons* _con_x674 = kk_std_core_types__as_Cons(_uniq_xs_0_10122_10443, _ctx);
      kk_box_t _box_x248 = _con_x674->head;
      kk_std_core_types__list _uniq_pat_3_0_10450 = _con_x674->tail;
      struct kk_std_core_types_Cons* _con_x675 = kk_std_core_types__as_Cons(_uniq_pat_3_0_10450, _ctx);
      kk_box_t _box_x249 = _con_x675->head;
      kk_std_core_types__list _uniq_pat_4_0_10452 = _con_x675->tail;
      if (kk_std_core_types__is_Nil(_uniq_pat_4_0_10452, _ctx)) {
        kk_string_t _uniq_x_0_10449 = kk_string_unbox(_box_x248);
        kk_string_t _uniq_y_10451 = kk_string_unbox(_box_x249);
        if kk_likely(kk_datatype_ptr_is_unique(_uniq_xs_0_10122_10443, _ctx)) {
          if kk_likely(kk_datatype_ptr_is_unique(_uniq_pat_3_0_10450, _ctx)) {
            kk_datatype_ptr_free(_uniq_pat_3_0_10450, _ctx);
          }
          else {
            kk_string_dup(_uniq_y_10451, _ctx);
            kk_datatype_ptr_decref(_uniq_pat_3_0_10450, _ctx);
          }
          kk_datatype_ptr_free(_uniq_xs_0_10122_10443, _ctx);
        }
        else {
          kk_string_dup(_uniq_x_0_10449, _ctx);
          kk_string_dup(_uniq_y_10451, _ctx);
          kk_datatype_ptr_decref(_uniq_xs_0_10122_10443, _ctx);
        }
        return kk_std_core_types__lp__plus__plus__rp_(_uniq_x_0_10449, _uniq_y_10451, _ctx);
      }
    }
    {
      kk_vector_t _x_x676 = kk_std_core_vector_unvlist(_uniq_xs_0_10122_10443, _ctx); /*vector<911>*/
      return kk_string_join(_x_x676,kk_context());
    }
  }
  {
    kk_string_drop(repl, _ctx);
    return kk_std_core_list_reverse_join(_uniq_acc_10423, _ctx);
  }
}
 
// lifted local: string/replace, @spec-x10507
// specialized: std/text/regex/concat-replace, on parameters @uniq-repl@10466, using:
// @uniq-repl@10466 = fn(@_wildcard-x158_35: (list :: V -> V)<sslice/sslice>){
//   repl;
// }

kk_string_t kk_std_text_regex_string_fs__lift_replace_10531(kk_string_t repl, kk_std_core_types__list _uniq_matches_10465, kk_std_core_types__list _uniq_acc_10467, kk_context_t* _ctx) { /* (repl : string, list<list<sslice/sslice>>, list<string>) -> string */ 
  kk__tailcall: ;
  if (kk_std_core_types__is_Cons(_uniq_matches_10465, _ctx)) {
    struct kk_std_core_types_Cons* _con_x677 = kk_std_core_types__as_Cons(_uniq_matches_10465, _ctx);
    kk_box_t _box_x250 = _con_x677->head;
    kk_std_core_types__list _uniq_pat_0_10470 = _con_x677->tail;
    if (kk_std_core_types__is_Cons(_uniq_pat_0_10470, _ctx)) {
      struct kk_std_core_types_Cons* _con_x678 = kk_std_core_types__as_Cons(_uniq_pat_0_10470, _ctx);
      kk_box_t _box_x251 = _con_x678->head;
      kk_std_core_types__list _uniq_pre_10469 = kk_std_core_types__list_unbox(_box_x250, KK_BORROWED, _ctx);
      kk_std_core_types__list _uniq_mm_10472 = _con_x678->tail;
      kk_std_core_types__list _uniq_m_10471 = kk_std_core_types__list_unbox(_box_x251, KK_BORROWED, _ctx);
      kk_reuse_t _ru_x443 = kk_reuse_null; /*@reuse*/;
      kk_reuse_t _ru_x442 = kk_reuse_null; /*@reuse*/;
      if kk_likely(kk_datatype_ptr_is_unique(_uniq_matches_10465, _ctx)) {
        if kk_likely(kk_datatype_ptr_is_unique(_uniq_pat_0_10470, _ctx)) {
          kk_std_core_types__list_drop(_uniq_m_10471, _ctx);
          _ru_x442 = (kk_datatype_ptr_reuse(_uniq_pat_0_10470, _ctx));
        }
        else {
          kk_std_core_types__list_dup(_uniq_mm_10472, _ctx);
          kk_datatype_ptr_decref(_uniq_pat_0_10470, _ctx);
        }
        _ru_x443 = (kk_datatype_ptr_reuse(_uniq_matches_10465, _ctx));
      }
      else {
        kk_std_core_types__list_dup(_uniq_mm_10472, _ctx);
        kk_std_core_types__list_dup(_uniq_pre_10469, _ctx);
        kk_datatype_ptr_decref(_uniq_matches_10465, _ctx);
      }
      kk_string_t _b_x253_259;
      if (kk_std_core_types__is_Cons(_uniq_pre_10469, _ctx)) {
        struct kk_std_core_types_Cons* _con_x679 = kk_std_core_types__as_Cons(_uniq_pre_10469, _ctx);
        kk_box_t _box_x252 = _con_x679->head;
        kk_std_core_sslice__sslice _uniq_s_10475 = kk_std_core_sslice__sslice_unbox(_box_x252, KK_BORROWED, _ctx);
        kk_std_core_types__list _uniq_pat_3_10479 = _con_x679->tail;
        if kk_likely(kk_datatype_ptr_is_unique(_uniq_pre_10469, _ctx)) {
          kk_std_core_types__list_drop(_uniq_pat_3_10479, _ctx);
          kk_std_core_sslice__sslice_dup(_uniq_s_10475, _ctx);
          kk_box_drop(_box_x252, _ctx);
          kk_datatype_ptr_free(_uniq_pre_10469, _ctx);
        }
        else {
          kk_std_core_sslice__sslice_dup(_uniq_s_10475, _ctx);
          kk_datatype_ptr_decref(_uniq_pre_10469, _ctx);
        }
        _b_x253_259 = kk_std_core_sslice_string(_uniq_s_10475, _ctx); /*string*/
      }
      else {
        _b_x253_259 = kk_string_empty(); /*string*/
      }
      { // tailcall
        kk_string_t _x_x681 = kk_string_dup(repl, _ctx); /*string*/
        kk_std_core_types__list _x_x682;
        kk_std_core_types__list _x_x683 = kk_std_core_types__new_Cons(_ru_x443, 0, kk_string_box(_b_x253_259), _uniq_acc_10467, _ctx); /*list<82>*/
        _x_x682 = kk_std_core_types__new_Cons(_ru_x442, 0, kk_string_box(repl), _x_x683, _ctx); /*list<82>*/
        repl = _x_x681;
        _uniq_matches_10465 = _uniq_mm_10472;
        _uniq_acc_10467 = _x_x682;
        goto kk__tailcall;
      }
    }
  }
  if (kk_std_core_types__is_Cons(_uniq_matches_10465, _ctx)) {
    struct kk_std_core_types_Cons* _con_x684 = kk_std_core_types__as_Cons(_uniq_matches_10465, _ctx);
    kk_box_t _box_x261 = _con_x684->head;
    kk_std_core_types__list _uniq_pat_2_0_10483 = _con_x684->tail;
    kk_std_core_types__list _uniq_post_10482 = kk_std_core_types__list_unbox(_box_x261, KK_BORROWED, _ctx);
    kk_string_drop(repl, _ctx);
    kk_reuse_t _ru_x445 = kk_reuse_null; /*@reuse*/;
    if kk_likely(kk_datatype_ptr_is_unique(_uniq_matches_10465, _ctx)) {
      _ru_x445 = (kk_datatype_ptr_reuse(_uniq_matches_10465, _ctx));
    }
    else {
      kk_std_core_types__list_dup(_uniq_post_10482, _ctx);
      kk_datatype_ptr_decref(_uniq_matches_10465, _ctx);
    }
    kk_string_t _b_x263_265;
    if (kk_std_core_types__is_Cons(_uniq_post_10482, _ctx)) {
      struct kk_std_core_types_Cons* _con_x685 = kk_std_core_types__as_Cons(_uniq_post_10482, _ctx);
      kk_box_t _box_x262 = _con_x685->head;
      kk_std_core_sslice__sslice _uniq_s_0_10500 = kk_std_core_sslice__sslice_unbox(_box_x262, KK_BORROWED, _ctx);
      kk_std_core_types__list _uniq_pat_3_1_10504 = _con_x685->tail;
      if kk_likely(kk_datatype_ptr_is_unique(_uniq_post_10482, _ctx)) {
        kk_std_core_types__list_drop(_uniq_pat_3_1_10504, _ctx);
        kk_std_core_sslice__sslice_dup(_uniq_s_0_10500, _ctx);
        kk_box_drop(_box_x262, _ctx);
        kk_datatype_ptr_free(_uniq_post_10482, _ctx);
      }
      else {
        kk_std_core_sslice__sslice_dup(_uniq_s_0_10500, _ctx);
        kk_datatype_ptr_decref(_uniq_post_10482, _ctx);
      }
      _b_x263_265 = kk_std_core_sslice_string(_uniq_s_0_10500, _ctx); /*string*/
    }
    else {
      _b_x263_265 = kk_string_empty(); /*string*/
    }
    kk_std_core_types__list _uniq_xs_10484 = kk_std_core_types__new_Cons(_ru_x445, 0, kk_string_box(_b_x263_265), _uniq_acc_10467, _ctx); /*list<string>*/;
    kk_std_core_types__list _uniq_xs_0_10122_10487;
    if (kk_std_core_types__is_Nil(_uniq_xs_10484, _ctx)) {
      _uniq_xs_0_10122_10487 = kk_std_core_types__new_Nil(_ctx); /*list<string>*/
    }
    else {
      _uniq_xs_0_10122_10487 = kk_std_core_list__unroll_reverse_acc_10003(kk_std_core_types__new_Nil(_ctx), _uniq_xs_10484, _ctx); /*list<string>*/
    }
    if (kk_std_core_types__is_Nil(_uniq_xs_0_10122_10487, _ctx)) {
      return kk_string_empty();
    }
    {
      struct kk_std_core_types_Cons* _con_x688 = kk_std_core_types__as_Cons(_uniq_xs_0_10122_10487, _ctx);
      kk_box_t _box_x267 = _con_x688->head;
      kk_std_core_types__list _uniq_pat_1_2_10491 = _con_x688->tail;
      if (kk_std_core_types__is_Nil(_uniq_pat_1_2_10491, _ctx)) {
        kk_string_t _uniq_x_10490 = kk_string_unbox(_box_x267);
        if kk_likely(kk_datatype_ptr_is_unique(_uniq_xs_0_10122_10487, _ctx)) {
          kk_datatype_ptr_free(_uniq_xs_0_10122_10487, _ctx);
        }
        else {
          kk_string_dup(_uniq_x_10490, _ctx);
          kk_datatype_ptr_decref(_uniq_xs_0_10122_10487, _ctx);
        }
        return _uniq_x_10490;
      }
    }
    {
      struct kk_std_core_types_Cons* _con_x689 = kk_std_core_types__as_Cons(_uniq_xs_0_10122_10487, _ctx);
      kk_box_t _box_x268 = _con_x689->head;
      kk_std_core_types__list _uniq_pat_3_0_10494 = _con_x689->tail;
      struct kk_std_core_types_Cons* _con_x690 = kk_std_core_types__as_Cons(_uniq_pat_3_0_10494, _ctx);
      kk_box_t _box_x269 = _con_x690->head;
      kk_std_core_types__list _uniq_pat_4_0_10496 = _con_x690->tail;
      if (kk_std_core_types__is_Nil(_uniq_pat_4_0_10496, _ctx)) {
        kk_string_t _uniq_x_0_10493 = kk_string_unbox(_box_x268);
        kk_string_t _uniq_y_10495 = kk_string_unbox(_box_x269);
        if kk_likely(kk_datatype_ptr_is_unique(_uniq_xs_0_10122_10487, _ctx)) {
          if kk_likely(kk_datatype_ptr_is_unique(_uniq_pat_3_0_10494, _ctx)) {
            kk_datatype_ptr_free(_uniq_pat_3_0_10494, _ctx);
          }
          else {
            kk_string_dup(_uniq_y_10495, _ctx);
            kk_datatype_ptr_decref(_uniq_pat_3_0_10494, _ctx);
          }
          kk_datatype_ptr_free(_uniq_xs_0_10122_10487, _ctx);
        }
        else {
          kk_string_dup(_uniq_x_0_10493, _ctx);
          kk_string_dup(_uniq_y_10495, _ctx);
          kk_datatype_ptr_decref(_uniq_xs_0_10122_10487, _ctx);
        }
        return kk_std_core_types__lp__plus__plus__rp_(_uniq_x_0_10493, _uniq_y_10495, _ctx);
      }
    }
    {
      kk_vector_t _x_x691 = kk_std_core_vector_unvlist(_uniq_xs_0_10122_10487, _ctx); /*vector<911>*/
      return kk_string_join(_x_x691,kk_context());
    }
  }
  {
    kk_string_drop(repl, _ctx);
    return kk_std_core_list_reverse_join(_uniq_acc_10467, _ctx);
  }
}
 
// Replace the first occurrence of `regex` with a replacement string `repl` in a string `s`.
// The replacement string can contain `$$` for a `$` sign, `$n` for a capture group,
// `$&` for the entire match `==$0`.

kk_string_t kk_std_text_regex_string_fs_replace(kk_string_t s, kk_std_text_regex__regex regex_0, kk_string_t repl, kk_context_t* _ctx) { /* (s : string, regex : regex, repl : string) -> string */ 
  bool _match_x335;
  kk_string_t _x_x692 = kk_string_dup(repl, _ctx); /*string*/
  kk_string_t _x_x693;
  kk_define_string_literal(, _s_x694, 1, "$", _ctx)
  _x_x693 = kk_string_dup(_s_x694, _ctx); /*string*/
  _match_x335 = kk_string_contains(_x_x692,_x_x693,kk_context()); /*bool*/
  if (_match_x335) {
    kk_std_core_types__list _uniq_matches_10421;
    kk_box_t _x_x695;
    {
      kk_box_t _x = regex_0.obj;
      kk_box_dup(_x, _ctx);
      kk_std_text_regex__regex_drop(regex_0, _ctx);
      _x_x695 = _x; /*any*/
    }
    _uniq_matches_10421 = kk_std_text_regex_regex_exec_all(_x_x695, s, (KK_IZ(0)), (KK_IZ(1)), _ctx); /*list<list<sslice/sslice>>*/
    return kk_std_text_regex_string_fs__lift_replace_10530(repl, _uniq_matches_10421, kk_std_core_types__new_Nil(_ctx), _ctx);
  }
  {
    kk_std_core_types__list _uniq_matches_10465;
    kk_box_t _x_x696;
    {
      kk_box_t _x_0 = regex_0.obj;
      kk_box_dup(_x_0, _ctx);
      kk_std_text_regex__regex_drop(regex_0, _ctx);
      _x_x696 = _x_0; /*any*/
    }
    _uniq_matches_10465 = kk_std_text_regex_regex_exec_all(_x_x696, s, (KK_IZ(0)), (KK_IZ(1)), _ctx); /*list<list<sslice/sslice>>*/
    return kk_std_text_regex_string_fs__lift_replace_10531(repl, _uniq_matches_10465, kk_std_core_types__new_Nil(_ctx), _ctx);
  }
}
 
// Filter only for the non-matched parts.

kk_std_core_types__list kk_std_text_regex__trmc_filter_non_matches(kk_std_core_types__list xs, kk_std_core_types__cctx _acc, kk_context_t* _ctx) { /* (xs : list<list<sslice/sslice>>, ctx<list<list<sslice/sslice>>>) -> list<list<sslice/sslice>> */ 
  kk__tailcall: ;
  if (kk_std_core_types__is_Cons(xs, _ctx)) {
    struct kk_std_core_types_Cons* _con_x697 = kk_std_core_types__as_Cons(xs, _ctx);
    kk_box_t _box_x270 = _con_x697->head;
    kk_std_core_types__list _pat_0 = _con_x697->tail;
    if (kk_std_core_types__is_Cons(_pat_0, _ctx)) {
      struct kk_std_core_types_Cons* _con_x698 = kk_std_core_types__as_Cons(_pat_0, _ctx);
      kk_box_t _box_x271 = _con_x698->head;
      kk_std_core_types__list s = kk_std_core_types__list_unbox(_box_x270, KK_BORROWED, _ctx);
      kk_std_core_types__list xx = _con_x698->tail;
      kk_std_core_types__list _pat_1 = kk_std_core_types__list_unbox(_box_x271, KK_BORROWED, _ctx);
      kk_reuse_t _ru_x451 = kk_reuse_null; /*@reuse*/;
      if kk_likely(kk_datatype_ptr_is_unique(xs, _ctx)) {
        if kk_likely(kk_datatype_ptr_is_unique(_pat_0, _ctx)) {
          kk_std_core_types__list_drop(_pat_1, _ctx);
          kk_datatype_ptr_free(_pat_0, _ctx);
        }
        else {
          kk_std_core_types__list_dup(xx, _ctx);
          kk_datatype_ptr_decref(_pat_0, _ctx);
        }
        _ru_x451 = (kk_datatype_ptr_reuse(xs, _ctx));
      }
      else {
        kk_std_core_types__list_dup(s, _ctx);
        kk_std_core_types__list_dup(xx, _ctx);
        kk_datatype_ptr_decref(xs, _ctx);
      }
      kk_std_core_types__list _trmc_x10554 = kk_datatype_null(); /*list<list<sslice/sslice>>*/;
      kk_std_core_types__list _trmc_x10555 = kk_std_core_types__new_Cons(_ru_x451, 0, kk_std_core_types__list_box(s, _ctx), _trmc_x10554, _ctx); /*list<list<sslice/sslice>>*/;
      kk_field_addr_t _b_x281_291 = kk_field_addr_create(&kk_std_core_types__as_Cons(_trmc_x10555, _ctx)->tail, _ctx); /*@field-addr<list<list<sslice/sslice>>>*/;
      { // tailcall
        kk_std_core_types__cctx _x_x699 = kk_cctx_extend_linear(_acc,(kk_std_core_types__list_box(_trmc_x10555, _ctx)),_b_x281_291,kk_context()); /*ctx<0>*/
        xs = xx;
        _acc = _x_x699;
        goto kk__tailcall;
      }
    }
  }
  if (kk_std_core_types__is_Cons(xs, _ctx)) {
    struct kk_std_core_types_Cons* _con_x700 = kk_std_core_types__as_Cons(xs, _ctx);
    kk_box_t _box_x282 = _con_x700->head;
    kk_std_core_types__list _pat_3 = _con_x700->tail;
    kk_std_core_types__list s_0 = kk_std_core_types__list_unbox(_box_x282, KK_BORROWED, _ctx);
    kk_reuse_t _ru_x452 = kk_reuse_null; /*@reuse*/;
    if kk_likely(kk_datatype_ptr_is_unique(xs, _ctx)) {
      _ru_x452 = (kk_datatype_ptr_reuse(xs, _ctx));
    }
    else {
      kk_std_core_types__list_dup(s_0, _ctx);
      kk_datatype_ptr_decref(xs, _ctx);
    }
    kk_box_t _x_x701;
    kk_box_t _x_x702;
    kk_std_core_types__list _x_x703 = kk_std_core_types__new_Cons(_ru_x452, 0, kk_std_core_types__list_box(s_0, _ctx), kk_std_core_types__new_Nil(_ctx), _ctx); /*list<82>*/
    _x_x702 = kk_std_core_types__list_box(_x_x703, _ctx); /*-1*/
    _x_x701 = kk_cctx_apply_linear(_acc,_x_x702,kk_context()); /*-1*/
    return kk_std_core_types__list_unbox(_x_x701, KK_OWNED, _ctx);
  }
  {
    kk_box_t _x_x704 = kk_cctx_apply_linear(_acc,(kk_std_core_types__list_box(kk_std_core_types__new_Nil(_ctx), _ctx)),kk_context()); /*-1*/
    return kk_std_core_types__list_unbox(_x_x704, KK_OWNED, _ctx);
  }
}
 
// Filter only for the non-matched parts.

kk_std_core_types__list kk_std_text_regex_filter_non_matches(kk_std_core_types__list xs_0, kk_context_t* _ctx) { /* (xs : list<list<sslice/sslice>>) -> list<list<sslice/sslice>> */ 
  kk_std_core_types__cctx _x_x705 = kk_cctx_empty(kk_context()); /*ctx<0>*/
  return kk_std_text_regex__trmc_filter_non_matches(xs_0, _x_x705, _ctx);
}
 
// lifted local: @lift-split@10532, split, @spec-x10520
// specialized: std/core/list/@unroll-map@10010, on parameters @uniq-f@10515, using:
// @uniq-f@10515 = std/text/regex/captured

kk_std_core_types__list kk_std_text_regex__trmc_lift_split_10533(kk_std_core_types__list _uniq_xs_10514, kk_std_core_types__cctx _acc, kk_context_t* _ctx) { /* (list<list<sslice/sslice>>, ctx<list<string>>) -> list<string> */ 
  if (kk_std_core_types__is_Cons(_uniq_xs_10514, _ctx)) {
    struct kk_std_core_types_Cons* _con_x706 = kk_std_core_types__as_Cons(_uniq_xs_10514, _ctx);
    kk_box_t _box_x301 = _con_x706->head;
    kk_std_core_types__list _uniq_xx_10518 = _con_x706->tail;
    kk_std_core_types__list _uniq_x_10517 = kk_std_core_types__list_unbox(_box_x301, KK_BORROWED, _ctx);
    kk_reuse_t _ru_x453 = kk_reuse_null; /*@reuse*/;
    if kk_likely(kk_datatype_ptr_is_unique(_uniq_xs_10514, _ctx)) {
      _ru_x453 = (kk_datatype_ptr_reuse(_uniq_xs_10514, _ctx));
    }
    else {
      kk_std_core_types__list_dup(_uniq_x_10517, _ctx);
      kk_std_core_types__list_dup(_uniq_xx_10518, _ctx);
      kk_datatype_ptr_decref(_uniq_xs_10514, _ctx);
    }
    kk_string_t _trmc_x10556;
    if (kk_std_core_types__is_Cons(_uniq_x_10517, _ctx)) {
      struct kk_std_core_types_Cons* _con_x707 = kk_std_core_types__as_Cons(_uniq_x_10517, _ctx);
      kk_box_t _box_x302 = _con_x707->head;
      kk_std_core_sslice__sslice s = kk_std_core_sslice__sslice_unbox(_box_x302, KK_BORROWED, _ctx);
      kk_std_core_types__list _pat_3 = _con_x707->tail;
      if kk_likely(kk_datatype_ptr_is_unique(_uniq_x_10517, _ctx)) {
        kk_std_core_types__list_drop(_pat_3, _ctx);
        kk_std_core_sslice__sslice_dup(s, _ctx);
        kk_box_drop(_box_x302, _ctx);
        kk_datatype_ptr_free(_uniq_x_10517, _ctx);
      }
      else {
        kk_std_core_sslice__sslice_dup(s, _ctx);
        kk_datatype_ptr_decref(_uniq_x_10517, _ctx);
      }
      _trmc_x10556 = kk_std_core_sslice_string(s, _ctx); /*string*/
    }
    else {
      _trmc_x10556 = kk_string_empty(); /*string*/
    }
    kk_std_core_types__list _trmc_x10557 = kk_datatype_null(); /*list<string>*/;
    kk_std_core_types__list _trmc_x10558 = kk_std_core_types__new_Cons(_ru_x453, 0, kk_string_box(_trmc_x10556), _trmc_x10557, _ctx); /*list<string>*/;
    kk_field_addr_t _b_x312_317 = kk_field_addr_create(&kk_std_core_types__as_Cons(_trmc_x10558, _ctx)->tail, _ctx); /*@field-addr<list<string>>*/;
    kk_std_core_types__cctx _brw_x333;
    kk_std_core_types__cctx _x_x709 = kk_std_core_types__cctx_dup(_acc, _ctx); /*ctx<list<string>>*/
    _brw_x333 = kk_cctx_extend_linear(_x_x709,(kk_std_core_types__list_box(_trmc_x10558, _ctx)),_b_x312_317,kk_context()); /*ctx<0>*/
    kk_std_core_types__list _brw_x334 = kk_std_text_regex__trmc_lift_split_10533(_uniq_xx_10518, _brw_x333, _ctx); /*list<string>*/;
    kk_std_core_types__cctx_drop(_brw_x333, _ctx);
    return _brw_x334;
  }
  {
    kk_box_t _x_x710;
    kk_std_core_types__cctx _x_x711 = kk_std_core_types__cctx_dup(_acc, _ctx); /*ctx<list<string>>*/
    _x_x710 = kk_cctx_apply_linear(_x_x711,(kk_std_core_types__list_box(kk_std_core_types__new_Nil(_ctx), _ctx)),kk_context()); /*-1*/
    return kk_std_core_types__list_unbox(_x_x710, KK_OWNED, _ctx);
  }
}
 
// lifted local: @lift-split@10532, split, @spec-x10520
// specialized: std/core/list/@unroll-map@10010, on parameters @uniq-f@10515, using:
// @uniq-f@10515 = std/text/regex/captured

kk_std_core_types__list kk_std_text_regex__lift_split_10533(kk_std_core_types__list _uniq_xs_10514_0, kk_context_t* _ctx) { /* (list<list<sslice/sslice>>) -> list<string> */ 
  kk_std_core_types__cctx _brw_x331 = kk_cctx_empty(kk_context()); /*ctx<0>*/;
  kk_std_core_types__list _brw_x332 = kk_std_text_regex__trmc_lift_split_10533(_uniq_xs_10514_0, _brw_x331, _ctx); /*list<string>*/;
  kk_std_core_types__cctx_drop(_brw_x331, _ctx);
  return _brw_x332;
}
 
// lifted local: split, @spec-x10513
// specialized: std/core/list/map, on parameters @uniq-f@10510, using:
// @uniq-f@10510 = std/text/regex/captured

kk_std_core_types__list kk_std_text_regex__lift_split_10532(kk_std_core_types__list _uniq_xs_10509, kk_context_t* _ctx) { /* (list<list<sslice/sslice>>) -> list<string> */ 
  if (kk_std_core_types__is_Nil(_uniq_xs_10509, _ctx)) {
    return kk_std_core_types__new_Nil(_ctx);
  }
  {
    kk_std_core_types__cctx _brw_x329 = kk_cctx_empty(kk_context()); /*ctx<0>*/;
    kk_std_core_types__list _brw_x330 = kk_std_text_regex__trmc_lift_split_10533(_uniq_xs_10509, _brw_x329, _ctx); /*list<string>*/;
    kk_std_core_types__cctx_drop(_brw_x329, _ctx);
    return _brw_x330;
  }
}
 
// Split a string `s` in at most `atmost` parts using a regular expression `r` as separator.

kk_std_core_types__list kk_std_text_regex_split(kk_string_t s, kk_std_text_regex__regex r, kk_std_core_types__optional atmost, kk_context_t* _ctx) { /* (s : string, r : regex, atmost : ? int) -> list<string> */ 
  kk_std_core_types__list xs_10586;
  kk_box_t _x_x712;
  {
    kk_box_t _x = r.obj;
    kk_box_dup(_x, _ctx);
    kk_std_text_regex__regex_drop(r, _ctx);
    _x_x712 = _x; /*any*/
  }
  kk_ssize_t _x_x713;
  kk_integer_t _x_x714;
  if (kk_std_core_types__is_Optional(atmost, _ctx)) {
    kk_box_t _box_x323 = atmost._cons._Optional.value;
    kk_integer_t _uniq_atmost_822 = kk_integer_unbox(_box_x323, _ctx);
    kk_integer_dup(_uniq_atmost_822, _ctx);
    kk_std_core_types__optional_drop(atmost, _ctx);
    _x_x714 = _uniq_atmost_822; /*int*/
  }
  else {
    kk_std_core_types__optional_drop(atmost, _ctx);
    _x_x714 = kk_integer_from_small(-1); /*int*/
  }
  _x_x713 = kk_std_core_int_ssize__t(_x_x714, _ctx); /*ssize_t*/
  xs_10586 = kk_std_text_regex_regex_exec_all(_x_x712, s, (KK_IZ(0)), _x_x713, _ctx); /*list<list<sslice/sslice>>*/
  kk_std_core_types__list xs_10055;
  kk_std_core_types__cctx _x_x715 = kk_cctx_empty(kk_context()); /*ctx<0>*/
  xs_10055 = kk_std_text_regex__trmc_filter_non_matches(xs_10586, _x_x715, _ctx); /*list<list<sslice/sslice>>*/
  return kk_std_text_regex__lift_split_10532(xs_10055, _ctx);
}

bool kk_std_text_regex_testabc(kk_string_t s, kk_context_t* _ctx) { /* (s : string) -> bool */ 
  kk_std_text_regex__regex r_10061;
  kk_box_t _x_x716;
  kk_string_t _x_x717;
  kk_define_string_literal(, _s_x718, 6, "[ab]+c", _ctx)
  _x_x717 = kk_string_dup(_s_x718, _ctx); /*string*/
  bool _x_x719;
  kk_std_core_types__optional _match_x328 = kk_std_core_types__new_None(_ctx); /*forall<a> ? a*/;
  if (kk_std_core_types__is_Optional(_match_x328, _ctx)) {
    kk_box_t _box_x324 = _match_x328._cons._Optional.value;
    bool _uniq_ignorecase_161 = kk_bool_unbox(_box_x324);
    kk_std_core_types__optional_drop(_match_x328, _ctx);
    _x_x719 = _uniq_ignorecase_161; /*bool*/
  }
  else {
    kk_std_core_types__optional_drop(_match_x328, _ctx);
    _x_x719 = false; /*bool*/
  }
  bool _x_x720;
  kk_std_core_types__optional _match_x327 = kk_std_core_types__new_None(_ctx); /*forall<a> ? a*/;
  if (kk_std_core_types__is_Optional(_match_x327, _ctx)) {
    kk_box_t _box_x325 = _match_x327._cons._Optional.value;
    bool _uniq_multiline_165 = kk_bool_unbox(_box_x325);
    kk_std_core_types__optional_drop(_match_x327, _ctx);
    _x_x720 = _uniq_multiline_165; /*bool*/
  }
  else {
    kk_std_core_types__optional_drop(_match_x327, _ctx);
    _x_x720 = false; /*bool*/
  }
  _x_x716 = kk_std_text_regex_regex_create(_x_x717, _x_x719, _x_x720, _ctx); /*any*/
  kk_string_t _x_x721;
  kk_define_string_literal(, _s_x722, 6, "[ab]+c", _ctx)
  _x_x721 = kk_string_dup(_s_x722, _ctx); /*string*/
  r_10061 = kk_std_text_regex__new_Regex(_x_x716, _x_x721, _ctx); /*std/text/regex/regex*/
  kk_std_core_types__list list_10065;
  kk_box_t _x_x723;
  {
    kk_box_t _x = r_10061.obj;
    kk_box_dup(_x, _ctx);
    kk_std_text_regex__regex_drop(r_10061, _ctx);
    _x_x723 = _x; /*any*/
  }
  list_10065 = kk_std_text_regex_regex_exec(_x_x723, s, (KK_IZ(0)), _ctx); /*list<sslice/sslice>*/
  if (kk_std_core_types__is_Cons(list_10065, _ctx)) {
    struct kk_std_core_types_Cons* _con_x724 = kk_std_core_types__as_Cons(list_10065, _ctx);
    kk_box_t _box_x326 = _con_x724->head;
    kk_std_core_types__list _pat_1 = _con_x724->tail;
    if kk_likely(kk_datatype_ptr_is_unique(list_10065, _ctx)) {
      kk_std_core_types__list_drop(_pat_1, _ctx);
      kk_box_drop(_box_x326, _ctx);
      kk_datatype_ptr_free(list_10065, _ctx);
    }
    else {
      kk_datatype_ptr_decref(list_10065, _ctx);
    }
    return true;
  }
  {
    return false;
  }
}

// initialization
void kk_std_text_regex__init(kk_context_t* _ctx){
  static bool _kk_initialized = false;
  if (_kk_initialized) return;
  _kk_initialized = true;
  kk_std_core_types__init(_ctx);
  kk_std_core_hnd__init(_ctx);
  kk_std_core_exn__init(_ctx);
  kk_std_core_bool__init(_ctx);
  kk_std_core_order__init(_ctx);
  kk_std_core_char__init(_ctx);
  kk_std_core_int__init(_ctx);
  kk_std_core_vector__init(_ctx);
  kk_std_core_string__init(_ctx);
  kk_std_core_sslice__init(_ctx);
  kk_std_core_list__init(_ctx);
  kk_std_core_maybe__init(_ctx);
  kk_std_core_maybe2__init(_ctx);
  kk_std_core_either__init(_ctx);
  kk_std_core_tuple__init(_ctx);
  kk_std_core_lazy__init(_ctx);
  kk_std_core_show__init(_ctx);
  kk_std_core_debug__init(_ctx);
  kk_std_core_delayed__init(_ctx);
  kk_std_core_console__init(_ctx);
  kk_std_core__init(_ctx);
  kk_std_num_int32__init(_ctx);
  #if defined(KK_CUSTOM_INIT)
    KK_CUSTOM_INIT (_ctx);
  #endif
  {
    kk_box_t _x_x462;
    kk_string_t _x_x463;
    kk_define_string_literal(, _s_x464, 45, "[^\\\\\\[(]+|\\\\[\\s\\S]\?|\\(\\\?|\\[(\?:[^\\\\\\]]|\\\\.)*\\]", _ctx)
    _x_x463 = kk_string_dup(_s_x464, _ctx); /*string*/
    bool _x_x465;
    kk_std_core_types__optional _match_x371 = kk_std_core_types__new_None(_ctx); /*forall<a> ? a*/;
    if (kk_std_core_types__is_Optional(_match_x371, _ctx)) {
      kk_box_t _box_x4 = _match_x371._cons._Optional.value;
      bool _uniq_ignorecase_161 = kk_bool_unbox(_box_x4);
      kk_std_core_types__optional_drop(_match_x371, _ctx);
      _x_x465 = _uniq_ignorecase_161; /*bool*/
    }
    else {
      kk_std_core_types__optional_drop(_match_x371, _ctx);
      _x_x465 = false; /*bool*/
    }
    bool _x_x466;
    kk_std_core_types__optional _match_x370 = kk_std_core_types__new_None(_ctx); /*forall<a> ? a*/;
    if (kk_std_core_types__is_Optional(_match_x370, _ctx)) {
      kk_box_t _box_x5 = _match_x370._cons._Optional.value;
      bool _uniq_multiline_165 = kk_bool_unbox(_box_x5);
      kk_std_core_types__optional_drop(_match_x370, _ctx);
      _x_x466 = _uniq_multiline_165; /*bool*/
    }
    else {
      kk_std_core_types__optional_drop(_match_x370, _ctx);
      _x_x466 = false; /*bool*/
    }
    _x_x462 = kk_std_text_regex_regex_create(_x_x463, _x_x465, _x_x466, _ctx); /*any*/
    kk_string_t _x_x467;
    kk_define_string_literal(, _s_x468, 45, "[^\\\\\\[(]+|\\\\[\\s\\S]\?|\\(\\\?|\\[(\?:[^\\\\\\]]|\\\\.)*\\]", _ctx)
    _x_x467 = kk_string_dup(_s_x468, _ctx); /*string*/
    kk_std_text_regex_rx_nongroup = kk_std_text_regex__new_Regex(_x_x462, _x_x467, _ctx); /*std/text/regex/regex*/
  }
}

// termination
void kk_std_text_regex__done(kk_context_t* _ctx){
  static bool _kk_done = false;
  if (_kk_done) return;
  _kk_done = true;
  #if defined(KK_CUSTOM_DONE)
    KK_CUSTOM_DONE (_ctx);
  #endif
  kk_std_text_regex__regex_drop(kk_std_text_regex_rx_nongroup, _ctx);
  kk_std_num_int32__done(_ctx);
  kk_std_core__done(_ctx);
  kk_std_core_console__done(_ctx);
  kk_std_core_delayed__done(_ctx);
  kk_std_core_debug__done(_ctx);
  kk_std_core_show__done(_ctx);
  kk_std_core_lazy__done(_ctx);
  kk_std_core_tuple__done(_ctx);
  kk_std_core_either__done(_ctx);
  kk_std_core_maybe2__done(_ctx);
  kk_std_core_maybe__done(_ctx);
  kk_std_core_list__done(_ctx);
  kk_std_core_sslice__done(_ctx);
  kk_std_core_string__done(_ctx);
  kk_std_core_vector__done(_ctx);
  kk_std_core_int__done(_ctx);
  kk_std_core_char__done(_ctx);
  kk_std_core_order__done(_ctx);
  kk_std_core_bool__done(_ctx);
  kk_std_core_exn__done(_ctx);
  kk_std_core_hnd__done(_ctx);
  kk_std_core_types__done(_ctx);
}
